#ifndef NEWSTG_H
#define NEWSTG_H

/*
  newstg.h

  Servo To Go board decls for new boards

  Modification history:

  25-Oct-2001 WPS created.
*/

/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif


static char __attribute__((unused)) stg_h[] = "$Id: newstg.h,v 1.5 2004/03/02 21:05:19 paul_c Exp $";

/*
 Number of axes in system defaults to 4. If STG_8_AXES is defined
 by compiler directive or other means, code will support 8 axes,
 e.g., -DSTG_8_AXES. Note that only 6 axes are really supported, since
 there is not enough digital IO for limit and home switches for all 8.

 Analog input is an option, normally not present. If STG_ANALOG_INPUT is
 defined by compiler directive or other means, code will support analog
 input, e.g., -DSTG_ANALOG_INPUT
 */

/*
  Base address defaults to 0x200
  */
#define DEFAULT_IO_BASE_ADDRESS 0x200
extern unsigned long int IO_BASE_ADDRESS;

#define DEFAULT_STG_IRQ 5
extern int STG_IRQ;

/* decls for external interface */

extern int stgMotInit(const char * stuff);
extern int stgMotQuit(void);

extern int stgDacNum(void);
extern int stgDacWrite(int dac, double volts);
extern int stgDacWriteAll(int max, double * volts);

extern int stgEncoderSetIndexModel(unsigned int model);
extern int stgEncoderNum(void);
extern int stgEncoderRead(int encoder, double * counts);
extern int stgEncoderReadAll(int max, double * counts);
extern int stgEncoderResetIndex(int encoder);
extern int stgEncoderReadLatch(int encoder, int * flag);
extern int stgEncoderReadLevel(int encoder, int * flag);

extern int stgMaxLimitSwitchRead(int axis, int * flag);
extern int stgMinLimitSwitchRead(int axis, int * flag);

extern int stgHomeSwitchRead(int axis, int * flag);

extern int stgAmpEnable(int axis, int enable);
extern int stgAmpFault(int axis, int * fault);

extern int stgDioInit(const char * stuff);
extern int stgDioQuit(void);

extern int stgDioMaxInputs(void);
extern int stgDioMaxOutputs(void);
extern int stgDioRead(int index, int *value);
extern int stgDioWrite(int index, int value);
extern int stgDioCheck(int index, int *value);
extern int stgDioByteRead(int index, unsigned char *byte);
extern int stgDioShortRead(int index, unsigned short *sh);
extern int stgDioWordRead(int index, unsigned int *word);
extern int stgDioByteWrite(int index, unsigned char byte);
extern int stgDioShortWrite(int index, unsigned short sh);
extern int stgDioWordWrite(int index, unsigned int word);
extern int stgDioByteCheck(int index, unsigned char *byte);
extern int stgDioShortCheck(int index, unsigned short *sh);
extern int stgDioWordCheck(int index, unsigned int *word);

extern int stgAioInit(const char * stuff);
extern int stgAioQuit(void);

extern int stgAioMaxInputs(void);
extern int stgAioMaxOutputs(void);
extern int stgAioStart(int index);
extern void stgAioWait(void);
extern int stgAioRead(int index, double *volts);
extern int stgAioWrite(int index, double volts);
extern int stgAioCheck(int index, double *volts);
extern char *stgModel(void);

#endif /* NEWSTG_H */
