#ifndef STG_H
#define STG_H

/*
  stg.h

  Servo To Go board decls

  Modification history:

  2-Aug-2001  FMP added stgAdcStart,Wait,Read function decls
  13-Mar-2000 WPS eliminated function declaration isn't a prototype compiler
  warning by putting in the (void) for functions with no arguments.
  28-Oct-1999  FMP added EncoderLatch() function
  20-Oct-1998  FMP fixed decl of RawDAC(), previously RawDac()
  7-Aug-1998  FMP changed stgInit/Quit() to stgMotInit/Quit(), added decls
  for stgDio/AioInit/Quit()
  31-Mar-1998  FMP added analog IO stuff
  26-Mar-1998  FMP added IO_BASE_ADDRESS value
  7-Nov-1997  FMP added stgAmpFault()
  25-Sep-1997  FMP added digital IO stuff
  19-Sep-1997  FMP removed number of axes, base address and put defaults
  in .c file. Now these can be overridden with -DSTG_8_AXES and
  -DIO_BASE_ADDRESS=0x300, for example
  31-Jul-1997  FMP added StartADC, RawADC
  10-Jun-1997  FMP created
*/

/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif


static char __attribute__((unused)) stg_h[] = "$Id: stg.h,v 1.9 2004/03/02 21:05:19 paul_c Exp $";

/*
 Number of axes in system defaults to 4. If STG_8_AXES is defined
 by compiler directive or other means, code will support 8 axes,
 e.g., -DSTG_8_AXES. Note that only 6 axes are really supported, since
 there is not enough digital IO for limit and home switches for all 8.

 Analog input is an option, normally not present. If STG_ANALOG_INPUT is
 defined by compiler directive or other means, code will support analog
 input, e.g., -DSTG_ANALOG_INPUT
 */

/*
  Base address defaults to 0x200
  */
#define DEFAULT_IO_BASE_ADDRESS 0x200
extern unsigned long int IO_BASE_ADDRESS;
extern int FIND_IO_BASE_ADDRESS;


/* decls for external interface */

extern int stgMotInit(const char * stuff);
extern int stgMotQuit(void);

extern int stgAdcNum(void);
extern int stgAdcStart(int adc);
extern void stgAdcWait(void);
extern int stgAdcRead(int adc, double * volts);

extern int stgDacNum(void);
extern int stgDacWrite(int dac, double volts);
extern int stgDacWriteAll(int max, double * volts);

extern int stgEncoderSetIndexModel(unsigned int model);
extern int stgEncoderNum(void);
extern int stgEncoderRead(int encoder, double * counts);
extern int stgEncoderReadAll(int max, double * counts);
extern int stgEncoderResetIndex(int encoder);
extern int stgEncoderReadLatch(int encoder, int * flag);
extern int stgEncoderReadLevel(int encoder, int * flag);

extern int stgMaxLimitSwitchRead(int axis, int * flag);
extern int stgMinLimitSwitchRead(int axis, int * flag);

extern int stgHomeSwitchRead(int axis, int * flag);

extern int stgAmpEnable(int axis, int enable);
extern int stgAmpFault(int axis, int * fault);

extern int stgDioInit(const char * stuff);
extern int stgDioQuit(void);

extern int stgDioMaxInputs(void);
extern int stgDioMaxOutputs(void);
extern int stgDioRead(int index, int *value);
extern int stgDioWrite(int index, int value);
extern int stgDioCheck(int index, int *value);
extern int stgDioByteRead(int index, unsigned char *byte);
extern int stgDioShortRead(int index, unsigned short *sh);
extern int stgDioWordRead(int index, unsigned int *word);
extern int stgDioByteWrite(int index, unsigned char byte);
extern int stgDioShortWrite(int index, unsigned short sh);
extern int stgDioWordWrite(int index, unsigned int word);
extern int stgDioByteCheck(int index, unsigned char *byte);
extern int stgDioShortCheck(int index, unsigned short *sh);
extern int stgDioWordCheck(int index, unsigned int *word);

extern int stgAioInit(const char * stuff);
extern int stgAioQuit(void);

extern int stgAioMaxInputs(void);
extern int stgAioMaxOutputs(void);
extern int stgAioStart(int index);
extern void stgAioWait(void);
extern int stgAioRead(int index, double *volts);
extern int stgAioWrite(int index, double volts);
extern int stgAioCheck(int index, double *volts);
extern char *stgModel(void);

typedef union
{
  unsigned short                          Word;
  struct   {unsigned char    high, low;}  Byte;
} WORDBYTE;

typedef union
{
  long           Long;
  unsigned char  Byte[4];
} LONGBYTE;

typedef union
{
  unsigned long               all;
  struct {unsigned char  A, B, C, D;} port;
} IO32;

extern unsigned short BaseFind(void);
extern void SetIrq(short nRequestedIrq);
extern short int Initialize(short int nReqInt);
extern int StartADC(unsigned short wAxis);
extern short RawADC(unsigned short wAxis);
extern void EncoderInit(void);
extern void SelectInterruptPeriod(long);
extern void ResetIndexLatch(void);
extern void SelectIndexAxis(unsigned char byAxis, unsigned char byPol);
extern unsigned char CurrentIRR(void);
extern int IndexPulseLatch(void);
extern int RawDAC(short nAxis, double volts);
extern int EncoderLatch(void);
extern int EncReadAll(LONGBYTE * lbEnc);
extern long RawDI(void);
extern void RawDO(unsigned char byBitNumber, unsigned char bySet0or1, unsigned short nPort);
extern void RawDOAll(IO32 xOutBits);
extern void DIODirection(unsigned short nSwDir);

#define CNT0_D     0x00
#define CNT1_D     0x01
#define CNT0_C     0x02
#define CNT1_C     0x03
#define CNT2_D     0x04
#define CNT3_D     0x05
#define CNT2_C     0x06
#define CNT3_C     0x07
#define CNT4_D     0x08
#define CNT5_D     0x09
#define CNT4_C     0x0a
#define CNT5_C     0x0b
#define CNT6_D     0x0c
#define CNT7_D     0x0d
#define CNT6_C     0x0e
#define CNT7_C     0x0f
#define DAC_0      0x10
#define DAC_1      0x12
#define DAC_2      0x14
#define DAC_3      0x16
#define DAC_4      0x18
#define DAC_5      0x1a
#define DAC_6      0x1c
#define DAC_7      0x1e
#define ADC_0      0x410
#define ADC_1      0x412
#define ADC_2      0x414
#define ADC_3      0x416
#define ADC_4      0x418
#define ADC_5      0x41a
#define ADC_6      0x41c
#define ADC_7      0x41e
#define DIO_A      0x400
#define DIO_B      0x402
#define BRDTST     0x403
#define DIO_C      0x404
#define MIO_1      0x406
#define DIO_D      0x401
#define INTC       0x405
#define MIO_2      0x407
#define ODDRST     0x407
#define TIMER_0    0x408
#define TIMER_1    0x40a
#define TIMER_2    0x40c
#define TMRCMD     0x40e
#define ICW1       0x409
#define ICW2       0x40b
#define OCW1       0x40b
#define OCW2       0x409
#define OCW3       0x409
#define IRR        0x409
#define ISR        0x409
#define IMR        0x40b

#define IXEVN      0x80
#define IXODD      0x40
#define LIXEVN     0x20
#define LIXODD     0x10
#define EOC        0x08
#define TP0        0x04
#define USR_INT    0x02
#define TP2        0x01

#define AUTOZERO   0x80
#define IXLVL      0x40
#define IXS1       0x20
#define IXS0       0x10
#define USRINT     0x08
#define IA2        0x04
#define IA1        0x02
#define IA0        0x01

#define  IRQSL   0x84

#define A_OUT      0x10
#define B_OUT      0x02
#define C_LOW_OUT  0x01
#define C_HI_OUT   0x08
#define D_OUT      0x04

#define _500_MICROSECONDS        500
#define _1_MILLISECOND           1000
#define _2_MILLISECONDS          2000
#define _3_MILLISECONDS          3000
#define _4_MILLISECONDS          4000
#define _5_MILLISECONDS          5000
#define _10_MILLISECONDS         10000
#define _100_MILLISECONDS        100000L
#define _1_SECOND                1000000L
#define MAX_PERIOD               -1L

#define NO_BOARD       1
#define BOARD_PRESENT  0



#endif
