/*
 * Servo To Go controller
 *
 * Copyright (c) 2000 Servo To Go Inc.
 * 8117 Groton Ln.
 * Indianapolis, IN 46260
 *
 * Version 0.1 November 1995
 *
 * MGS added declaration of TimerOff function.
 */

#ifndef _STGDEFS_H_
#define _STGDEFS_H_

#ifndef BOOL
#define BOOL int
#endif

// hardware direction bit definitions
#define A_DIR_BIT      0x10
#define B_DIR_BIT      0x02
#define C_LOW_DIR_BIT  0x01
#define C_HI_DIR_BIT   0x08
#define D_DIR_BIT      0x10
#define D_LOW_DIR_BIT  0x01
#define D_HI_DIR_BIT   0x08

/* parameters to the SelectInterruptPeriod Command */
#define _500_MICROSECONDS        500
#define _1_MILLISECOND           1000
#define _2_MILLISECONDS          2000
#define _3_MILLISECONDS          3000
#define _4_MILLISECONDS          4000
#define _5_MILLISECONDS          5000
#define _10_MILLISECONDS         10000
#define _100_MILLISECONDS        100000L
#define _1_SECOND                1000000L
#define MAX_PERIOD               -1L

#define NO_BOARD       1
#define BOARD_PRESENT  0

#define MODEL_NO_ID    0
#define MODEL1         1
#define MODEL2         2

typedef union
{
    unsigned long               all;
    struct {unsigned char  A, B, C, D;} port;
} IO32;

typedef union
{
    long           Long;
    unsigned char  Byte[4];
} LONGBYTE;

typedef union
{
    unsigned short                          Word;
    struct   {unsigned char    high, low;}  Byte;
} WORDBYTE;

void ServoToGoDestructor(void);
void ServoToGoConstructor(unsigned short wRequestedIrq);
void MotSim(void);
void           Stg_Init(unsigned short wRequestedIrq, unsigned short wAdd);
unsigned short BaseAddress(void);
void           RawDAC(unsigned short nAxis, long lCounts);
void           StartADC(unsigned short wAxis);
short          SpinReadADC(unsigned short wAxis);
long           ReadADC(unsigned short wAxis, short *counts);
void           AutoZeroAdc(void);
void           DontAutoZeroAdc(void);
void           CalADC(void);
void           EncoderInit(void);
void           EncoderLatch(void);
void           EncoderResetAddr(void);
void           SetEncoderCounts(unsigned short nAxis, long lCounts);
void           SelectIndexAxisWithPolarity(unsigned char byAxis, unsigned char byPol);
void           SelectIndexAxis(unsigned char byAxis);
void           SelectIndexOrExtLatch(unsigned char bySelectBits);
void           EnableCounterLatchOnIndexOrExt(unsigned char bySelectBits);
short          IndexPulse(void);
unsigned short IndexPulseLatch(void);
void           ResetIndexLatches(unsigned char byLatchBits);
unsigned char  CurrentIRR(void);
void           ResetIndexLatch(void);
unsigned char  GetIndexLatches(void);
unsigned char  GetSELDI(void);
unsigned char  GetIDLEN(void);
unsigned char  GetCNTRL0(void);
unsigned char  GetCNTRL1(void);
unsigned char  GetBRDTST(void);
void           ResetWatchdogLatch(void);
void           EncReadAll(LONGBYTE *lbEnc);
unsigned long  RawDIAll(void);
unsigned char  RawDIBitPort(unsigned char byBitNumber, short nPort);
unsigned char  RawDIPort(short nPort);
//    unsigned long  RawDIPortBit(long lPort, long lBitNumber);
void           RawDOAll(unsigned long lOutBits);
    void           RawDOBitValPort(unsigned char byBitNumber, unsigned char bySet0or1, unsigned short nPort);
void           RawDOBytePort(unsigned char byData, short nPort);
void           RawDOIO32(IO32 xOutBits);
void           DigitalOut1(unsigned char byData, short nPortBits);
void           DigitalOut2(unsigned char byBitNumber, unsigned char bySet0or1,
			   unsigned short nPortBits);
void           DioDirection1(unsigned short nPort, unsigned short nDirection);
void           DioDirection2(const unsigned short nSwDir);
void           StopTimer(void);
void           TimerOff(void);
void           StartInterrupts(void);
void           IrqReset(void);
void           SelectInterruptPeriod(long lPeriodSelect);
void           StopInterrupts(void);
unsigned short GetBoardPresence(void);
unsigned short GetAxes(void);
unsigned short GetIrq(void);
unsigned short GetAddr(void);
void           SetAddr(unsigned short add);
unsigned short GetModel(void);

void           Timer2Delay(unsigned short counts);
void           StartTimer2TerminalCount(unsigned short count);
void           StartTimer2RTI(unsigned short count);
unsigned short ReadTimer2TerminalCount(void);
short          PollTimer2(void);
void           MaskTimer2Interrupt(void);
void           UnMaskTimer2Interrupt(void);
BOOL           AddressInRange(long);
void                  Initialize(unsigned short wRequestedIrq);
void                  SetIrq(unsigned short wRequestedIrq);
unsigned short        BaseFind(void);
unsigned short        BrdtstOK(unsigned short wBaseAddress);
void                  SetDDir(unsigned short nSwDir);

#endif   // _STGDEFS_H_
