/*
  dro.c

  Modification history:

  12-Jul-2000 WPS put in stupid stuff see FIXME FIXME FIXME.
  15-Jun-2000 WPS changed include asm/io.h to sys/io.h for rtlinux_2_2.
  12-Apr-2000 WPS forced this file to compile on a SUN.
  13-Jan-2000 WPS updated for rtlinux2 :  see etime(),
*/

#ifdef realtime
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <asm/io.h>             /* outb, inb */
int DRO_BASE_ADDRESS;
module_param(DRO_BASE_ADDRESS, int, S_IRUGO);
MODULE_PARM_DESC(DRO_BASE_ADDRESS, "Base address for Kaluga DRO card");
MODULE_VERSION("$Id: dro.c,v 1.1.1.1 2005/05/31 11:12:19 paul Exp $");
#else
#include <stdio.h>              /* printf(), fflush() */
#include <stdlib.h>             /* exit() */
#include <unistd.h>             /* usleep() */
#include <sys/io.h>
#endif


#include "dro.h"                /* these decls */
//#include "emcmot.h"
#include "diagnostics.h"


#define CLOCK_DATA              0x0E
#define CLOCK_SETUP             0x98
#define INPUT_SETUP             0xC1
#define QUAD_X1                 0xA8
#define QUAD_X2                 0xB0
#define QUAD_X4                 0xB8
#define BP_RESET                0x01
#define BP_RESETB               0x81
#define CNTR_RESET              0x02
#define CNTR_RESETB             0x82
#define TRSFRPR_CNTR            0x08
#define TRSFRCNTR_OL            0x90
#define EFLAG_RESET             0x86
#define INDEX_DISABLE           0xE0

static int Present[4] = {1, 1, 1, 1};

#ifdef MAIN
int DRO_BASE_ADDRESS = 0x200;
#endif


#define DATA(x) (DRO_BASE_ADDRESS + (2 * x))
#define CTRL(x) (DRO_BASE_ADDRESS + (2 * x) + 1)

/*
   LS7266Init()

   Returns char, masked with axes present
*/
int LS7266Init(void)
{
  int t;
  
//  diagnostics("LS7266Init: DRO_BASE_ADDRESS = 0x%x\n", DRO_BASE_ADDRESS);

  for (t = 0; t < 4; t++) {
    /* set present flag to true, clear it later if not there */
    Present[t] = 1;
    /* if even, initialize chip half */
    if (! (t % 2)) {
      outb(INDEX_DISABLE, CTRL(t));  
      outb(EFLAG_RESET, CTRL(t));
      outb(BP_RESETB, CTRL(t));
      outb(CLOCK_DATA, DATA(t));
      outb(CLOCK_SETUP, CTRL(t));
      outb(INPUT_SETUP, CTRL(t));
      outb(QUAD_X4, CTRL(t));
      outb(CNTR_RESETB, CTRL(t));

      outb(184, CTRL(t));
      outb(135, CTRL(t));
      outb(133, CTRL(t));
      outb(131, CTRL(t));
      outb(193, CTRL(t));
      outb(224, CTRL(t));
    }

    /* initialize encoder input */
    outb(BP_RESET, CTRL(t));
    outb(3 * t + 1, DATA(t));
    outb(3 * t + 2, DATA(t));
    outb(3 * t + 3, DATA(t));

    outb(TRSFRPR_CNTR, CTRL(t));
    outb(TRSFRCNTR_OL, CTRL(t));
    outb(CNTR_RESET, CTRL(t));

    outb(BP_RESET, CTRL(t));
    if (inb(DATA(t)) != 3 * t + 1) {
      Present[t] = 0;
    }
    if (inb(DATA(t)) != 3 * t + 2) {
      Present[t] = 0;
    }
    if (inb(DATA(t)) != 3 * t + 3) {
      Present[t] = 0;
    }
  }

  if (Present[0] || Present[1] || Present[2] || Present[3]) {
    return 0;
  }
  return -1;
}

/*
  LS7266Read()
*/
long LS7266Read(int i)
{
  union pos_tag {
    long l;
    struct byte_tag { char b0; char b1; char b2; char b3;} byte;
  } pos;

  outb(TRSFRCNTR_OL, CTRL(i));
  outb(BP_RESET, CTRL(i));
  pos.byte.b0=inb(DATA(i));
  pos.byte.b1=inb(DATA(i));
  pos.byte.b2=inb(DATA(i));
  if (pos.byte.b2 < 0) {
    pos.byte.b3 = -1;
  }
  else {
    pos.byte.b3 = 0;
  }

  return pos.l;
}

/*
  LS7266Write()
*/
void LS7266Write(int i, long in)
{
  union pos_tag {
    long l;
    struct byte_tag { char b0; char b1; char b2; char b3;} byte;
  } pos;

  pos.l = in;
  outb(BP_RESET, CTRL(i));
  outb(pos.byte.b0, DATA(i));
  outb(pos.byte.b1, DATA(i));
  outb(pos.byte.b2, DATA(i));
  outb(TRSFRPR_CNTR, CTRL(i));
  outb(TRSFRCNTR_OL, CTRL(i));
  outb(BP_RESET, CTRL(i));

  return;
}

#ifdef MAIN

int main(int argc, char *argv[])
{
  int t;

  if (argc == 1) {
    /* no args, so leave BASE7266 as is */
  }
  else if (argc == 2) {
    if (1 != sscanf(argv[1], "%i", &DRO_BASE_ADDRESS)) {
      fprintf(stderr, "invalid board address: %s\n", argv[1]);
      exit(1);
    }
  }
  else {
    fprintf(stderr, "syntax: %s <address>\n", argv[0]);
    exit(1);
  }

  if (0 != iopl(3)) {
    fprintf(stderr, "can't access 7266 chip-- need to be root\n");
    exit(1);
  }

  if (0 != LS7266Init()) {
    fprintf(stderr, "no card detected\n");
    exit(1);
  }

  printf("board detected at 0x%X\n", DRO_BASE_ADDRESS);
  printf("axes %c%c%c%c\n",
         Present[0] ? 'X' : '-',
         Present[1] ? 'Y' : '-',
         Present[2] ? 'Z' : '-',
         Present[3] ? 'W' : '-');

  while (1) {
    for (t = 0; t < 4; t++) {
      if (Present[t]) {
        printf("%10ld\t", LS7266Read(t));
      }
      else {
        printf("-\t");
      }
    }
    printf("\r");
    fflush(stdout);
    usleep(10000);
  }

  return 0;
}

#endif /* MAIN */
