#ifndef PPMC_H
#define PPMC_H

#if defined (realtime)
#ifndef MODULE
#define MODULE
#endif
#ifndef __KERNEL__
#define __KERNEL__
#endif

# include <linux/version.h>
# include <linux/module.h>
# include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/vmalloc.h>

#endif

/*
  ppmc.h

  Parallel Port Motion Control decls

  Modification history:

  17-Sep-2000 JME started conversion from STG.c
  
*/

/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x) 
#endif
#endif

static char __attribute__((unused)) ppmc_h[] = "$Id: ppmc.h,v 1.1.1.1 2005/05/31 11:12:19 paul Exp $";

/* 
 Number of axes in system defaults to 4. If PPMC_8_AXES is defined
 by compiler directive or other means, code will support 8 axes,
 e.g., -DPPMC_8_AXES.

 Analog input is an option, normally not present. If STG_ANALOG_INPUT is
 defined by compiler directive or other means, code will support analog
 input, e.g., -DSTG_ANALOG_INPUT
 */
#ifdef PPMC_8_AXES
#define PPMC_MAX_AXIS 8
#else
#define PPMC_MAX_AXIS 4
#endif
/*
  Base address defaults to 0x378 i.e. lpt1
  */
#define DEFAULT_IO_BASE_ADDRESS 0x378
extern int IO_BASE_ADDRESS;
#define ppmc_base_addr IO_BASE_ADDRESS

extern char *ppmcModel(void);

extern int ppmcMotInit(const char * stuff);
extern int ppmcMotQuit(void);

extern int ppmcDacNum(void);
extern int ppmcDacWrite(int dac, double volts);
extern int ppmcDacWriteAll(int max, double * volts);

extern int ppmcEncoderSetIndexModel(unsigned int model);
extern int ppmcEncoderNum(void);
extern int ppmcEncoderRead(int encoder, double * counts);
extern int ppmcEncoderReadAll(int max, double * counts);
extern int ppmcEncoderResetIndex(int encoder);
extern int ppmcEncoderReadLatch(int encoder, int * flag);
extern int ppmcEncoderReadLevel(int encoder, int * flag);

extern int ppmcMaxLimitSwitchRead(int axis, int * flag);
extern int ppmcMinLimitSwitchRead(int axis, int * flag);

extern int ppmcHomeSwitchRead(int axis, int * flag);

extern int ppmcAmpEnable(int axis, int enable);
extern int ppmcAmpFault(int axis, int * fault);

extern int ppmcDioInit(const char * stuff);
extern int ppmcDioQuit(void);

extern int ppmcDioMaxInputs(void);
extern int ppmcDioMaxOutputs(void);
extern int ppmcDioRead(int index, int *value);
extern int ppmcDioWrite(int index, int value);
extern int ppmcDioCheck(int index, int *value);
extern int ppmcDioByteRead(int index, unsigned char *byte);
extern int ppmcDioByteWrite(int index, unsigned char byte);
extern int ppmcDioByteCheck(int index, unsigned char *byte);

extern int ppmcAioInit(const char * stuff);
extern int ppmcAioQuit(void);

extern int ppmcAioMaxInputs(void);
extern int ppmcAioMaxOutputs(void);
extern int ppmcAioRead(int index, double *volts);
extern int ppmcAioWrite(int index, double volts);
extern int ppmcAioCheck(int index, double *volts);

typedef union
{
  unsigned short                          Word;
  struct   {unsigned char    high, low;}  Byte;
} WORDBYTE;

typedef union
{  
  long           Long;
  unsigned char  Byte[4];
} LONGBYTE;

#define ENCCNT0    0x00          /*  EPP address to read first byte of encoder counter */
#define ENCCNT1    0x03
#define ENCCNT2    0x06
#define ENCCNT3    0x09

#define ENCCTRL_0  0x03          /* EPP address of encoder control register */
#define ENCRATE_0  0x04          /* interrupt rate register, only on master encoder */
#define ENCISR_0   0x0C		//index sense register
#define ENCLOAD_0  0x00          /* EPP address to write into first byte of preset register
				        for channels 0 - 3 */
#define ENCCNT4    0x20          /* EPP addr of first chan of 2nd encoder board */
#define ENCCNT5    0x23
#define ENCCNT6    0x26
#define ENCCNT7    0x29

#define ENCCTRL_1  0x23          /* EPP address of encoder control register for board 2 */
#define ENCRATE_1  0x24
#define ENCISR_1   0x2C		//index sense register
#define ENCLOAD_1  0x20

#define DAC_0      0x40          /* EPP address of low byte of DAC 0 */
#define DAC_1      0x42
#define DAC_2      0x44
#define DAC_3      0x46

#define DAC_4      0x50          /* EPP addr of first chan of DAC board 2 */
#define DAC_5      0x52
#define DAC_6      0x54
#define DAC_7      0x56

#define DAC_MODE_0 0x4C
#define DAC_WRITE_0 0x48

#define DAC_MODE_1 0x5C
#define DAC_WRITE_1 0x58

#define RATE_GEN_0      0x10          /* EPP addr of low byte of rate generator 0 LSB */
//      RATE_GEN_0a     0x11       /* middle byte */
//      RATE_GEN_0b     0x12       /* MSB */
#define RATE_GEN_1      0x13
#define RATE_GEN_2      0x16
#define RATE_GEN_3      0x19
#define RATE_CTRL_0     0x1C
#define RATE_SETUP_0    0x1D
#define RATE_WIDTH_0    0x1E

#define RATE_GEN_4      0x30          /* EPP addr of low byte of rate generator 0 */
#define RATE_GEN_5      0x33
#define RATE_GEN_6      0x36
#define RATE_GEN_7      0x39
#define RATE_CTRL_4     0x3C
#define RATE_SETUP_4    0x3D
#define RATE_WIDTH_4    0x3E

#ifdef UNIV_STEP
#define DINA_0      0x0D          /* EPP address of digital inputs on univstep */
#define DINB_0      0x0E
#define ESTOP_IN     0x0E

#define DOUTA_0     0x1F          /* EPP address of digital outputs */
#define DOUTB_0     0x1F          /* doesn't exist */
#define ESTOP_OUT   0x1F

#else
#define DINA_0      0x60          /* EPP address of digital inputs on DIO */
#define DINB_0      0x61
#define ESTOP_0     0x62

#define DOUTA_0     0x60          /* EPP address of digital outputs */
#define DOUTB_0     0x61
#endif

#define NO_BOARD       1
#define BOARD_PRESENT  0
    
typedef union
{ 
  unsigned long               all;
  struct {unsigned char  A, B, C, D;} port;
} IO32;
    
//we don't have dio yet, so can't home or limit
/* Uncomment any of the following declarations to inhibit
   the use of home switches or index pulses when homing, or to
   ignore limit switches if you don't have them. */

/* Uncomment NO_HOME_SWITCH if you want homing sequence to
   ignore the home switch, and only use the index pulse for homing.
   This has the effect of making the home switches always appear tripped. */
// #define NO_HOME_SWITCH

/* Uncomment NO_INDEX_PULSE if you want homing sequence to
   ignore the index pulse, and only use the home switch for homing.
   This has the effect of making the index pulse always appear present. */
/* #define NO_INDEX_PULSE */

/* Uncomment NO_LIMIT_SWITCH if you don't have limit switches.
   This has the effect of making the positive and negative travel
   limit switches never appear tripped. */
// #define NO_LIMIT_SWITCH 

#define _500_MICROSECONDS        500
#define _1_MILLISECOND           1000
#define _2_MILLISECONDS          2000
#define _3_MILLISECONDS          3000
#define _4_MILLISECONDS          4000
#define _5_MILLISECONDS          5000
#define _10_MILLISECONDS         10000
#define _100_MILLISECONDS        100000L
#define _1_SECOND                1000000L
#define MAX_PERIOD               -1L

void SelAddr(unsigned int port);
void WriteData(unsigned char byte);
void WriteData16(unsigned short word);
unsigned short ReadData(void);
unsigned short SelRead(unsigned int port);
void SelWrt(unsigned int port, unsigned char byte);
void SelWrt16(unsigned int port, unsigned short word);

short DACQuit(void);
short DACInit(void);
int DIOQuit(void);
int DIOInit(void);
int EncoderQuit(void);
int EncoderInit(void);

#endif
