//ppmc dac functions

#include "ppmc.h"
//  fix a bunch of problems in 8-axis support.

/* diagnostics printing */
#include "diagnostics.h"

char *ppmcModel(void)
{
    return "Parallel port interface for Servos";
}


/*
VoltToCount(double volts)  converts a voltage to a
number of counts to output to the dac

  output to DAC word works like:

  0xFFFF -> +10 V
  0x8000 ->   0 V
  0x0000 -> -10 V
  */


__inline unsigned short VoltToCounts(double volts)
{
	long nCounts;
	nCounts = (long) (((volts / 10.0) * 0x7FFF)+0x8000);

	if (nCounts > 0xffff)
    {
      nCounts = 0xffff;
    }
	if (nCounts < 0 )
    {
      nCounts = 0;
    }
	return (unsigned short)nCounts;
}

int RawDAC(int nAxis, double volts)
{
  

  if ( (nAxis > PPMC_MAX_AXIS) || (nAxis < 0) )        /* is axis within range? */
    return -1;

  /* convert -10.0 -> 0,0 to 0x8000 10 -> 0xFFFF */
  
  if (nAxis <=3)
   {
    SelWrt16(DAC_0 + (nAxis << 1), VoltToCounts(volts));
	SelWrt(DAC_WRITE_0, 0);
   }
  else {
    SelWrt16(DAC_4 + ((nAxis-4) << 1), VoltToCounts(volts));
	SelWrt(DAC_WRITE_1, 0);
  }
	
	
  return 0;
}

short DACInit(void)
{
  int i,j;
  j=DAC_0+15;
  i = SelRead(j);
  if ((i & 0xf0) != 0x20) {
    diagnostics("ppmc_dac could not find a DAC module at IEEE-1284 address 0x%2x\n",j); 
    return -1;              /* blow it up ! */
  }
else
    diagnostics("ppmc_dac found a DAC ver %d module at IEEE-1284 address 0x%2x\n",i&0x0f,j);

       	SelWrt(DAC_WRITE_0,0);    // reset strobe
       	SelWrt(DAC_MODE_0,0);     // sel strobe to INTR/ line
#ifdef PPMC_8_AXIS
       	SelWrt(DAC_WRITE_1,0);    // reset strobe
       	SelWrt(DAC_MODE_1,0);     // sel strobe to INTR/ line
#endif
	return 0;
}

short DACQuit(void)
{
	   return DACInit();  //init sets all outputs to zero
}
int ppmcDacNum(void)
{
	return PPMC_MAX_AXIS;
}

int ppmcDacWrite(int dac, double volts)
{
  return RawDAC(dac, volts);
}

int ppmcDacWriteAll(int max, double * volts)
{
	int t;
  int smax;
 
  /* clip smax to max supported-- if they want more, ignore */
  if (max > PPMC_MAX_AXIS) {
    smax = PPMC_MAX_AXIS;
  }
  else {
    smax = max;
  }
#ifndef PPMC_8_AXES
  //take advantage of auto increment of address pointer in ppmc
	SelAddr(DAC_0);
	for (t = 0; t < smax; t++)
	{
  /*	  printf(" dac channel %d volts %12.4f   digital # %d\n",
	  t,volts[t],VoltToCounts(volts[t]));  */
		WriteData16(VoltToCounts(volts[t]));
    }
	/*	printf("\n");  */
#else
	SelAddr(DAC_0);
	if(smax<=4)
	{
	   for (t = 0; t < smax; t++)
	     {
	        WriteData16(VoltToCounts(volts[t]));
	     }
	   return 0;
	}
	else
	{
	  for (t = 0; t < 4; t++)
	    {
	      WriteData16(VoltToCounts(volts[t]));
 	    }
	  SelAddr(DAC_4);
	  for (t = 4; t < smax; t++)
	    {
		WriteData16(VoltToCounts(volts[t]));
	    }
	  SelWrt(DAC_WRITE_1, 0);  // force data out now
    }
#endif
  return 0;
}

























