//ppmc pwm generator functions
// Jon Elson 6/20/2004 connect emcmotConfig->setup_time to set PWM frequency
//  tricky because the pointer is not available when DACInit is first called!

// Jon Elson 12/15/2003 don't try to talk to non-existant modules

/* Mod Jon Elson 7-12-2003 to allow mixing of calls to ppmcDacWrite and ppmcDacWriteAll
   without stepping on each other's toes, especially in relation to the control
   registers, which are shared between 4 axes each. */
//   it turns out the above hackery is not used by EMC

#include <linux/types.h>
#ifdef __attribute_used__
#undef __attribute_used__
#endif
#ifdef __attribute_pure__
#undef __attribute_pure__
#endif
#include <sys/cdefs.h>
#include <float.h>
#include <math.h>
#include "emcmot.h"
#include "ppmc.h"

#if defined (realtime)
MODULE_VERSION("$Id: ppmc_pwm.c,v 1.3 2005/09/10 21:33:01 paul Exp $");
#endif

#define TRUE 1
#define FALSE 0


#define RateClock 10000000.0 /* oscillator, 10 MHz */
// define PWMCycle 200     // 10 MHz / 200 = 50 KHz

/* diagnostics printing */
#include "diagnostics.h"

/* need to point to emcmotIo and emcmotConfig structures */
extern EMCMOT_IO *emcmotIo;
extern EMCMOT_CONFIG *emcmotConfig;

static int CtrlByte[2];
static int estop;
static long PWMCycle = 200;  // number of clock cycles (10 MHz) in PWM wave
static int inited = TRUE;

char *ppmcModel(void)
{
    return "Parallel port interface for Universal PWM servos";
}

  // state machine estop :  1 = in E Stop
  //                        2 = just out of estop one servo cycle
  //                        3 = output + pulse
  //                        4 = output - pulse
  //                        5 = normal

/* VoltToCount(double volts)  converts a commanded 'voltage' to a
value to program the PWM pulse width  */
// current formula 10.0 Volts in means 95% PWM out, which is the max
__inline long VoltToCounts(double volts)
{
  long Count;
  int i;

  if (inited) {
    PWMCycle = RateClock/emcmotConfig->setup_time[0];
    diagnostics("PWMCycle = %ld\n",PWMCycle);
    i = 65535 - PWMCycle;
    SelWrt(RATE_CTRL_0,0);    // turn off all axes
    SelWrt(0x1d,(i & 0xff));    // send low byte of limit count
    WriteData(i>>8 & 0xff);           // and high byte of limit count
    inited = FALSE;
  }
  Count = fabs (volts) * PWMCycle / 10.0;
  if (Count <  5 * PWMCycle/100) Count = 5 * PWMCycle/100;
  if (Count > 95 * PWMCycle/100) Count = 95 * PWMCycle/100;
  Count = 65535 - Count;
  return (long)Count;
}

int RawDAC(int nAxis, double volts)
{
  long count;
  int CBMask,dacval;
  
  if (nAxis == 3 )        /* special for spindle DAC */
    {
      dacval = fabs(volts)*25.5;
      dacval = 123;
      SelWrt(ENCCNT0 + 15,dacval & 0xff);
      return 0;
    }
  if ( (nAxis > PPMC_MAX_AXIS) || (nAxis < 0) )        /* is axis within range? */
    return -1;

  if (nAxis <=3)
   {
     SelAddr(RATE_GEN_0 + nAxis * 2);
     count = VoltToCounts(volts);
     WriteData(count & 0xff);
     WriteData(count>>8 & 0xff);
     CBMask = 1 << (nAxis * 2);
     CtrlByte[0] &= ~(CBMask * 3);    // clear mask bits for this axis only
     if (fabs (volts) > .001) CtrlByte[0]+=CBMask * 2;
     if (volts < 0.0) CtrlByte[0]+=CBMask;
     SelWrt(RATE_CTRL_0,CtrlByte[0] & 0xff);
    }
  else {
     SelAddr(RATE_GEN_4 + (nAxis-4) * 2);
     count = VoltToCounts(volts);
     WriteData(count & 0xff);
     WriteData(count>>8 & 0xff);
     CBMask = 1 << ((nAxis-4) * 2);
     CtrlByte[1] &= ~(CBMask * 3);    // clear mask bits for this axis only
     if (fabs (volts) > .001) CtrlByte[1]+=CBMask * 2;
     if (volts < 0.0) CtrlByte[1]+=CBMask;
     SelWrt(RATE_CTRL_4,CtrlByte[1] & 0xff);
    }

	
	
  return 0;
}

short DACInit(void)
{
  int i,j;
  CtrlByte[0] = 0;
  CtrlByte[1] = 0;
  j=ENCCNT0+15;
  i = SelRead(j);
  if ((i & 0xf0) != 0x50) {
    diagnostics("ppmc_PWM V1.1 could not find a Universal PWM module at IEEE-1284 address 0x%2x\n",j); 
    return -1;              /* blow it up ! */
  }
else
    diagnostics("ppmc_PWM V1.1 found a Universal PWM ver %d module at IEEE-1284 address 0x%2x\n",i&0x0f,j);

  // talk to 2nd module first, in case it doesn't exist
  //       	SelWrt(RATE_CTRL_4,0);    // turn off all axes
  //	WriteData(155);           // Setup delay = 10.0 uS
  //	WriteData(245);           // Step Width = 1.5 uS    // JE 12-10-2003
  inited = TRUE;
  PWMCycle = RateClock/200;
  i = 65535 - PWMCycle;
  SelWrt(RATE_CTRL_0,0);    // turn off all axes
  SelWrt(0x1d,(i & 0xff));    // send low byte of limit count
  WriteData(i>>8 & 0xff);           // and high byte of limit count
  return 0;
}

short DACQuit(void)
{
	   return DACInit();  //init sets all outputs to zero
}
int ppmcDacNum(void)
{
	return PPMC_MAX_AXIS;
}

int ppmcDacWrite(int dac, double volts)
{
  return RawDAC(dac, volts);
}

int ppmcDacWriteAll(int max, double * volts)
{
  long count;
  int t,CBMask;
  int smax,tmax;

 
  /* clip smax to max supported-- if they want more, ignore */
  if (max > PPMC_MAX_AXIS) {
    smax = PPMC_MAX_AXIS;
  }
  else {
    smax = max;
  }
#ifndef PPMC_8_AXIS
  if ((emcmotIo->OutBytes[0] & 0x80) == 0) {
    if (estop != 1) diagnostics("going into E-stop\n");
    estop = 1;
  }
  if ((emcmotIo->OutBytes[0] & 0x80) != 0) {
    if ((estop >=1) && (estop <= 4)) {
      if (estop == 1) inited = TRUE;  // force unpdate of PWM rate
      estop++;
      diagnostics("ppmc_PWM V1.0 estop is now %d\n",estop); 
    }
  }
  /*  counter++;
  if (counter == 1000) {
    counter=0;
    counter1++;
    diagnostics("PWM %d, estop = %d      emcmotIo->OutBytes[0,1,2] = %d,%d,%d\n",counter1,estop,
		emcmotIo->OutBytes[0],
		emcmotIo->OutBytes[1],
		emcmotIo->OutBytes[2]);
		} */
  
  //take advantage of auto increment of address pointer in ppmc

  SelAddr(RATE_GEN_0);
  CBMask = 1;
  //	for (t = 0; t < smax; t++)
  tmax = smax;
  if (tmax > 4) tmax = 4; 
  for (t = 0; t < tmax; t++)
    {
      /*	  printf(" rate_gen channel %d volts %12.4f   digital # %d\n",
		  t,volts[t],VoltToCounts(volts[t]));  */
      if (estop != 5) {
	if (estop <=2 ) volts[t] = 0;
	if (estop ==3 ) volts[t] = 0.1;
	if (estop ==4 ) volts[t] = -0.1;
	}
      count	= VoltToCounts(volts[t]);
      CtrlByte[0] &= ~(CBMask * 3);    // JE 7-12-2003 clear mask bits for this axis only
      if (fabs (volts[t]) > .001) CtrlByte[0]+=CBMask * 2;
      if (volts[t] < 0.0) CtrlByte[0]+=CBMask;
      WriteData(count & 0xff);
      WriteData(count>>8 & 0xff);
      CBMask = CBMask * 4;
    }
  SelWrt(RATE_CTRL_0,CtrlByte[0] & 0xff);
  WriteData(0x37);   // restore limit
  WriteData(0xff);
  //    SelWrt(0x2C,(CtrlByte | 0x02));
  /*	printf("\n");  */
#else
  SelAddr(RATE_GEN_4);
  CBMask = 1;
  for (t = 4; t < smax; t++)
    {
      /*	  printf(" RATE_GEN channel %d volts %12.4f   digital # %d\n",
		  t,volts[t],VoltToCounts(volts[t]));  */
      count	= VoltToCounts(volts[t]);
      CtrlByte[1] &= ~(CBMAsk * 3);    // JE 7-12-2003 clear mask bits for this axis only
      if (fabs (volts[t]) > .001) CtrlByte[1]+=CBMask * 2;
      if (volts[t] < 0.0) CtrlByte[1]+=CBMask;
      WriteData(count & 0xff);
      WriteData(count>>8 & 0xff);
      CBMask = CBMask * 4;
    }
  SelWrt(RATE_CTRL_4,CtrlByte[1] & 0xff);
  //    SelWrt(0x2C,(CtrlByte | 0x02));
  /*	printf("\n");  */
#endif
  return 0;
}
