#include <linux/types.h>
#ifdef __attribute_used__
#undef __attribute_used__
#endif
#ifdef __attribute_pure__
#undef __attribute_pure__
#endif
#include <sys/cdefs.h>
#include <float.h>
#include <math.h>
//ppmc rate generator functions
#include "posemath.h"
#include "emcmot.h"
#include "ppmc.h"

#define TRUE 1
#define FALSE 0

/* Many hacks here, as this code needs to know relationship
   between steps and velocity */

#define MaxVel        2.0     /* velocity in IPS  */
#define CtsPerIn  4000.0    /* counts / inch   */
#define RateClock 10000000.0 /* oscillator, 10 MHz */

/* diagnostics printing */
#include "diagnostics.h"

static int CtrlByte[2];
static int estop;
static int inited = TRUE;  // makes it get setup and hold time from ini file when coming out of E-stop
// this insanity is needed because the ini file data is not available when the hardware is initted by
//    the DAC_Init function!

// Jon Elson 6/2/2005 get setup and hold times from ini file parameters
// Jon Elson 12/15/2003 don't try to talk to non-existant modules
// Jon Elson 12/10/2003 turn up thew speed some more
// Jon Elson 9/19/2003 fix a bunch of bugs in the 8-axis support

/* Mod Jon Elson 7-12-2003 to allow mixing of calls to ppmcDacWrite and ppmcDacWriteAll
   without stepping on each other's toes, especially in relation to the control
   registers, which are shared between 4 axes each. */
//   it turns out the above hackery is not used by EMC

char *ppmcModel(void)
{
    return "Parallel port interface for Universal steppers";
}

/* need to point to emcmotIo and emcmotConfig structures */
extern EMCMOT_IO *emcmotIo;
extern EMCMOT_CONFIG *emcmotConfig;

/*
VoltToCount(double volts)  converts a commanded 'voltage' to a
number of count cycles to program the rate generator.  */

__inline long VoltToCounts(double volts)
{
	long Count;
        double Velocity, Rate;
	Velocity = fabs (volts) * MaxVel / 10.0;
	Rate = Velocity * CtsPerIn;
	if (Rate == 0.0)
	    Count = 0xffffff;
	else
	    Count = (long) ((RateClock / Rate) - 2.0);
	if (Count > 0xffffff) Count = 0xffffff;
	if (Count < 30 ) Count = 30;  // max rate = 300 KHz  // JE 12-10-2003
	return (long)Count;
}

int RawDAC(int nAxis, double volts)
{
  long count;
  int CBMask,dacval;
  
  if (nAxis == 3 )        /* special for spindle DAC */
    {
      dacval = fabs(volts)*25.5;
      dacval = 123;
      SelWrt(ENCCNT0 + 15,dacval & 0xff);
      return 0;
    }
  if ( (nAxis > PPMC_MAX_AXIS) || (nAxis < 0) )        /* is axis within range? */
    return -1;

  /* convert -10.0 -> 0,0 to 0x8000 10 -> 0xFFFF */
  if (nAxis <=3)
   {
     SelAddr(RATE_GEN_0 + nAxis * 3);
     count = VoltToCounts(volts);
     WriteData(count & 0xff);
     WriteData(count>>8 & 0xff);
     WriteData(count>>16 & 0xff);
     CBMask = 1 << (nAxis * 2);
     CtrlByte[0] &= ~(CBMask * 3);    // clear mask bits for this axis only
     if (fabs (volts) > .001) CtrlByte[0]+=CBMask * 2;
     if (volts < 0.0) CtrlByte[0]+=CBMask;
     SelWrt(RATE_CTRL_0,CtrlByte[0] & 0xff);
    }
  else {
     SelAddr(RATE_GEN_4 + (nAxis-4) * 3);
     count = VoltToCounts(volts);
     WriteData(count & 0xff);
     WriteData(count>>8 & 0xff);
     WriteData(count>>16 & 0xff);
     CBMask = 1 << ((nAxis-4) * 2);
     CtrlByte[1] &= ~(CBMask * 3);    // clear mask bits for this axis only
     if (fabs (volts) > .001) CtrlByte[1]+=CBMask * 2;
     if (volts < 0.0) CtrlByte[1]+=CBMask;
     SelWrt(RATE_CTRL_4,CtrlByte[1] & 0xff);
    }

	
	
  return 0;
}

short DACInit(void)
{
  int i,j;
  CtrlByte[0] = 0;
  CtrlByte[1] = 0;
  j=ENCCNT0+15;
  i = SelRead(j);
  if ((i & 0xf0) != 0x40) {
    diagnostics("ppmc_rate V1.5 could not find a Universal Stepper module at IEEE-1284 address 0x%2x\n",j); 
    return -1;              /* blow it up ! */
  }
else
    diagnostics("ppmc_rate V1.5 found a Universal Stepper ver %d module at IEEE-1284 address 0x%2x\n",i&0x0f,j);

  // talk to 2nd module first, in case it doesn't exist
  //       	SelWrt(RATE_CTRL_4,0);    // turn off all axes
  //	WriteData(155);           // Setup delay = 10.0 uS
  //	WriteData(245);           // Step Width = 1.5 uS    // JE 12-10-2003
       	SelWrt(RATE_CTRL_0,0);    // turn off all axes
return 0;
}

short DACQuit(void)
{
	   return DACInit();  //init sets all outputs to zero
}
int ppmcDacNum(void)
{
	return PPMC_MAX_AXIS;
}

int ppmcDacWrite(int dac, double volts)
{
  return RawDAC(dac, volts);
}

int ppmcDacWriteAll(int max, double * volts)
{
  long count;
  int t,CBMask;
  int smax,tmax,i;
 
  /* clip smax to max supported-- if they want more, ignore */
  if (max > PPMC_MAX_AXIS) {
    smax = PPMC_MAX_AXIS;
  }
  else {
    smax = max;
  }
#ifndef PPMC_8_AXIS
  if ((emcmotIo->OutBytes[0] & 0x80) == 0) {
    if (estop != 1) diagnostics("going into E-stop\n");
    estop = 1;
  }
  if ((emcmotIo->OutBytes[0] & 0x80) != 0) {
    if (estop == 1) {
      inited = TRUE;  // force update of SETUP and HOLD
      estop = 0;      // do update of SETUP and HOLD only once
      diagnostics("ppmc_rate V1.5 estop is now off\n"); 
    }
  }

	if (inited) {
	  SelWrt(RATE_CTRL_0,0);    // turn off all axes
//	  diagnostics("ppmc_rate V1.5 SETUP_TIME * 1000 = %d\n",(int) (emcmotConfig->setup_time[0]*1000)); 
//	  diagnostics("ppmc_rate V1.5  HOLD_TIME * 1000 = %d\n",(int) (emcmotConfig->hold_time[0]*1000)); 
	  i = (255- (int) (emcmotConfig->setup_time[0]*10.0 + 0.5)) & 0xff;
//	  diagnostics("ppmc_rate V1.5 SETUP_TIME =  %d\n",i); 
	  WriteData(i);
	  i = (255- (int) (emcmotConfig->hold_time[0]*10.0 + 0.5)) & 0xff;
//	  diagnostics("ppmc_rate V1.5 HOLD_TIME =  %d\n",i); 
	  WriteData(i);
	  inited = FALSE;
	}
  //take advantage of auto increment of address pointer in ppmc
	SelAddr(RATE_GEN_0);
	CBMask = 1;
	//	for (t = 0; t < smax; t++)
	tmax = smax;
	if (tmax > 4) tmax = 4; 
	for (t = 0; t < tmax; t++)
	{
  /*	  printf(" rate_gen channel %d volts %12.4f   digital # %d\n",
	  t,volts[t],VoltToCounts(volts[t]));  */
	  count	= VoltToCounts(volts[t]);
	  CtrlByte[0] &= ~(CBMask * 3);    // JE 7-12-2003 clear mask bits for this axis only
	  if (fabs (volts[t]) > .001) CtrlByte[0]+=CBMask * 2;
	  if (volts[t] < 0.0) CtrlByte[0]+=CBMask;
	  WriteData(count & 0xff);
	  WriteData(count>>8 & 0xff);
	  WriteData(count>>16 & 0xff);
	  CBMask = CBMask * 4;
    }
          SelWrt(RATE_CTRL_0,CtrlByte[0] & 0xff);
	  //    SelWrt(0x2C,(CtrlByte | 0x02));
	/*	printf("\n");  */
#else
	SelAddr(RATE_GEN_4);
	CBMask = 1;
	for (t = 4; t < smax; t++)
	{
  /*	  printf(" RATE_GEN channel %d volts %12.4f   digital # %d\n",
	  t,volts[t],VoltToCounts(volts[t]));  */
	  count	= VoltToCounts(volts[t]);
	  CtrlByte[1] &= ~(CBMAsk * 3);    // JE 7-12-2003 clear mask bits for this axis only
	  if (fabs (volts[t]) > .001) CtrlByte[1]+=CBMask * 2;
	  if (volts[t] < 0.0) CtrlByte[1]+=CBMask;
	  WriteData(count & 0xff);
	  WriteData(count>>8 & 0xff);
	  WriteData(count>>16 & 0xff);
	  CBMask = CBMask * 4;
    }
          SelWrt(RATE_CTRL_4,CtrlByte[1] & 0xff);
	  //    SelWrt(0x2C,(CtrlByte | 0x02));
	/*	printf("\n");  */
#endif
  return 0;
}
