#ifndef PARPORT_H
#define PARPORT_H

/*
  parport.h

  Decls for parallel port I/O

  Modification history:

  13-Mar-2000 WPS added unused attribute to parport_h to avoid 'defined but not used' compiler warning, and added (void) to functions with no arguments to avoid 'declaration is not a prototype' compiler warnings.
  14-Jan-1999 WPS added extern "C"'s so that PARPORT_IO_ADDRESS can 
  be used in C++ files with the Microsoft Visual C++ compiler which apparantly
  considers a C integer and a C++ integer to be different.
  7-Aug-1998  FMP changed extInit/Quit() to extDioInit/Quit()
  9-Jul-1998  FMP created PARPORT_IO_ADDRESS variable, set DEFAULT_ to be
  0x278 for *external* card
  24-Oct-1997 FMP created
  */

#ifdef __cplusplus
extern "C" {
#endif

/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x) 
#endif
#endif

static char __attribute__((unused)) parport_h[] = "$Id: ppmcparport.h,v 1.1.1.1 2005/05/31 11:12:19 paul Exp $";


/* base I/O address */
extern unsigned long int IO_BASE_ADDRESS;

/* change the default for your board */
#define DEFAULT_IO_BASE_ADDRESS 0x278
/* #define DEFAULT_IO_BASE_ADDRESS 0x378 */

extern int pptDioInit(const char * stuff);
extern int pptDioQuit(void);

/* returns the max input index, output index; max bytes, shorts, and words */
extern int pptDioMaxInputs(void);	/* index < this for pptDioRead() */
extern int pptDioMaxOutputs(void);	/* index < this for pptDioWrite(),Check() */

/* reads value of digital input at index, stores in value */
extern int pptDioRead(int index, int *value);

/* writes value (non-zero means 1, 0 is 0) at digital out at index */
extern int pptDioWrite(int index, int value);

/* reads value of digital OUT at index, stores in value. Useful
   for checking values of previous writes. Returns 0 if OK, -1 if
   bad index or can't read if they're write-only. */
extern int pptDioCheck(int index, int *value);

/* byte, short, and word reads, writes. Index starts at 0, indexes up
   through bytes, short ints, and ints */
extern int pptDioByteRead(int index, unsigned char *byte);
extern int pptDioShortRead(int index, unsigned short *sh);
extern int pptDioWordRead(int index, unsigned int *word);
extern int pptDioByteWrite(int index, unsigned char byte);
extern int pptDioShortWrite(int index, unsigned short sh);
extern int pptDioWordWrite(int index, unsigned int word);
extern int pptDioByteCheck(int index, unsigned char *byte);
extern int pptDioShortCheck(int index, unsigned short *sh);
extern int pptDioWordCheck(int index, unsigned int *word);


#ifdef __cplusplus
}
#endif

#endif
