/********************************************************************
*
* Description: realtimeio.c
*   Low level driver to route user space IO through to the realtime
*   code. Typically, this would be linked to bridgeportio and used
*   in conjunction with the Vital card or Jon Elson's ppmc interface.
*
* Author: Paul Corner
* License: GPL Version 2
* System: Linux
*    
* Copyright (c) 2004 All rights reserved.
*
* Last change: 
* $Revision: 1.2 $
* $Author: paul $
* $Date: 2005/08/08 22:56:38 $
********************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include "extintf.h"
#include "emcmot.h"
#include "emcioctl.h"
#include "usrmotintf.h"

/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif

static char __attribute__((unused)) ident[] = "$Id: realtimeio.c,v 1.2 2005/08/08 22:56:38 paul Exp $";

static EMCMOT_COMMAND emcmotCommand;
static EMCMOT_STATUS emcmotStatus;
int IO_BASE_ADDRESS = 0x0080;	// Use a dummy IO address..

int extDioInit(const char * stuff)
{
  return 0;
}

int extDioQuit(void)
{
  return 0;
}

int extDioRead(int index, int *value)
{
  emcmotCommand.command = EMCMOT_READ_INDEX_BIT;
  emcmotCommand.index = index;

  usrmotWriteEmcmotCommand(&emcmotCommand);
  if (0 != usrmotReadEmcmotStatus(&emcmotStatus)) {
    return -1;
  }
  *value = (emcmotStatus.level) ? 1 : 0;

  return 0;
}

int extDioWrite(int index, int value)
{
  emcmotCommand.command = EMCMOT_READ_INDEX_BIT;
  emcmotCommand.index = index;
  emcmotCommand.level = value;
  return usrmotWriteEmcmotCommand(&emcmotCommand);
}

int extDioCheck(int index, int *value)
{
  emcmotCommand.command = EMCMOT_CHECK_INDEX_BIT;
  emcmotCommand.index = index;

  usrmotWriteEmcmotCommand(&emcmotCommand);
  if (0 != usrmotReadEmcmotStatus(&emcmotStatus)) {
    return -1;
  }
  *value = (emcmotStatus.level) ? 1 : 0;
  return 0;
}

int extAioWrite(int index, double volts)
{
  return 0;
}

#if 0
static int emc_fd = -1;	// File descriptor for h/w accesses.

int rtDioInit(const char * stuff)
{
    emc_fd = open("/dev/emc", O_RDWR);
    if (emc_fd < 0) {
	printf("Operation failed\n");
	return -1;
    }
    return 0;
}

int rtDioQuit(void)
{
    close(emc_fd);
    return 0;
}

int rtDioWrite(int index, int value)
{
    int ret = 0;

    if (value > 0) {
	ret = ioctl(emc_fd, EMC_IOW_DIO_SET, &index);
    } else {
	ret = ioctl(emc_fd, EMC_IOW_DIO_CLEAR, &index);
    }
    if (ret < 0) {
	perror("IO error");
	return -1;
    }
    return 0;
}

int rtDioRead(int index, int *value)
{
    int ret = 0;

    printf("\nCall ioctl\n");
    ret = ioctl(emc_fd, EMC_IOW_DIO_READ, &index);
    if (ret < 0) {
	perror("IO error");
	return -1;
    }
    return 0;
}

int rtAioWrite(int index, double value)
{
    int ret = 0;
    int data[2];

    data[0] = index;
    data[1] = (int) value;
    ret = ioctl(emc_fd, EMC_IOW_AIO_WRITE, data);
    if (ret < 0) {
	perror("IO error");
	return -1;
    }
    return 0;
}

int rtAioRead(int index, double *value)
{
    int ret = 0;
    int tmp;

    tmp = index;
    ret = ioctl(emc_fd, EMC_IOW_AIO_READ, &tmp);
    if (ret < 0) {
	perror("IO error");
	return -1;
    }
    *value = tmp;
    return 0;
}

char *rtModel(char *buffer)
{
    int ret = 0;

    ret = ioctl(emc_fd, EMC_IOR_MODEL, buffer);
    if (ret < 0) {
	perror("IO error");
	return NULL;
    }
    return buffer;
}

#ifdef MAIN
int main(void)
{
    int tmp = 0;
    char buf[LINELEN] = "I'm just an empty string";
    rtDioInit("stuff\n");
    printf("Model %s\n", rtModel(buf));
    rtDioRead(0x66, &tmp);
    rtDioWrite(0x74, tmp);
    rtAioWrite(0x74, tmp);
    rtDioQuit();
    return 0;
}
#endif
#endif
