/********************************************************************
*
* Description: ext526mot.c
*   Wrapper for Sensorray 526 functions
*
* Author: Paul Corner
* License: GPL Version 2
* System: Linux
*    
* Copyright (c) 2005 All rights reserved.
*
* Last change: 
* $Revision: 1.1.1.1 $
* $Author: paul $
* $Date: 2005/05/31 11:12:19 $
********************************************************************/

#include "extintf.h"
#include "emcmot.h"
#include "sensoray526.h"

/* Uncomment any of the following declarations to inhibit
   the use of home switches or index pulses when homing, or to
   ignore limit switches if you don't have them. */

/* Uncomment NO_HOME_SWITCH if you want homing sequence to
   ignore the home switch, and only use the index pulse for homing.
   This has the effect of making the home switches always appear tripped. */
/* #define NO_HOME_SWITCH */

/* Uncomment NO_INDEX_PULSE if you want homing sequence to
   ignore the index pulse, and only use the home switch for homing.
   This has the effect of making the index pulse always appear present. */
/* #define NO_INDEX_PULSE */

/* Uncomment NO_LIMIT_SWITCH if you don't have limit switches.
   This has the effect of making the positive and negative travel
   limit switches never appear tripped. */
/* #define NO_LIMIT_SWITCH */

char *extModel(void)
{
    return s526Model();
}

int extMotInit(const char * stuff)
{
  return s526MotInit(stuff);
}

int extAioInit(const char * stuff)
{
  return s526AioInit(stuff);
}

int extDioInit(const char * stuff)
{
  return s526DioInit(stuff);
}

int extMotQuit(void)
{
  return s526MotQuit();
}

int extAioQuit(void) 
{
  return s526AioQuit();
}

int extDioQuit(void)
{
  return s526DioQuit();
}

int extMotCycle(const char * stuff)
{
  return 0;
}

int extAioWrite(int index, double volts)
{
  return s526AioWrite(index, volts);
}

int extEncoderReadAll(int max, double * counts)
{
  return s526EncoderReadAll(max, counts);
}

int extDacWrite(int max, double volts)
{
  return s526DacWrite(max, volts);
}

int extDacWriteAll(int max, double * volts)
{
  return s526DacWriteAll(max, volts);
}

int extEncoderSetIndexModel(unsigned int model)
{
  return s526EncoderSetIndexModel(model);
}

int extEncoderResetIndex(int axis)
{
  return s526EncoderResetIndex(axis);
}

int extEncoderReadLatch(int axis, int * flag)
{
#ifdef NO_INDEX_PULSE
  *flag = 1;
  return 0;
#else
  return s526EncoderReadLatch(axis, flag);
#endif
}

int extMaxLimitSwitchRead(int axis, int * flag) 
{
#ifdef NO_LIMIT_SWITCH
  *flag = 0;
  return 0;
#else
  return s526MaxLimitSwitchRead(axis, flag);
#endif
}

int extMinLimitSwitchRead(int axis, int * flag) 
{
#ifdef NO_LIMIT_SWITCH
  *flag = 0;
  return 0;
#else
  return s526MinLimitSwitchRead(axis, flag);
#endif
}

int extHomeSwitchRead(int axis, int * flag) 
{
#ifdef NO_HOME_SWITCH
  *flag = 1;
  return 0;
#else
  return s526HomeSwitchRead(axis, flag);
#endif
}

int extAmpEnable(int axis, int enable)
{
  return s526AmpEnable(axis, enable);
}

int extAmpFault(int axis, int * fault)
{
  return s526AmpFault(axis, fault);
}

int extDioRead(int index, int *value) 
{
  return s526DioRead(index, value);
}

int extDioWrite(int index, int value) 
{
  return s526DioWrite(index, value);
}

int extDioCheck(int index, int *value)
{
  return s526DioCheck(index, value);
}

extern int IO_BASE_ADDRESS;

int extBaseFind(void)
{
    return IO_BASE_ADDRESS;
}

int extDioMaxInputs(void)
{
    return s526DioMaxInputs();
}

int extDioMaxOutputs(void)
{
    return s526DioMaxOutputs();
}

int extAioStart(int index)
{
    return 0;
}

int extAioReady(void)
{
    return 0;
}

int extAioRead(int index, double *value)
{
    return 0;
}

int extEncoderReadLevel(int encoder, int *flag)
{
    return 0;
}

