/********************************************************************
*
* Description: sensoray526.h
*   Defines & declarations for sensoray526.c
*
* Author: Paul Corner
* License: GPL Version 2
* System: Linux
*    
* Copyright (c) 2005 All rights reserved.
*
* Last change: 
* $Revision: 1.1.1.1 $
* $Author: paul $
* $Date: 2005/05/31 11:12:19 $
********************************************************************/
#ifndef SENSORAY526_H
#define SENSORAY526_H

// Use the default base address that Sensoray ship their board with.
#define DEFAULT_IO_BASE_ADDRESS 0x2c0

#ifdef __cplusplus
extern "C" {
#endif

// Number of axes in system 
#define S526_MAX_AXIS 4
#include "emcmotcfg.h"

#define TCR IO_BASE_ADDRESS + 0x00
#define WDC IO_BASE_ADDRESS + 0x02
#define DAC IO_BASE_ADDRESS + 0x04
#define ADC IO_BASE_ADDRESS + 0x06
#define ADD IO_BASE_ADDRESS + 0x08
#define DIO IO_BASE_ADDRESS + 0x0a
#define IER IO_BASE_ADDRESS + 0x0c
#define ISR IO_BASE_ADDRESS + 0x0e
#define MSC IO_BASE_ADDRESS + 0x10
#define C0L IO_BASE_ADDRESS + 0x12
#define C0H IO_BASE_ADDRESS + 0x14
#define C0M IO_BASE_ADDRESS + 0x16
#define C0C IO_BASE_ADDRESS + 0x18
#define C1L IO_BASE_ADDRESS + 0x1a
#define C1H IO_BASE_ADDRESS + 0x1c
#define C1M IO_BASE_ADDRESS + 0x1e
#define C1C IO_BASE_ADDRESS + 0x20
#define C2L IO_BASE_ADDRESS + 0x22
#define C2H IO_BASE_ADDRESS + 0x24
#define C2M IO_BASE_ADDRESS + 0x26
#define C2C IO_BASE_ADDRESS + 0x28
#define C3L IO_BASE_ADDRESS + 0x2a
#define C3H IO_BASE_ADDRESS + 0x2c
#define C3M IO_BASE_ADDRESS + 0x2e
#define C3C IO_BASE_ADDRESS + 0x30
#define EDD IO_BASE_ADDRESS + 0x32
#define ECC IO_BASE_ADDRESS + 0x34

/* Config word for the IO port
   D0-D3 inputs - Clear bit 10
   D4-D7 outputs - Set bit 11
   interrupts - Not bothered ! */
#define DIO_CONFIG 0x0800

/* Bit defines for the inputs */
#define INPUT_BASE 0
#define S526_MAX_DIO_INPUTS 4
#define S526_HOME_INDEX	     0x00
#define S526_MIN_LIMIT_INDEX 0x01
#define S526_MAX_LIMIT_INDEX 0x02
#define S526_AMP_FAULT_INDEX 0x03
/* Bit defines for the outputs */
  /* NOTE: The watchdog timer is enabled by the software
     and is set for a 125mSec timeout. The relay *should* 
     be part of the Amp Enable circuitry as the o/p changes
     state when the system is active. Should the servo loop
     die, the timer will "flip" the relay o/p automatically
     on time out.
*/
#define OUTPUT_BASE 0x04
#define S526_MAX_DIO_OUTPUTS 4
#define S526_AMP_ENABLE_INDEX 0x00

extern int IO_BASE_ADDRESS;

extern char *s526Model(void);

extern int s526MotInit(const char * stuff);
extern int s526MotQuit(void);

extern int s526DioInit(const char * stuff);
extern int s526DioQuit(void);

extern int s526AioInit(const char * stuff);
extern int s526AioQuit(void);

extern int s526DacNum(void);
extern int s526DacWrite(int axis, double volts);
extern int s526DacWriteAll(int max, double * volts);

extern int s526EncoderSetIndexModel(unsigned int model);
extern int s526EncoderNum(void);
extern int s526EncoderRead(int axis, double * counts);
extern int s526EncoderReadAll(int max, double * counts);
extern int s526EncoderResetIndex(int axis);
extern int s526EncoderReadLatch(int axis, int * flag);
extern int s526EncoderReadLevel(int axis, int * flag);

extern int s526MaxLimitSwitchRead(int axis, int * flag);
extern int s526MinLimitSwitchRead(int axis, int * flag);
extern int s526HomeSwitchRead(int axis, int * flag);

extern int s526AmpEnable(int axis, int enable);
extern int s526AmpFault(int axis, int * fault);

extern int s526DioMaxInputs(void);
extern int s526DioMaxOutputs(void);
extern int s526DioRead(int index, int *value);
extern int s526DioWrite(int index, int value);
extern int s526DioCheck(int index, int *value);

extern int s526AioMaxInputs(void);
extern int s526AioMaxOutputs(void);
extern int s526AioStart(int index);
extern int s526AioReady(void);
extern int s526AioRead(int index, double *volts);
extern int s526AioWrite(int index, double volts);
extern int s526AioCheck(int index, double *volts);

#ifdef __cplusplus
}
#endif

#endif
