#ifndef SIM_H
#define SIM_H

/*
  sim.h

  Decls for external interface simulation

  Modification history:

   7-Nov-2000 WPS added simPos to store a simulated position to test against for better probe simulation.
  20-Mar-2000 WPS added unused attribute to sim_h to avoid 'defined but not used' compiler warning.
  20-Mar-2000 WPS added (void) to functions with no arguments to avoid the
  "function declaration isn't a prototype" compiler warning.
  10-Aug-1998  FMP fixed decls for simAioXXX (they were extAioXX by mistake)
  7-Aug-1998  FMP changed to simDio/Aio/MotInit/Quit()
  18-Mar-1998  FMP took out simInit(), simQuit() and replaced with
  simMotInit(), simIoInit(), etc.
  16-Oct-1997  FMP created
  */

/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif

static char __attribute__((unused)) sim_h[] = "$Id: sim.h,v 1.1.1.1 2005/05/31 11:12:19 paul Exp $";

#ifdef __cplusplus
extern "C" {
#endif

/* motion part */
extern int simMotInit(const char * stuff);
extern int simMotQuit(void);
extern int simDacNum(void);
extern int simDacWrite(int dac, double volts);
extern int simDacWriteAll(int max, double * volts);
extern int simEncoderSetIndexModel(unsigned int model);
extern int simEncoderNum(void);
extern int simEncoderRead(int encoder, double * counts);
extern int simEncoderReadAll(int max, double * counts);
extern int simEncoderResetIndex(int encoder);
extern int simEncoderReadLatch(int encoder, int * flag);
extern int simEncoderReadLevel(int encoder, int * flag);
extern int simMaxLimitSwitchRead(int axis, int * flag);
extern int simMinLimitSwitchRead(int axis, int * flag);
extern int simHomeSwitchRead(int axis, int * flag);
extern int simAmpEnable(int axis, int enable);
extern int simAmpFault(int axis, int * fault);

/* digital IO part */
extern int simDioInit(const char * stuff);
extern int simDioQuit(void);
extern int simDioMaxInputs(void);       /* index < this for simDioRead() */
extern int simDioMaxOutputs(void);      /* index < this for simDioWrite(),Check() */
extern int simDioRead(int index, int *value);
extern int simDioWrite(int index, int value);
extern int simDioCheck(int index, int *value);
extern int simDioByteRead(int index, unsigned char *byte);
extern int simDioByteWrite(int index, unsigned char byte);
extern int simDioByteCheck(int index, unsigned char *byte);

/* analog IO part */
extern int simAioInit(const char * stuff);
extern int simAioQuit(void);
extern int simAioMaxInputs(void);
extern int simAioMaxOutputs(void);
extern int simAioRead(int index, double *volts);
extern int simAioWrite(int index, double volts);
extern int simAioCheck(int index, double *volts);


extern double simPos[];

#ifdef __cplusplus
}
#endif

#endif
