/*
  simaio.c

  Simulated external interface for analog IO

  Modification history:

  23-Mar-2004  FMP took out EMC_DEBUG flag, since emcglb.c,h bring along
  a lot of excess baggage
  20-Mar-2000 WPS added unused attribute to ident to avoid 'defined but not used' compiler warning.
  20-Mar-2000 WPS added (void) to functions with no arguments to avoid the
  "function declaration isn't a prototype" compiler warning.
  21-Sep-1999  WPS eliminate sscanf and printf calls not supported under CE.
  15-Sep-1999  FMP bracketed printing with check for EMC_DEBUG &
  EMC_DEBUG_IO_POINTS
  7-Aug-1998  FMP created from analog part of simio.c
  */

#ifdef nonrealtime
#include <stdio.h>              /* printf() */
#include "rcs_print.hh"          /* rcs_print() */
#endif
#include "sim.h"                /* these decls */

/* ident tag */

#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif

static char  __attribute__((unused)) ident[] = "$Id: simaio.c,v 1.1.1.1 2005/05/31 11:12:19 paul Exp $";

/*
  Analog IO model

  Provides for simulated analog ins, outs
  */

#define SIM_AIO_MAX_INPUTS 8
#define SIM_AIO_MAX_OUTPUTS 8

static double simAioInputs[SIM_AIO_MAX_INPUTS];
static double simAioOutputs[SIM_AIO_MAX_OUTPUTS];

int simAioInit(const char * filename)
{
  return 0;
}

int simAioQuit(void)
{
  return 0;
}

int simAioMaxInputs(void)
{
  return SIM_AIO_MAX_INPUTS;
}

int simAioMaxOutputs(void)
{
  return SIM_AIO_MAX_OUTPUTS;
}

int simAioRead(int index, double *volts)
{
  if (index < 0 || index >= SIM_AIO_MAX_INPUTS) {
    return -1;
  }

  *volts = simAioInputs[index];

  return 0;
}

int simAioWrite(int index, double volts)
{
  if (index < 0 || index >= SIM_AIO_MAX_OUTPUTS) {
    return -1;
  }

  simAioOutputs[index] = volts;

#ifdef nonrealtime
  rcs_print("[AOUT]\t%d\t%f\n", index, volts);
#endif

  return 0;
}

int simAioCheck(int index, double *volts)
{
  if (index < 0 || index >= SIM_AIO_MAX_OUTPUTS) {
    return -1;
  }

  *volts = simAioOutputs[index];

  return 0;
}
