
#include "config.h"
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/io.h>
#include <errno.h>

#include "extintf.h"

/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif

static char __attribute__ ((unused)) ident[] = "$Id: stgdiag.c,v 1.2 2005/08/08 22:56:38 paul Exp $";

/*
  Be root to run it
    */

#define NUM_AXES 4
extern int IO_BASE_ADDRESS;

static int done = 0;
static void quit(int sig)
{
    done = 1;
}

enum { POS = 1, LIMIT, HOME, INDEX, LATCH, DAC, DIO, ADC };

int main(int argc, char *argv[])
{
    int which = POS;
    int newline = 0;
    double pos[NUM_AXES];
    double oldpos[NUM_AXES];
    int lim[NUM_AXES * 2];
    int oldlim[NUM_AXES * 2];
    int home[NUM_AXES];
    int oldhome[NUM_AXES];
    int index;
    int latch;
    int axis;
    int oldlatch;
    double volts;
    int t;
    char buf[BUFFERLEN];
    char *cptr;
    double adc[NUM_AXES];
    int argsused = 0;

    /* enable I/O port access */
    if (0 != iopl(3)) {
	printf("must be root to run\n");
	exit(1);
    }

    if (argc > 1) {
	if (!strcmp(argv[1], "pos")) {
	    which = POS;
	} else if (!strcmp(argv[1], "dio")) {
	    which = DIO;
	} else if (!strcmp(argv[1], "dac")) {
	    which = DAC;
	} else if (!strcmp(argv[1], "adc")) {
	    which = ADC;
	} else if (!strcmp(argv[1], "limit")) {
	    which = LIMIT;
	} else if (!strcmp(argv[1], "home")) {
	    which = HOME;
	} else if (!strcmp(argv[1], "index")) {
	    if (argc < 2) {
		which = 0;
		printf("No axis specified (using 0)\n");
	    } else if (1 == sscanf(argv[2], "%d", &axis)) {
		argsused = 1;
		which = INDEX;
	    } else {
		printf
		    ("syntax: %s {limit | home | index <axis> | latch <axis>}\n",
		     argv[0]);
		exit(1);
	    }
	} else if (!strcmp(argv[1], "latch")) {
	    if (1 == sscanf(argv[2], "%d", &axis)) {
		argsused = 1;
		which = LATCH;
	    } else {
		printf
		    ("syntax: %s {limit | home | index <axis> | latch <axis>}\n",
		     argv[0]);
		exit(1);
	    }
	} else {
	    printf
		("syntax: %s {limit | home | index <axis> | latch <axis>}\n",
		 argv[0]);
	    exit(1);
	}
    }

    if (argc > 2 + argsused) {
	if (1 != sscanf(argv[2], "%i", &IO_BASE_ADDRESS)) {
	    printf("bad address: %s\n", argv[2]);
	    exit(1);
	} else {
	    printf("setting base address to 0x%X\n", IO_BASE_ADDRESS);
	}
    }

    signal(SIGINT, quit);

    if (extMotInit(NULL) != 0) {
	exit(1);
    }
    if (extDioInit(NULL) != 0) {
	extMotQuit();
	exit(1);
    }
    if (extAioInit(NULL) != 0) {
	extDioQuit();
	extMotQuit();
	exit(1);
    }

    printf("Board Model = %s\n", extModel());
    IO_BASE_ADDRESS = extBaseFind();
    if (IO_BASE_ADDRESS == 0) {
	printf("can't find board\n");
	exit(1);
    } else {
	printf("found board at 0x%X\n", IO_BASE_ADDRESS);
    }

    if (which == POS) {
	while (!done) {
	    newline = 0;
	    extEncoderReadAll(NUM_AXES, pos);
	    for (t = 0; t < NUM_AXES; t++) {
		if (fabs(pos[t] - oldpos[t]) > 1000.0 && !newline) {
		    newline = 1;
		    printf("\n");
		}
		oldpos[t] = pos[t];
	    }
	    for (t = 0; t < NUM_AXES; t++) {
		printf("%+15.3f\t", pos[t]);
	    }
	    printf("\r");
	    fflush(stdout);
	    usleep(10000);
	}
    } else if (which == LIMIT) {
	while (!done) {
	    newline = 0;
	    for (t = 0; t < NUM_AXES; t++) {
		extMaxLimitSwitchRead(1, &lim[2 * t]);
		if (oldlim[2 * t] != lim[2 * t] && !newline) {
		    printf("\n");
		    newline = 1;
		}
		oldlim[2 * t] = lim[2 * t];
		extMinLimitSwitchRead(1, &lim[2 * t + 1]);
		if (oldlim[2 * t + 1] != lim[2 * t + 1] && !newline) {
		    printf("\n");
		    newline = 1;
		}
		oldlim[2 * t + 1] = lim[2 * t + 1];
	    }
	    if (newline) {
		for (t = 0; t < NUM_AXES; t++) {
		    printf("%+d\t", lim[2 * t]);
		    printf("%+d\t", lim[2 * t + 1]);
		}
		printf("\r");
		fflush(stdout);
	    }
	    usleep(10000);
	}
    } else if (which == HOME) {
	while (!done) {
	    newline = 0;
	    for (t = 0; t < NUM_AXES; t++) {
		extHomeSwitchRead(t, &home[t]);
		if (oldhome[t] != home[t] && !newline) {
		    printf("\n");
		    newline = 1;
		}
		oldhome[t] = home[t];
	    }
	    if (newline) {
		for (t = 0; t < NUM_AXES; t++) {
		    printf("%d\t", home[t]);
		}
		printf("\r");
		fflush(stdout);
	    }
	    usleep(10000);
	}
    } else if (which == INDEX) {
	if (axis < 0 || axis >= NUM_AXES) {
	    printf
		("syntax: %s {limit | home | index <axis> | latch <axis>}\n",
		 argv[0]);
	    exit(1);
	}

	extEncoderResetIndex(axis);
	while (!done) {
	    extEncoderReadLevel(axis, &index);
	    printf("%d ", index);
	    fflush(stdout);
	    usleep(10000);
	}
    } else if (which == LATCH) {
	if (axis < 0 || axis >= NUM_AXES) {
	    printf
		("syntax: %s {limit | home | index <axis> | latch <axis>}\n",
		 argv[0]);
	    exit(1);
	}

	if (-1 == fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK)) {
	    printf("Couldn's set flag for non-blocking on socket.\n");
	    return -1;
	}

	extEncoderResetIndex(axis);
	extEncoderReadLatch(axis, &latch);
	printf("%d\n          ", latch);
	fflush(stdout);
	oldlatch = latch;
	while (!done) {
	    if (read(STDIN_FILENO, buf, sizeof(buf)) > 0) {
		extEncoderResetIndex(axis);
	    }
	    extEncoderReadLatch(axis, &latch);
	    if (latch != oldlatch) {
		printf("\n%d\n        ", latch);
	    } else {
		printf(".");
	    }
	    oldlatch = latch;
	    fflush(stdout);
	    usleep(10000);
	}
    } else if (which == DAC) {
	printf("Axis? ");
	fgets(buf, BUFFERLEN, stdin);
	axis = (int) strtol(buf, 0, 0);
	while (!done) {
	    printf("DAC(%d): ", axis);
	    if (fgets(buf, BUFFERLEN, stdin) == NULL) {
		break;
	    }
	    if (!isdigit(buf[0]) && buf[0] != '-') {
		break;
	    }
	    cptr = buf;
	    while ((*cptr >= '0' && *cptr <= '9')
		   || *cptr == '-' || *cptr == '.' || *cptr == 'E') {
		cptr++;
	    }
	    *cptr = 0;
	    volts = strtod(buf, 0);
	    printf("volts=%f\n", volts);
	    extDacWrite(axis, volts);
	}
    } else if (which == ADC) {
	while (!done) {
	    for (t = 0; t < NUM_AXES; t++) {
		extAioStart(t);
                do {} while(extAioReady() == -EBUSY);
		extAioRead(t, &adc[t]);
		printf("%f\t", adc[t]);
	    }
	    printf("\r");
	    fflush(stdout);
	    usleep(10000);
	}
    } else if (which == DIO) {
	int val, i, max;
	max = extDioMaxInputs();
	for (i = 0; i < max; i++) {
	    printf("%d", i % 8);
	}
	printf("\n");
	for (i = 0; i < max; i++) {
	    extDioRead(i, &val);
	    printf("%d", val);
	}
	printf("\n");
    }
    printf("\n");
    extAioQuit();
    extDioQuit();
    extMotQuit();

    return 0;
}
