/*
  extvtimot.c

  Dispatcher of external motion functions for the Vigilant Technologies
  ISA-ENCDAC board.

  Modification history:

  24-Mar-2002  P.C. created from STG sources.
  */

#include "extintf.h"            /* these decls */
#include "vti.h"                /* decls for stg implementation */

/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif

static char __attribute__((unused))  ident[] = "$Id: extvtimot.c,v 1.7 2004/03/11 16:14:16 paul_c Exp $";

/* Uncomment any of the following declarations to inhibit
   the use of home switches or index pulses when homing, or to
   ignore limit switches if you don't have them. */

/* Uncomment NO_HOME_SWITCH if you want homing sequence to
   ignore the home switch, and only use the index pulse for homing.
   This has the effect of making the home switches always appear tripped. */
/* #define NO_HOME_SWITCH */

/* Uncomment NO_INDEX_PULSE if you want homing sequence to
   ignore the index pulse, and only use the home switch for homing.
   This has the effect of making the index pulse always appear present. */
/* #define NO_INDEX_PULSE */

/* Uncomment NO_LIMIT_SWITCH if you don't have limit switches.
   This has the effect of making the positive and negative travel
   limit switches never appear tripped. */
/* #define NO_LIMIT_SWITCH */

int extMotInit(const char * stuff)
{
  return vtiMotInit(stuff);
}

int extBaseFind()
{
  return vtiBaseFind();
}

char * extModel()
{
  return "Vigilant card";
}

int extMotQuit(void)
{
  return vtiMotQuit();
}

int extMotCycle(const char * stuff)
{
  return 0;
}

int extDacNum(void)
{
  return vtiDacNum();
}

int extDacWrite(int axis, double volts)
{
  return vtiDacWrite(axis, volts);
}

int extDacWriteAll(int max, double * volts)
{
  return vtiDacWriteAll(max, volts);
}

int extEncoderSetIndexModel(unsigned int model)
{
  return vtiEncoderSetIndexModel(model);
}

int extEncoderNum(void)
{
  return vtiEncoderNum();
}

int extEncoderRead(int axis, double * counts)
{
  return vtiEncoderRead(axis, counts);
}

int extEncoderReadAll(int max, double * counts)
{
  return vtiEncoderReadAll(max, counts);
}

int extEncoderResetIndex(int axis)
{
  return vtiEncoderResetIndex(axis);
}

int extEncoderReadLatch(int axis, int * flag)
{
#ifdef NO_INDEX_PULSE
  *flag = 1;
  return 0;
#else
  return vtiEncoderReadLatch(axis, flag);
#endif
}

int extEncoderReadLevel(int axis, int * flag)
{
  return vtiEncoderReadLevel(axis, flag);
}

int extMaxLimitSwitchRead(int axis, int * flag)
{
#ifdef NO_LIMIT_SWITCH
  *flag = 0;
  return 0;
#else
  return vtiMaxLimitSwitchRead(axis, flag);
#endif
}

int extMinLimitSwitchRead(int axis, int * flag)
{
#ifdef NO_LIMIT_SWITCH
  *flag = 0;
  return 0;
#else
  return vtiMinLimitSwitchRead(axis, flag);
#endif
}

int extHomeSwitchRead(int axis, int * flag)
{
#ifdef NO_HOME_SWITCH
  *flag = 1;
  return 0;
#else
  return vtiHomeSwitchRead(axis, flag);
#endif
}

int extAmpEnable(int axis, int enable)
{
  return vtiAmpEnable(axis, enable);
}

int extAmpFault(int axis, int * fault)
{
  return vtiAmpFault(axis, fault);
}

int extDioInit(const char * stuff)
{
  return vtiDioInit(stuff);
}

int extDioQuit(void)
{
  return vtiDioQuit();
}

int extDioMaxInputs(void)
{
  return vtiDioMaxInputs();
}

int extDioMaxOutputs(void)
{
  return vtiDioMaxOutputs();
}

int extDioRead(int index, int *value)
{
  return vtiDioRead(index, value);
}

int extDioWrite(int index, int value)
{
  return vtiDioWrite(index, value);
}

int extDioCheck(int index, int *value)
{
  return vtiDioCheck(index, value);
}

int extDioByteRead(int index, unsigned char *byte)
{
  return vtiDioByteRead(index, byte);
}

int extDioShortRead(int index, unsigned short *sh)
{
  return vtiDioShortRead(index, sh);
}

int extDioWordRead(int index, unsigned int *word)
{
  return vtiDioWordRead(index, word);
}

int extDioByteWrite(int index, unsigned char byte)
{
  return vtiDioByteWrite(index, byte);
}

int extDioShortWrite(int index, unsigned short sh)
{
  return vtiDioShortWrite(index, sh);
}

int extDioWordWrite(int index, unsigned int word)
{
  return vtiDioWordWrite(index, word);
}

int extDioByteCheck(int index, unsigned char *byte)
{
  return vtiDioByteCheck(index, byte);
}

int extDioShortCheck(int index, unsigned short *sh)
{
  return vtiDioShortCheck(index, sh);
}

int extDioWordCheck(int index, unsigned int *word)
{
  return vtiDioWordCheck(index, word);
}

int extAioInit(const char * stuff)
{
  return vtiAioInit(stuff);
}

int extAioQuit(void)
{
  return vtiAioQuit();
}

int extAioMaxInputs(void)
{
  return 0;
}

int extAioMaxOutputs(void)
{
  return 0;
}

int extAioStart(int index)
{
  return 0;
}

void extAioWait(void)
{
  return;
}

int extAioRead(int index, double *volts)
{
  return 0;
}

int extAioWrite(int index, double volts)
{
  return 0;
}

int extAioCheck(int index, double *volts)
{
  return 0;
}

/* Stub until such time as it is needed.. */
int extAioReady(void){return 0;}
