/*
   vti.c

    Functions for Vigilant Products motion control boards.

  Modification history:

  4-July-2002   P.C. Merged PCI routines
  26-March-2002 P.C. Created from stg sources.

*/


/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif

static char __attribute__((unused)) ident[] = "$Id: vti.c,v 1.15 2005/02/28 21:13:46 paul_c Exp $";

#define PCI

#ifdef realtime

#ifndef __KERNEL__
#define __KERNEL__
#endif
#include <linux/version.h>
#include <linux/pci.h>
#include <linux/vmalloc.h>

#else
// nonrealtime headers..
#include <asm/io.h>

#ifdef PCI
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/mman.h>
/* libpci header - package name usually called pciutils-dev */
#include <pci/pci.h>
#endif
#endif

#include "vti.h"               /* these decls */
#include "extintf.h"            /* EXT_ENCODER_INDEX_MODEL */
#include "diagnostics.h"       /* printf functions for rtai/rtlinux/linux */

/* base address-- override default at compile time in vti.h, or at module 
   load time with IO_BASE_ADDRESS=value. Unused with the PCI card */
unsigned short IO_BASE_ADDRESS = DEFAULT_IO_BASE_ADDRESS;

/*
  Here follows the routines to interface to the Encoder-DAC card.
   Most of the function descriptions have been lifted from extinf.h
  some of which will do nothing due to lack of IO.
*/
#ifdef PCI
/* Structures used by the PCI driver */
struct pci_dev *dev = NULL;
struct pci_access *device;
volatile struct encoder *encoder = NULL;
volatile struct timer *timer = NULL;
volatile struct dac *dac = NULL;
volatile struct ip *ip = NULL;
#ifndef __KERNEL__
static int mem_fd;		 /* File descriptor for /dev/mem access */
#endif
#endif

/* Init/quit functions */

/*
  vtiMotInit(const char * stuff)

  Call once before any of the other motion IO functions are called.
  'stuff' argument can be used to pass board-specific stuff like config
  files.
  */
int vtiMotInit(const char * stuff)
{
#ifdef PCI
#ifdef __KERNEL__
    dev = pci_find_device(VENDOR, DEVICE, dev);
    if (dev) {
	encoder=(volatile struct encoder *)ioremap(pci_resource_start(dev, 2), sizeof(encoder));
    } else {
/* If dev=NULL then no card is present */
	diagnostics("<1>**** No PCI-ENCDAC card detected **** %x\n",(int) dev);
	return -1;
    }
  diagnostics("Card detected in Slot: %2x\n", PCI_SLOT(dev->devfn));
#else
/* Code used to find PCI card and base address in user space only */
    device = pci_alloc();		/* initialise the pci_access struct */
    pci_init(device);		/* initialise the pci lib */
    pci_scan_bus(device);		/* Build a list of devices on the PCI bus */
    dev = device->devices;
    while ((dev != NULL)&&(dev->vendor_id != VENDOR)&&(dev->device_id != DEVICE)) {
    dev = dev->next;
    }
    mem_fd = open("/dev/mem", O_RDWR);
    if ((dev == NULL) || (mem_fd == -1)) {
      diagnostics("No PCI card detected.\n");
      return -1;
    } else {
      diagnostics("Card found in slot %x:%x\n", dev->bus, dev->dev);
    }
    /* using MAP_SHARED prevents linux from buffering any read/writes and so *should*
       be written immediately to the hardware */
    encoder =  mmap(NULL, sizeof(encoder), PROT_READ | PROT_WRITE, MAP_SHARED, mem_fd,
		     (off_t)dev->base_addr[2] & PCI_BASE_ADDRESS_MEM_MASK);
#endif
  diagnostics("Encoders mapped to %x\n", (int) encoder);
#else
    int t;

  t = INT_STATUS;
  diagnostics("First card test with an IF %x %x \n", t, IO_BASE_ADDRESS);
  if ((INT_STATUS & 0x0030) != 0x0020 && (INT_STATUS & 0x0030) != 0x0010) {
/* Check for the presence of the card - If the ID bits are not correct,
   then the board is probably not here. */
     diagnostics("No card detected\n");
      return -1;
    }
  /*
   Interrupts are not used by EMC (I think). So make sure they are
   masked - The card should power up at these settings, but still
   wise to force the defaults.
   */
  t = INT_STATUS;
  diagnostics("Card detected at 0x%x\n", IO_BASE_ADDRESS );
  LOGIC_CFG;			// Issue a reset to the board
  INT_DIGCFG(0);		// Set all DIO to input and mask interrupts
#endif
  ENC_MASK03(0);		// Mask axes 0-3 encoder interrupts
  ENC_MASK47(0);		// Mask axes 4-7 encoder interrupts
  ENC_RESET;			// Just one write to reset all encoder channels.
#ifndef PCI
  if ( ENC_COUNT0 != 0x0000) {
/* If the encoders have not been set to zero, then the card is either faulty
   or not present*/
     diagnostics("Nuts, no Encoder\n");
      return -1;
    }
#endif
   return 0;
}

/*
  vtiDioInit(const char * stuff)

  Call once before any of the other digital IO functions are called.
  'stuff' argument can be used to pass board-specific stuff like config
  files.
  */
int vtiDioInit(const char * stuff)
{
  return 0;
}

/*
  vtiDioQuit()

  Call once, after which point no other functions will be called until
  after a call to extDioInit().
  */
int vtiDioQuit(void)
{
  return 0;
}

/*
  extAioInit(const char * stuff)

  Call once before any of the other analog IO functions are called.
  'stuff' argument can be used to pass board-specific stuff like config
  files.
  */
int vtiAioInit(const char * stuff)
{
    char t;
#ifdef PCI
#ifdef __KERNEL__
    dac=(volatile struct dac *)ioremap(pci_resource_start(dev, 4), sizeof(dac));
#else
    if ((dev == NULL) || (mem_fd == -1)) {
      return -1;
    }
    dac =  mmap(NULL, sizeof(dac), PROT_READ | PROT_WRITE, MAP_SHARED, mem_fd,
		     (off_t)dev->base_addr[4] & PCI_BASE_ADDRESS_MEM_MASK);
#endif
    diagnostics("DACs mapped to %x\n", (int) dac);
#endif

      /* output 0's to amps */
      /* write 0 to DACs */
    for (t = 0; t < VTI_MAX_AXIS; t++) {
	diagnostics("Setting DAC %x to Zero.\n", t );
	vtiDacWrite(t, 0.0);
	}
    DAC_ENABLE;			/* Enable the DAC outputs */
    return 0;
}

/*
  vtiAioQuit()

  Call once, after which point no other functions will be called until
  after a call to vtiAioInit().
  */
int vtiAioQuit(void)
{
#ifdef PCI
    diagnostics("Unmapping DACs\n");
#ifdef __KERNEL__
    /* cast the pointer as a void to get rid of "discards qualifiers
       from pointer" warning at compile time. */
    iounmap((void *)dac);
#else
    munmap((void *)dac, sizeof(dac));
#endif
#else
    DAC_DISARM;		/* disable the relay and set DAC outputs to 0V */
#endif
    return 0;
}

/*
  vtiMotQuit()

  Call once, after which point no other functions will be called until
  after a call to vtiMotInit().  */
int vtiMotQuit(void)
{
#ifdef PCI
    diagnostics("Unmapping Encoders\n");
#ifdef __KERNEL__
    iounmap((void *)encoder);
#else
    munmap((void *)encoder, sizeof(encoder));
    close(mem_fd);
#endif
#endif
    return 0;
}

/* vtiBaseFind() returns the base address of the IO card. */
int vtiBaseFind(void)
{
#ifdef PCI
    return (int) encoder;
#else
    return IO_BASE_ADDRESS;
#endif
}


/* DAC functions */

/*
  vtiDacNum()

  returns number of DACs in system which is 8 in this case.
  */
int vtiDacNum(void)
{
#ifdef VTI_8_AXES
    return 8;
#else
    return 4;
#endif
}

/*
  vtiDacWrite(int axis, double volts)

  writes the value to the DAC at indicated dac, 0 .. max DAC - 1.
  Value is in volts. Returns 0 if OK, -1 if dac is out of range.
  */
int vtiDacWrite(int axis, double volts)
{
  if ((axis > VTI_MAX_AXIS) || (axis < 0)) {
      return -1;
    }
  DAC_SETIND;			/* Set DACs to individual update mode */
  RawDacOut(axis, volts);
  return 0;
}

/*
  vtiDacWriteAll(int max, double * volts)

  writes the values to the DAC range. Values are array of DAC outputs
  in volts. Returns 0 if OK, -1 if not max is out of range. This is
  provided for systems in which DACs can be written all at once for
  speed or synchronicity. max is number of DACs, so for dacs 0..3
  max would be 4.
  */
int vtiDacWriteAll(int max, double * volts)
{
  int t;
  if ((max > VTI_MAX_AXIS) || (max < 0)) {
      return -1;
    }
  DAC_SETSIM;			// Set DAC to simultaneous update mode    
  for (t = 0; t < max; t++) {
      RawDacOut(t, volts[t]);
     }
  DAC_UPDATE;			// Now update the DACs
  return 0;
}

/* Encoder functions */

/*
  vtiEncoderIndexModel()

  Returns the type of encoder indexing done by the board.

  The handling of index pulses for encoder homing is problematic. There
  are two models (at least) for how incremental encoders can be homed
  off the index pulse. The first assumes the encoder index pulse sets
  a flag when it occurs, but the count value doesn't change. The controller
  polls on this flag, and when it is seen to be latched the controller
  reads the position and uses this as an offset. There is a latency that
  makes this less accurate than the second method, in which the occurrence
  of the index pulse resets the count automatically. The problem is that
  the controller logic is different for the two methods, so if you replace
  a board which does it the first way with one that does it the second,
  you need to recode the logic. The function "vtiEncoderIndexModel()"
  returns the model used by the board, so at least you can detect it
  and if you have coded up both types you can switch automatically.

  EXT_ENCODER_INDEX_MODEL_MANUAL
  indicates that the index pulse sets a latch flag, but you have to
  read this and then the encoder value and handle offsets yourself.
  The board won't change its count value on the index pulse.

  EXT_ENCODER_INDEX_MODEL_AUTO
  indicates that the index pulse zeros the encoder count automatically.
  */

/* flags defined bit-exclusive for OR'ing if board can do multiple ways */
#define EXT_ENCODER_INDEX_MODEL_MANUAL 0x00000001
#define EXT_ENCODER_INDEX_MODEL_AUTO   0x00000002

unsigned int vtiEncoderIndexModel(void)
{
  return EXT_ENCODER_INDEX_MODEL_MANUAL;
}


/*
  vtiEncoderSetIndexModel(unsigned int model)

  For boards that support multiple index models, select which one
  is to be used. Returns 0 if OK, -1 if model can't be supported.
  */
int vtiEncoderSetIndexModel(unsigned int model)
{
  return -1;
}


/*
  vtiEncoderNum()

  returns number of encoders in system.
  */
int vtiEncoderNum(void)
{
#ifdef VTI_8_AXES
    return 8;
#else
    return 4;
#endif
}

/*
  vtiEncoderRead(int axis, double * counts)

  Stores the encoder's counts in counts arg. Returns 0 if
  OK or -1 if axis is out of range. axis is in range
  0 .. max axis - 1.
  */
int vtiEncoderRead(int axis, double * counts)
{
  return vtiEncoderReadAll(VTI_MAX_AXIS, counts);
}

/*
  vtiEncoderReadAll(int max, double * counts)

  Stores the range of encoders' counts in counts array. Returns 0 if
  OK or -1 if the max is greater than number of encoders. max is
  number of encoders, so for encoders 0..3 max would be 4.
  */
int vtiEncoderReadAll(int max, double * counts)
{
    int t;
    int status3 = ENC_STAT03;
    int status7 = ENC_STAT47;
    char overflow;
    char direction;
    LongWord Encdata;

    if ((max > VTI_MAX_AXIS) || (max < 0)) {
	max=VTI_MAX_AXIS;
    }

    overflow = (status7 << 4) | (status3 & 0x0f);
    direction = (status7 & 0x0f) | ((status3 & 0xf0) >> 4);

    for (t = 0; t < max; t++) {
	Encdata.Long = counts[t];
	if (status3 != ENC_STAT03) {
	    if ((overflow >> t) & 0x01) {
		if ((direction >> t) & 0x01) {
		    Encdata.Word[1] = Encdata.Word[1] + 1;
		} else {
		    Encdata.Word[1] = Encdata.Word[1] - 1;
		}
	    }
	}
#ifndef PCI
	Encdata.Word[0] = ENCODER(t);
#else
	if (t < 4 ) {
	    Encdata.Word[0] = encoder->Counter[t];
	} else {
#ifdef VTI_8_AXES
	    Encdata.Word[0] = ip->Counter[t-4];
#endif
	}
#endif
        counts[t] = Encdata.Long;
    }
    return 0;
}

/*
  vtiEncoderResetIndex(int axis)

  Resets index latching for the indicated axis. Returns 0 if OK or -1
  if the index is out of range. This applies to both
  EXT_ENCODER_INDEX_MODEL_MANUAL and EXT_ENCODER_INDEX_MODEL_AUTO.
  For the first, it resets the latch flag. For the second, it enables
  zeroing on the next index. encoder is range 0..max encoder - 1.
  */
int vtiEncoderResetIndex(int axis)
{
    return 0;
}

/*
  vtiEncoderReadLatch(int encoder, int * flag)

  For EXT_ENCODER_INDEX_MODEL_MANUAL, stores 1 if index has latched
  the flag, 0 if not. For EXT_ENCODER_INDEX_MODEL_AUTO, stores 1 if
  the encoder has been zeroed. Returns 0 if OK, -1 if not valid.
  */
int vtiEncoderReadLatch(int axis, int * flag)
{
    char Byte;
    Byte=(ENC_STAT03 >> 8) & 0x0f;
    Byte=Byte + ((ENC_STAT47 >> 4) & 0xf0);
    *flag=(Byte >> axis) & 0x01;

    return 0;
}

/*
  vtiEncoderReadLevel(int axis, int * index)

  For EXT_ENCODER_INDEX_MODEL_MANUAL, stores 1 if encoder is on
  the index pulse right now, 0 if not. Useful for slow polling
  to get more accuracy since the manual model has latency.

  Not valid for EXT_ENCODER_INDEX_MODEL_AUTO. Returns 0 if OK,
  -1 if not valid.
  */
int vtiEncoderReadLevel(int axis, int * flag)
{
    vtiEncoderReadLatch(axis, flag);
    return 0;
}

/* Limit switch functions */

/*
  vtiMaxLimitSwitchRead(int axis, int * flag)

  sets *flag to 0 if the limit switch is not tripped, i.e., everything
  is fine, 1 if the limit switch is tripped, i.e., the axis went
  too far in the associated direction.

  Maximum is defined as the direction in which encoder values increase,
  minimum is the other direction.

  Returns 0 if OK, -1 if not valid (axis is out of range).
  */
int vtiMaxLimitSwitchRead(int axis, int * flag)
{
    if (axis > VTI_MAX_AXIS || axis < 0 ) {
	return -1;
    }

    *flag = vtiDioRead(MAX_LIMIT_BASE_INDEX + axis, flag);
    return 0;
}

int vtiMinLimitSwitchRead(int axis, int * flag)
{
    if (axis > VTI_MAX_AXIS || axis < 0 ) {
    return -1;
    }

    *flag = vtiDioRead(MIN_LIMIT_BASE_INDEX + axis, flag);
    return 0;
}

/*
  vtiHomeSwitchRead(int axis, int * flag)

  sets *flag to 0 if the home switch is not tripped, i.e., everything
  is fine, 1 if the home switch is tripped.

  Returns 0 if OK, -1 if not valid (axis is out of range).
  */
int vtiHomeSwitchRead(int axis, int * flag)
{
    if (axis > VTI_MAX_AXIS || axis < 0 ) {
	return -1;
    }

    *flag = vtiDioRead(HOME_BASE_INDEX + axis, flag);
    return 0;
}

/* Amp functions */

/*
   vtiAmpEnable(int axis, int enable)

   enables or disables amplifier for indicated axis; enable flag is
   1 to enable, 0 to disable

   Returns 0 if OK, -1 if not valid (axis is out of range)
   */
int vtiAmpEnable(int axis, int enable)
{
    if (axis > VTI_MAX_AXIS || axis < 0 ) {
	return -1;
    }
    return vtiDioWrite(axis, enable);
}

/*
  vtiAmpFault(int axis, int * fault)

  Copies into 'fault' the fault state of the amplifier. 1 is faulted,
  0 is OK.

  Returns 0 if OK, -1 if not valid (axis out of range)
  */
int vtiAmpFault(int axis, int * fault)
{
    if (axis > VTI_MAX_AXIS || axis < 0 ) {
	return -1;
    }

    *fault = vtiDioRead(AMP_FAULT_BASE_INDEX + axis, fault);
    return 0;
}

/*
  Digital I/O model

  If you need to call board-specific code to set up the digital I/O
  registers, do it in vtiDioInit().

  "index" begins at 0.

  Code for each implementation should shift index up into appropriate
  register so that it matches initialization and R/W setup in vtiInit().

  Returns 0 if OK, -1 if error (invalid index).
  */

/* returns the max input index, output index; max bytes, shorts, and words */
int vtiDioMaxInputs(void)       /* index < this for vtiDioRead() */
{
    return 64;
}

int vtiDioMaxOutputs(void)      /* index < this for vtiDioWrite(),Check() */
{
    return 64;
}

/* reads value of digital input at index, stores in value */
int vtiDioRead(int index, int *value)
{
#ifndef PCI
    unsigned short int mask;
    mask = 0x01 << (index % 16);

    if (index >= 0 || index < 16) {
	*value = (DIG_RD_BK0 & mask ? 1 : 0);
	return 0;
    }
    if (index >= 16 || index < 32) {
	*value = (DIG_RD_BK1 & mask ? 1 : 0);
	return 0;
    }
    if (index >= 32 || index < 48) {
	*value = (DIG_RD_BK2 & mask ? 1 : 0);
	return 0;
    }
    if (index >= 48 || index < 64) {
	*value = (DIG_RD_BK3 & mask ? 1 : 0);
	return 0;
    }
/* Bad index value */
    return -1;
#endif
    return 0;
}

/* writes value (non-zero means 1, 0 is 0) at digital out at index */
int vtiDioWrite(int index, int value)
{
#ifndef PCI
    unsigned short int data;
    int mask;
    mask = (index % 16);

    if (index >= 0 || index < 16) {
/* Read the port and OR with the mask before writing it back */
	data = DIG_RD_BK0;
	if (value == 0) {
	    DIG_WR_BK0(data &= ~(1 << mask));
	} else {
	    DIG_WR_BK0(data |= (1 << mask));
	}
	return 0;
    }
    if (index >= 16 || index < 32) {
	data = DIG_RD_BK1;
	if (value == 0) {
	    DIG_WR_BK1(data &= ~(1 << mask));
	} else {
	    DIG_WR_BK1(data |= (1 << mask));
	}
	return 0;
    }
    if (index >= 32 || index < 48) {
	data = DIG_RD_BK2;
	if (value == 0) {
	    DIG_WR_BK2(data &= ~(1 << mask));
	} else {
	    DIG_WR_BK2(data |= (1 << mask));
	}
	return 0;
    }
    if (index >= 48 || index < 64) {
	data = DIG_RD_BK3;
	if (value == 0) {
	    DIG_WR_BK3(data &= ~(1 << mask));
	} else {
	  DIG_WR_BK3(data |= (1 << mask));
	}
	return 0;
    }
/* Bad index value */
    return -1;
#endif
    return 0;
}

/* reads value of digital OUT at index, stores in value. Useful
   for checking values of previous writes. Returns 0 if OK, -1 if
   bad index or can't read if they're write-only. */
int vtiDioCheck(int index, int *value)
{
    return vtiDioRead(index, value);
}

/* byte, short, and word reads, writes. Index starts at 0, indexes up
   through bytes, short ints, and ints */
int vtiDioByteRead(int index, unsigned char *byte)
{
#ifndef PCI
    switch(index) {
	case 0:
	    *byte = DIG_RD_BK0 & 0x00ff;
	    break;
        case 1:
	    *byte = (DIG_RD_BK0 >> 8) & 0x00ff;
	    break;
	case 2:
	    *byte = DIG_RD_BK1 & 0x00ff;
	    break;
	case 3:
    	    *byte = (DIG_RD_BK1 >> 8) & 0x00ff;
	    break;
	case 4:
    	    *byte = DIG_RD_BK2 & 0x00ff;
	    break;
	case 5:
    	    *byte = (DIG_RD_BK2 >> 8) & 0x00ff;
	    break;
	case 6:
    	    *byte = DIG_RD_BK3 & 0x00ff;
	    break;
	case 7:
	    *byte = (DIG_RD_BK3 >> 8) & 0x00ff;
	    break;
    }
#endif
    return 0;
}
int vtiDioShortRead(int index, unsigned short *sh)
{
#ifndef PCI
    switch(index) {
	case 0:
    	    *sh = DIG_RD_BK0;
	    break;
	case 1:
    	    *sh = DIG_RD_BK1;
	    break;
	case 2:
    	    *sh = DIG_RD_BK2;
	    break;
	case 3:
    	    *sh = DIG_RD_BK3;
    }
#endif
    return 0;
}

int vtiDioWordRead(int index, unsigned int *word)
{
#ifndef PCI
    switch(index) {
	case 0:
    	    *word = DIG_RD_BK0;
	    *word += DIG_RD_BK1 << 16;
	    break;
	case 1:
    	    *word = DIG_RD_BK2;
	    *word += DIG_RD_BK3 << 16;
	    break;
    }
#endif
    return 0;
}

int vtiDioByteWrite(int index, unsigned char byte)
{
#ifndef PCI
    switch(index) {
	case 0:
    	    DIG_WR_BK0((DIG_RD_BK0 & 0xff00) | byte);
	    break;
	case 1:
    	    DIG_WR_BK0((DIG_RD_BK0 & 0x00ff) | (byte << 8));
	    break;
	case 2:
    	    DIG_WR_BK1((DIG_RD_BK1 & 0xff00) | byte);
	    break;
	case 3:
    	    DIG_WR_BK1((DIG_RD_BK1 & 0x00ff) | (byte << 8));
	    break;
	case 4:
    	    DIG_WR_BK2((DIG_RD_BK2 & 0xff00) | byte);
	    break;
	case 5:
    	    DIG_WR_BK2((DIG_RD_BK2 & 0x00ff) | (byte << 8));
	    break;
	case 6:
    	    DIG_WR_BK3((DIG_RD_BK3 & 0xff00) | byte);
	    break;
	case 7:
    	    DIG_WR_BK3((DIG_RD_BK3 & 0x00ff) | (byte << 8));
	    break;
    }
#endif
    return 0;
}

int vtiDioShortWrite(int index, unsigned short sh)
{
#ifndef PCI
    switch(index) {
	case 0:
    	    DIG_WR_BK0(sh);
	    break;
	case 1:
    	    DIG_WR_BK1(sh);
	    break;
	case 2:
    	    DIG_WR_BK2(sh);
	    break;
	case 3:
    	    DIG_WR_BK3(sh);
    }
#endif
    return 0;
}

int vtiDioWordWrite(int index, unsigned int word)
{
#ifndef PCI
    switch(index) {
	case 0:
    	    DIG_WR_BK0(word & 0xffff);
	    DIG_WR_BK1((word >> 16) & 0xffff);
	    break;
	case 1:
    	    DIG_WR_BK2(word & 0xffff);
	    DIG_WR_BK3((word >> 16) & 0xffff);
	    break;
    }
#endif
    return 0;
}

int vtiDioByteCheck(int index, unsigned char *byte)
{
  return 0;
}

int vtiDioShortCheck(int index, unsigned short *sh){
  return 0;
}

int vtiDioWordCheck(int index, unsigned int *word)
{
  return 0;
}

/* writes a byte of IO synched with motion start/end */
int vtiMotDout(unsigned char byte)
{
  return 0;
}

/* Internal functions */

void RawDacOut(int axis, double volts)
{
    unsigned short int RawVolts;	/* Limit resolution to 16 bits */

/* Convert float volts to unsigned hex
    -10V = 0x0000
      0V = 0x0800
    +10V = 0x0fff */
    if (volts > 10) {	/* Limit the range for DAC output */
	volts = 10;
    }
    if (volts < -10) {
      volts = -10;
     }
    RawVolts = ((volts/10) * 0x07ff) + 0x0800; /* and convert to Hex value */
    switch(axis) {
	case 0:
    	    DAC_CHAN0(RawVolts);
	    break;
        case 1:
    	    DAC_CHAN1(RawVolts);
	    break;
        case 2:
    	    DAC_CHAN2(RawVolts);
	    break;
	case 3:
    	    DAC_CHAN3(RawVolts);
	    break;
	case 4:
    	    DAC_CHAN4(RawVolts);
	    break;
	case 5:
    	    DAC_CHAN5(RawVolts);
	    break;
	case 6:
    	    DAC_CHAN6(RawVolts);
	    break;
	case 7:
    	    DAC_CHAN7(RawVolts);
	    break;
    }
    return;
}
