#ifndef VTI_H
#define VTI_H

/*
  vti.h

  Declarations for Vigilant Products cards
  
  Modification history:

  1 July 2002 P.C. Added structures and support for the PCI card.
  24 March 2002 P.C. Created from newstg.h and VTI's isatest.cpp sources.
*/

/* vedor and device ID for the PCI card */
#define VENDOR 0x10b5
#define DEVICE 0x9050

/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif

static char __attribute__((unused)) vti_h[] = "$Id: vti.h,v 1.7 2004/03/11 16:14:16 paul_c Exp $";

#ifdef __cplusplus
extern "C" {
#endif

/*
 Number of axes in system defaults to 4. If VTI_8_AXES is defined
 by compiler directive or other means, code will support 8 axes,
 e.g., -DVTI_8_AXES. With 48 lines of digital IO, there should be
 sufficient to support all 8 axes.
 */
#if defined(PCI) && defined(VTI_8_AXIS)
#undef VTI_8_AXES  // 8 axis won't work on the PCI card (yet)...
#endif
#if defined(VTI_8_AXES) && !defined(PCI)
#define VTI_MAX_AXIS EMCMOT_MAX_AXIS
#else
#define VTI_MAX_AXIS 4
#endif
#include "emcmotcfg.h"

/*  Base address defaults to 0x300 and is only used by the ISA card.
    The PCI card automatically detects the base addresses via the pci
    service routines within the kernel. */
#define DEFAULT_IO_BASE_ADDRESS 0x0300
extern unsigned short IO_BASE_ADDRESS;

#define DEFAULT_VTI_IRQ 5
extern int VTI_IRQ;

#ifndef PCI
// =========Register Offsets============
#define   ENCODER(x)       inw(IO_BASE_ADDRESS+(x*2))
#define   ENC_COUNT0       inw(IO_BASE_ADDRESS+0x0)
#define   ENC_COUNT1       inw(IO_BASE_ADDRESS+0x2)
#define   ENC_COUNT2       inw(IO_BASE_ADDRESS+0x4)
#define   ENC_COUNT3       inw(IO_BASE_ADDRESS+0x6)
#define   ENC_COUNT4       inw(IO_BASE_ADDRESS+0x8)
#define   ENC_COUNT5       inw(IO_BASE_ADDRESS+0xa)
#define   ENC_COUNT6       inw(IO_BASE_ADDRESS+0xc)
#define   ENC_COUNT7       inw(IO_BASE_ADDRESS+0xe)
#define   ENC_RESET        outw(0,IO_BASE_ADDRESS+0x10)
#define   ENC_STAT03       inw(IO_BASE_ADDRESS+0x12)
#define   ENC_STAT47       inw(IO_BASE_ADDRESS+0x14)
#define   ENC_MASK03(x)    outw(x,IO_BASE_ADDRESS+0x12)
#define   ENC_MASK47(x)    outw(x,IO_BASE_ADDRESS+0x14)
#define   INT_STATUS       inw(IO_BASE_ADDRESS+0x16)
#define   INT_DIGCFG(x)    outw(x,IO_BASE_ADDRESS+0x16)
#define   DIG_RD_BK0       inw(IO_BASE_ADDRESS+0x18)
#define   DIG_RD_BK1       inw(IO_BASE_ADDRESS+0x1a)
#define   DIG_RD_BK2       inw(IO_BASE_ADDRESS+0x1c)
#define   DIG_RD_BK3       inw(IO_BASE_ADDRESS+0x1e)
#define   DIG_WR_BK0(x)    outw(x,IO_BASE_ADDRESS+0x18)
#define   DIG_WR_BK1(x)    outw(x,IO_BASE_ADDRESS+0x1a)
#define   DIG_WR_BK2(x)    outw(x,IO_BASE_ADDRESS+0x1c)
#define   DIG_WR_BK3(x)    outw(x,IO_BASE_ADDRESS+0x1e)
#define   DAC(x,y)         outw(x,IO_BASE_ADDRESS+0x20+(y*2))
#define   DAC_CHAN0(x)     outw(x,IO_BASE_ADDRESS+0x20)
#define   DAC_CHAN1(x)     outw(x,IO_BASE_ADDRESS+0x22)
#define   DAC_CHAN2(x)     outw(x,IO_BASE_ADDRESS+0x24)
#define   DAC_CHAN3(x)     outw(x,IO_BASE_ADDRESS+0x26)
#define   DAC_CHAN4(x)     outw(x,IO_BASE_ADDRESS+0x28)
#define   DAC_CHAN5(x)     outw(x,IO_BASE_ADDRESS+0x2a)
#define   DAC_CHAN6(x)     outw(x,IO_BASE_ADDRESS+0x2c)
#define   DAC_CHAN7(x)     outw(x,IO_BASE_ADDRESS+0x2e)
#define   TMRD_CNTR0       inb(IO_BASE_ADDRESS+0x30)
#define   TMRD_CNTR1       inb(IO_BASE_ADDRESS+0x32)
#define   TMRD_CNTR2       inb(IO_BASE_ADDRESS+0x34)
#define   TMWR_CNTR0(x)    outb(IO_BASE_ADDRESS+0x30)
#define   TMWR_CNTR1(x)    outb(IO_BASE_ADDRESS+0x32)
#define   TMWR_CNTR2(x)    outb(IO_BASE_ADDRESS+0x34)
#define   TIM_CNCTL(x)     outb(IO_BASE_ADDRESS+0x36)
#define   DAC_ENABLE       outb(0x5a,IO_BASE_ADDRESS+0x38)
#define   DAC_DISARM       outb(0xa5,IO_BASE_ADDRESS+0x38)
#define   DAC_SETIND       inb(IO_BASE_ADDRESS+0x3a)
#define   DAC_SETSIM       outb(0,IO_BASE_ADDRESS+0x3a)
#define   DAC_UPDATE       outb(0,IO_BASE_ADDRESS+0x3c)
#define   LOGIC_CFG        outb(0,IO_BASE_ADDRESS+0x3e)

// =========Interrupt definitions=======
#define   IRQ_10_VECTOR  0x72
#define   IRQ_11_VECTOR  0x73
#define   IRQ_12_VECTOR  0x74
#define   IRQ_15_VECTOR  0x77
#define   P8259_A_0      0x20
#define   P8259_A_1      0x21
#define   P8259_B_0      0xA0
#define   P8259_B_1      0xA1
#define   END_OF_INT     0x20  //Nonspecific EOI command
#endif

/* The mask for setting up the direction of the digital IO
   Consult the ISA-EncoderDAC manual for the format.
*/
#define DIO_CFG_MASK 0x0010

/* The switch and Amp pins are arranged in 8 bit groups - Makes
   life sooo easy for returning the pin status.
   Index range     Group function
    0 - 7           Amp Enable (output)
    8 - 15          Amp Fault (input)
   16 - 23          + Limit Switches (input)
   24 - 31          - Limit Switches (input)
   32 - 39          Home Switches (input)
   40 - 47          Spare Inputs
   48 - 55          Spare Inputs
   56 - 64          Spare Inputs
*/
#define AMP_FAULT_BASE_INDEX 8
#define MAX_LIMIT_BASE_INDEX 16
#define MIN_LIMIT_BASE_INDEX 24
#define HOME_BASE_INDEX 32
/*
Very few machines will use six axes, let alone all eight - Unused pins can be
used for general inputs. It is not recommended that the spare Amp outputs are
used by any other process - A race condition will result, producing unpredictable
results. */

/* decls for external interface */

extern int vtiMotInit(const char * stuff);
extern int vtiMotQuit(void);
extern int vtiBaseFind(void);

extern int vtiDacNum(void);
extern int vtiDacWrite(int axis, double volts);
extern int vtiDacWriteAll(int max, double * volts);

extern int vtiEncoderSetIndexModel(unsigned int model);
extern int vtiEncoderNum(void);
extern int vtiEncoderRead(int axis, double * counts);
extern int vtiEncoderReadAll(int max, double * counts);
extern int vtiEncoderResetIndex(int axis);
extern int vtiEncoderReadLatch(int axis, int * flag);
extern int vtiEncoderReadLevel(int axis, int * flag);

extern int vtiMaxLimitSwitchRead(int axis, int * flag);
extern int vtiMinLimitSwitchRead(int axis, int * flag);

extern int vtiHomeSwitchRead(int axis, int * flag);

extern int vtiAmpEnable(int axis, int enable);
extern int vtiAmpFault(int axis, int * fault);

extern int vtiDioInit(const char * stuff);
extern int vtiDioQuit(void);

extern int vtiDioMaxInputs(void);
extern int vtiDioMaxOutputs(void);
extern int vtiDioRead(int index, int *value);
extern int vtiDioWrite(int index, int value);
extern int vtiDioCheck(int index, int *value);
extern int vtiDioByteRead(int index, unsigned char *byte);
extern int vtiDioShortRead(int index, unsigned short *sh);
extern int vtiDioWordRead(int index, unsigned int *word);
extern int vtiDioByteWrite(int index, unsigned char byte);
extern int vtiDioShortWrite(int index, unsigned short sh);
extern int vtiDioWordWrite(int index, unsigned int word);
extern int vtiDioByteCheck(int index, unsigned char *byte);
extern int vtiDioShortCheck(int index, unsigned short *sh);
extern int vtiDioWordCheck(int index, unsigned int *word);

extern int vtiAioInit(const char * stuff);
extern int vtiAioQuit(void);

extern int vtiAioMaxInputs(void);
extern int vtiAioMaxOutputs(void);
extern int vtiAioStart(int index);
extern void vtiAioWait(void);
extern int vtiAioRead(int index, double *volts);
extern int vtiAioWrite(int index, double volts);
extern int vtiAioCheck(int index, double *volts);
extern int vtiModel(void);

#ifdef PCI

#define   ENCODER(x)	encoder->Counter[x]
#define   ENC_COUNT0	encoder->Counter[0]
#define   ENC_COUNT1	encoder->Counter[1]
#define   ENC_COUNT2	encoder->Counter[2]
#define   ENC_COUNT3	encoder->Counter[3]
#define   ENC_COUNT4
#define   ENC_COUNT5
#define   ENC_COUNT6
#define   ENC_COUNT7
#define   ENC_RESET	encoder->Reset = 0
#define   ENC_STAT03	encoder->Status
#define   ENC_STAT47	0
#define   ENC_MASK03(x)	encoder->Status=x
#define   ENC_MASK47(x)
#define   INT_STATUS
#define   INT_DIGCFG(x)
#define   DIG_RD_BK0
#define   DIG_RD_BK1
#define   DIG_RD_BK2
#define   DIG_RD_BK3
#define   DIG_WR_BK0(x)
#define   DIG_WR_BK1(x)
#define   DIG_WR_BK2(x)
#define   DIG_WR_BK3(x)

#define   DAC_CHAN0(x)     dac->dac[0]=(x)
#define   DAC_CHAN1(x)     dac->dac[1]=(x)
#define   DAC_CHAN2(x)     dac->dac[2]=(x)
#define   DAC_CHAN3(x)     dac->dac[3]=(x)
#define   DAC_CHAN4(x)
#define   DAC_CHAN5(x)
#define   DAC_CHAN6(x)
#define   DAC_CHAN7(x)
#define   TMRD_CNTR0
#define   TMRD_CNTR1
#define   TMRD_CNTR2
#define   TMWR_CNTR0(x)
#define   TMWR_CNTR1(x)
#define   TMWR_CNTR2(x)
#define   TIM_CNCTL(x)
#define   DAC_ENABLE       encoder->DAC = 0x5a
#define   DAC_DISARM       encoder->DAC = 0xa5
#define   DAC_SETIND       dac->mode
#define   DAC_SETSIM       dac->mode = 0
#define   DAC_UPDATE       dac->update = 0
#define   LOGIC_CFG

/* Structures used by the PCI card */
struct encoder {
    u16 Counter[4];
    u8 Status;		// 0x08
    u8 dummy;		// dummy space
    u16 Reset;		// 0x0a
    u8 reserved[3];	// dummy space 0x0c - 0x0f
    u16 Interrupt;	// 0x10
    u16 DAC;		// 0x12
    u16 DIO;		// 0x14
    };

struct timer {
    u8 reserved[0x2c];
    u8 ctc0;		// 8254 Counter 0
    u8 ctc1;		// 8254 Counter 1
    u8 ctc2;		// 8254 Counter 2
    u8 control;		// 8254 command register
    };

struct dac {
    u8 reserved[0x13f]; // dummy space
    u16 dac[4];
    u16 dac_adj[4];
    u16 mode;
    u16 update;
    u16 reset;
    u8 dummy[0x2a];	// dummy space
    u16 DIO[0x10];	// Digital IO channels
    u16 config0;	// Direction configs for channels 00-63
    u16 config1;	// Direction configs for channels 64-127
    u16 status;
    };

/* IndustryPack mappings - Need to check IF u8 or u16, also check boundary address*/
struct ip {
    u8 ID[0x40];	// IndustryPack ID - The first (12*2) addresses used for ID info
			// consult P20 of vip-4encdac.pdf for more info.
    u16 IP_int;		// Interrupt status
    u8 reserved[0x3e];	// dummy space
    u16 IO[0x30];	// IndustryPack IO - Only need the first 0x30 address.
			// Again, consult manual - Might it be better to use another struct here ??
			// and also for the ID stuff....
    };

#endif

/* Internal functions */
extern void RawDacOut(int DAC, double volts);

typedef union
{
  signed long int	Long;
  signed short int	Word[2];
} LongWord;

#ifdef __cplusplus
}
#endif

#endif /* VTI_H */

