#ifndef VITAL_H
#define VITAL_H

/*
  vital.h

  $Id: vital.h,v 1.1.1.1 2005/05/31 11:12:19 paul Exp $

  Declarations for Vital Systems cards

  Modification history:

  29-March-2004 P.C. Modified the PCI detection routines prior to committing.
    Moved the majority of the defines into the header.
  3-March-2004 AAR Adapted the Vigilant sources to Vital Systems Motenc100
    pci card.

*/

#ifdef __cplusplus
extern "C" {
#endif


// Number of axes in system 
#define VITAL_MAX_AXIS 8
#include "emcmotcfg.h"

// Device and Vendor ID
#define VENDOR 0x10b5
#define DEVICE 0x3001
#define PCI_MEM_LEN 512

#define MOTENC_PLD2_BASE	8
#define MOTENC_DAC_BASE		24

// digital input bit assignments

// Connector J6
#define LIM_HI_0	0
#define LIM_LO_0	1
#define HOME_0		2
#define FAULT_0		3

#define LIM_HI_1	4
#define LIM_LO_1	5
#define HOME_1		6
#define FAULT_1		7

#define LIM_HI_2	8
#define LIM_LO_2	9
#define HOME_2		10
#define FAULT_2		11

#define LIM_HI_3	12
#define LIM_LO_3	13
#define HOME_3		14
#define FAULT_3		15

// Connector J7
#define LIM_HI_4	16
#define LIM_LO_4	17
#define HOME_4		18
#define FAULT_4		19

#define LIM_HI_5	20
#define LIM_LO_5	21
#define HOME_5		22
#define FAULT_5		23

#define LIM_HI_6	24
#define LIM_LO_6	25
#define HOME_6		26
#define FAULT_6		27

#define LIM_HI_7	28
#define LIM_LO_7	29
#define HOME_7		30
#define FAULT_7		31

// Output Bits Assignment
// Connector J6
#define AMP_EN_0	0
#define AMP_EN_1	1
#define AMP_EN_2	2
#define AMP_EN_3	3

// Connector J7
#define AMP_EN_4	8
#define AMP_EN_5	9
#define AMP_EN_6	10
#define AMP_EN_7	11


/* decls for external interface */

extern int vitalMotInit(const char * stuff);
extern int vitalMotQuit(void);

extern int vitalDacNum(void);
extern int vitalDacWrite(int axis, double volts);
extern int vitalDacWriteAll(int max, double * volts);

extern int vitalEncoderSetIndexModel(unsigned int model);
extern int vitalEncoderNum(void);
extern int vitalEncoderRead(int axis, double * counts);
extern int vitalEncoderReadAll(int max, double * counts);
extern int vitalEncoderResetIndex(int axis);
extern int vitalEncoderReadLatch(int axis, int * flag);
extern int vitalEncoderReadLevel(int axis, int * flag);

extern int vitalMaxLimitSwitchRead(int axis, int * flag);
extern int vitalMinLimitSwitchRead(int axis, int * flag);

extern int vitalHomeSwitchRead(int axis, int * flag);

extern int vitalAmpEnable(int axis, int enable);
extern int vitalAmpFault(int axis, int * fault);

extern int vitalDioInit(const char * stuff);
extern int vitalDioQuit(void);

extern int vitalDioMaxInputs(void);
extern int vitalDioMaxOutputs(void);
extern int vitalDioRead(int index, int *value);
extern int vitalDioWrite(int index, int value);
extern int vitalDioCheck(int index, int *value);

extern int vitalAioInit(const char * stuff);
extern int vitalAioQuit(void);

extern int vitalAioMaxInputs(void);
extern int vitalAioMaxOutputs(void);
extern int vitalAioStart(int index);
extern int vitalAioReady(void);
extern int vitalAioRead(int index, double *volts);
extern int vitalAioWrite(int index, double volts);
extern int vitalAioCheck(int index, double *volts);
extern char *vitalModel(void);


#ifdef __cplusplus
}
#endif

#endif /* VITAL_H */

