/********************************************************************
*
* Description: emcioctl.h
*
*       ioctl definitions for interfacing with the emc kernel module.
*
*    
* Author: Paul Corner
* License: GPL Version 2
* System: Linux
*    
* Last change: 
* $Revision: 1.2 $
* $Author: paul $
* $Date: 2005/08/08 22:56:39 $
*
* Copyright (c) 2004  All rights reserved.
*
********************************************************************/
#ifndef EMCIOCTL_H

#define EMC_IOC_MAGIC 'p'

/* General notes on usage. Data is passed to the realtime code
   via pointers. Any reads will have the results returned via
   the same pointer.
*/
/* DIO_READ returns a non zero result if the indexed bit is set,
   zero if it is not. The function call will return -EFAULT
   if the index is out of range. */
#define EMC_IOW_DIO_READ        _IOWR(EMC_IOC_MAGIC, 1, int*)

/* DIO_SET sets the indexed bit. The function returns -EFAULT
   if the index is out of range. */
#define EMC_IOW_DIO_SET         _IOW(EMC_IOC_MAGIC, 2, int*)

/* DIO_CLEAR resets the indexed bit. The function returns -EFAULT
   if the index is out of range. */
#define EMC_IOW_DIO_CLEAR       _IOW(EMC_IOC_MAGIC, 3, int*)

/* AIO_READ reads index via the pointer and writes the
   value to the same location. Returns -EFAULT if index is out
   of range or -ENOSYS if no DACs installed.
   Data is a signed 32 bit int scaled so that 2^31 = +10V
   and -2^31 = -10V */
#define EMC_IOW_AIO_READ        _IOWR(EMC_IOC_MAGIC, 4, int*)

/* AIO_ WRITE takes an array {index,value} and writes the
   value to the indexed DAC. Returns -EFAULT if index is out
   of range or -ENOSYS if no DACs installed.
   Data is a signed 32 bit int scaled so that 2^31 = +10V
   and -2^31 = -10V */
#define EMC_IOW_AIO_WRITE       _IOW(EMC_IOC_MAGIC, 5, int*)

/* AIO_READY tests to see if the ADC has completed a conversion
   returns 0 if data is ready, or -EBUSY if conversion is still
   in progress. */
#define EMC_IOW_AIO_READY       _IOW(EMC_IOC_MAGIC, 6, int*)

/* Returns a string describing the hardware being used by the
   realtime code. char* must point to a buffer at least LINELEN
   bytes in length. No error checks are made, so buffer over-
   runs are probable if the buffer is smaller. */
#define EMC_IOR_MODEL           _IOR(EMC_IOC_MAGIC, 7, char*)

#endif
