/********************************************************************
*
* Description: emcmodule.c
*
*	Module load/unload, init/cleanup routines.
*	Much of this can be simplified and shared memory requiremets
*	reduced by removing logging and debug data - Config data
*	should be in kernel space and not shared. But we do need to
*	stick with the RMA model in principle.
*
*    
* Author:
* License:
* System: Linux
*    
* Last change: 
* $Date: 2005/08/12 11:37:42 $
* Matt Timmermans - moved setup of real-time timer into other quickstep,
*                   freqmod, and emcservo.
* 
* $Revision: 1.2 $
* $Author: paul $
* $Date: 2005/08/12 11:37:42 $
*
* Copyright (c) 2004  All rights reserved.
*
********************************************************************/

#ifndef __KERNEL__
#define __KERNEL__
#endif

#ifndef MODULE
#define MODULE
#endif
#ifndef EXPORT_SYMTAB
#define EXPORT_SYMTAB
#endif

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>

#include "emcprocfs.h"

#include "rtai.h"
#include "rtai_sched.h"
#include "rtai_shm.h"

#include "posemath.h"		/* PmPose, pmCartMag() */
#include "emcpos.h"
#include "emcmotcfg.h"
#include "emcmotglb.h"
#include "emcmot.h"
#include "pid.h"
#include "cubic.h"
#include "trajplan.h"
#include "extintf.h"
#include "mmxavg.h"
#include "emcmotlog.h"

/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif

#include "diagnostics.h"

static char __attribute__ ((unused)) ident[] =
    "$Id: emcmodule.c,v 1.2 2005/08/12 11:37:42 paul Exp $";

extern int realtime_startup(int servoCycleTicks);
extern void realtime_shutdown(void);

/* need to point to command, status, error, and log */
EMCMOT_STRUCT *emcmotStruct;

/* ptrs to either buffered copies or direct memory for
   command and status */
EMCMOT_COMMAND *emcmotCommand;
EMCMOT_STATUS *emcmotStatus;
EMCMOT_CONFIG *emcmotConfig;
EMCMOT_DEBUG *emcmotDebug;
EMCMOT_ERROR *emcmotError;	/* unused for RT_FIFO */
EMCMOT_LOG *emcmotLog;		/* unused for RT_FIFO */
EMCMOT_IO *emcmotIo;		/* new struct added 8/21/2001 JME */
EMCMOT_COMP *emcmotComp[EMCMOT_MAX_AXIS];	/* unused for RT_FIFO */
EMCMOT_LOG_STRUCT ls;
EMCMOT_DEBUG localEmcmotDebug;

//int DRO_BASE_ADDRESS;

/* MT: this will cause a compiler error if the size of this structure changes -- helps
 * ensure compatability with older user-space code
 * If it compiles, it evaluates to 0.
 * (Look how big that struct is!)
 */
int DEBUG_EMCMOT = ((1 / (sizeof(EMCMOT_STRUCT) == 688592)) & 7) / 8;

extern int IO_BASE_ADDRESS;
extern int FIND_IO_BASE_ADDRESS;
extern int BLEND_TOL_REL_NS;

#define DEFAULT_EMCMOT_TASK_PRIORITY 2	/* the highest is 1, the lowest is  RT_LOWEST_PRIORITY */
#define DEFAULT_EMCMOT_TASK_STACK_SIZE 12288
static int EMCMOT_TASK_PRIORITY = DEFAULT_EMCMOT_TASK_PRIORITY;
static int EMCMOT_TASK_STACK_SIZE = DEFAULT_EMCMOT_TASK_STACK_SIZE;

/* task struct */
RT_TASK emcmotTask;

static int emcmotTask_created = 0;
static int freqTask_created = 0;
int emcmotTask_started = 0;

int iperiod;
RTIME iperiod_rtime;

/* the type of kinematics, from emcmot.h */
int kinType = 0;


/* FIXME */
int EMCMOT_NO_FORWARD_KINEMATICS = 0;
/* FIXME-- testing */
int debouncecount[EMCMOT_MAX_AXIS] = { 0 };

/* register symbols to be modified by insmod
   see "Linux Device Drivers", Alessandro Rubini, p. 385
   (p.42-44 in 2nd edition) */

int IGNORE_REALTIME_ERRORS = 0;
module_param(IGNORE_REALTIME_ERRORS, int, S_IRUGO);
MODULE_PARM_DESC(IGNORE_REALTIME_ERRORS, "No description");

module_param(DEBUG_EMCMOT, int, S_IRUGO);
MODULE_PARM_DESC(DEBUG_EMCMOT, "Extended debug info");

module_param(BLEND_TOL_REL_NS, int, S_IRUGO);
MODULE_PARM_DESC(BLEND_TOL_REL_NS,
		 "Default blending error limit, in ns of feed");

//MODULE_PARM(SHMEM_BASE_ADDRESS, "l");
//MODULE_PARM_DEC(SHMEM_BASE_ADDRESS, "Base address for shared memory");

module_param(SHMEM_KEY, int, S_IRUGO);
MODULE_PARM_DESC(SHMEM_KEY, "Shared memory key");

module_param(IO_BASE_ADDRESS, int, S_IRUGO);
MODULE_PARM_DESC(IO_BASE_ADDRESS, "Base address for the IO ports");

module_param(FIND_IO_BASE_ADDRESS, int, S_IRUGO);
//MODULE_PARM_DESC(FIND_IO_BASE_ADDRESS, "No description");

module_param(EMCMOT_TASK_PRIORITY, int, S_IRUGO);
MODULE_PARM_DESC(EMCMOT_TASK_PRIORITY, "No description");

module_param(EMCMOT_TASK_STACK_SIZE, int, S_IRUGO);
MODULE_PARM_DESC(EMCMOT_TASK_STACK_SIZE, "No description");

module_param(EMCMOT_NO_FORWARD_KINEMATICS, int, S_IRUGO);
MODULE_PARM_DESC(EMCMOT_NO_FORWARD_KINEMATICS, "No description");

#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL v2");
#endif

void emcmotsig(void)
{
}

int module_error_code;

int init_module(void)
{
    int axis;
    int t;
    int servoCycleTicks;
    RTIME now;
    PID_STRUCT pid;

    /* Init the procfs */
    if (-1 == proc_init()) {
	diagnostics("Can not initialize the procfs\n");
	goto cleanup_proc;
    }
//  diagnostics("emcmot: SHMEM_BASE_ADDRESS = %ld\n", SHMEM_BASE_ADDRESS);
    diagnostics("emcmot: SHMEM_KEY = %d\n", SHMEM_KEY);
    diagnostics("emcmot: IO_BASE_ADDRESS = %X\n", IO_BASE_ADDRESS);
    diagnostics("emcmot: EMCMOT_TASK_PRIORITY = %d\n",
		EMCMOT_TASK_PRIORITY);
    diagnostics("emcmot: EMCMOT_TASK_STACK_SIZE = %d\n",
		EMCMOT_TASK_STACK_SIZE);
    diagnostics("emcmot: sizeof(RT_TASK) = %d\n", sizeof(RT_TASK));

    diagnostics("emcmot: sizeof(EMCMOT_STRUCT) = %d\n",
		sizeof(EMCMOT_STRUCT));
    diagnostics("emcmot: default blend tolerance = %d ns*feed\n",
		BLEND_TOL_REL_NS);

    emcmotTask_created = 0;
    freqTask_created = 0;
    emcmotStruct = 0;
    emcmotDebug = 0;
    emcmotStatus = 0;
    emcmotCommand = 0;
    emcmotIo = 0;
    emcmotConfig = 0;

    /* record the kinematics type of the machine */
    kinType = kinematicsType();
    if (EMCMOT_NO_FORWARD_KINEMATICS) {
	kinType = KINEMATICS_INVERSE_ONLY;
    }

    emcmotStruct = rtai_kmalloc(SHMEM_KEY, sizeof(EMCMOT_STRUCT));
    if (0 == emcmotStruct) {
	diagnostics
	    ("emcmot: can't allocate shared memory with rtai_kmalloc(%d,%d)\n.",
	     SHMEM_KEY, sizeof(EMCMOT_STRUCT));
	goto cleanup_proc;
    }
    diagnostics
	("emcmot: emcmotStruct allocated at 0x%X from rtai_kmalloc(%d,%d)\n",
	 ((unsigned int) emcmotStruct), SHMEM_KEY, sizeof(EMCMOT_STRUCT));


    /* zero shared memory */
    for (t = 0; t < sizeof(EMCMOT_STRUCT); t++) {
	((char *) emcmotStruct)[t] = 0;
    }
    /* we'll reference emcmotStruct directly */
    emcmotCommand = (EMCMOT_COMMAND *) & emcmotStruct->command;
    emcmotStatus = (EMCMOT_STATUS *) & emcmotStruct->status;
    emcmotConfig = (EMCMOT_CONFIG *) & emcmotStruct->config;
    if (DEBUG_EMCMOT) {
	emcmotDebug = (EMCMOT_DEBUG *) & emcmotStruct->debug;
    } else {
	emcmotDebug = &localEmcmotDebug;
    }

    emcmotError = (EMCMOT_ERROR *) & emcmotStruct->error;
    emcmotIo = (EMCMOT_IO *) & emcmotStruct->io;	/* set address of struct JE 8/21/2001 */
    emcmotLog = (EMCMOT_LOG *) & emcmotStruct->log;

    /* init error struct */
    emcmotErrorInit(emcmotError);

    SET_MOTION_ERROR_FLAG(0);
    SET_MOTION_COORD_FLAG(0);
    SET_MOTION_TELEOP_FLAG(0);
    emcmotConfig->numAxes = EMCMOT_MAX_AXIS;
    emcmotConfig->trajCycleTime = TRAJ_CYCLE_TIME;
    emcmotConfig->servoCycleTime = SERVO_CYCLE_TIME;
    emcmotStatus->vel = VELOCITY;
    emcmotConfig->limitVel = VELOCITY;
    emcmotStatus->acc = ACCELERATION;
    emcmotStatus->qVscale = 1.0;
    SET_MOTION_INPOS_FLAG(1);
    SET_MOTION_ENABLE_FLAG(0);
    emcmotConfig->kinematics_type = kinType;

    emcmotDebug->oldPos = emcmotStatus->pos;

    MARK_EMCMOT_CONFIG_CHANGE();

/* FIXME - Pointless setting values here as they could be totally wrong
   and won't be correct until the ini file has been parsed */
    /* init the axis components */

/* Load some default values to init the PID loop with. */
    pid.p = P_GAIN;
    pid.i = I_GAIN;
    pid.d = D_GAIN;
    pid.ff0 = FF0_GAIN;
    pid.ff1 = FF1_GAIN;
    pid.ff2 = FF2_GAIN;
    pid.backlash = BACKLASH;
    pid.bias = BIAS;
    pid.maxError = MAX_ERROR;

/* FIXME - Pointless setting values here as they could be totally wrong
   and won't be correct until the ini file has been parsed */
    for (axis = 0; axis < EMCMOT_MAX_AXIS; axis++) {

/* FIXME - MUST set the emcmotComp[] array with real values. So this
   is about the only thing that needs to be done after the memory has
   been zero'd */
	emcmotComp[axis] = (EMCMOT_COMP *) & emcmotStruct->comp[axis];
	emcmotDebug->bcomp[axis] = 0;	/* backlash comp value */
	emcmotDebug->bcompdir[axis] = 0;	/* 0=none, 1=pos, -1=neg */
	emcmotDebug->bcompincr[axis] = 0;	/* incremental backlash comp */
	emcmotDebug->bac_done[axis] = 0;
	emcmotDebug->bac_d[axis] = 0;
	emcmotDebug->bac_di[axis] = 0;
	emcmotDebug->bac_D[axis] = 0;
	emcmotDebug->bac_halfD[axis] = 0;
	emcmotDebug->bac_incrincr[axis] = 0;
	emcmotDebug->bac_incr[axis] = 0;

	emcmotConfig->homingVel[axis] = VELOCITY;
	emcmotConfig->maxLimit[axis] = MAX_LIMIT;
	emcmotConfig->minLimit[axis] = MIN_LIMIT;
	emcmotConfig->maxOutput[axis] = MAX_OUTPUT;
	emcmotConfig->minOutput[axis] = MIN_OUTPUT;
	emcmotConfig->maxFerror[axis] = MAX_FERROR;
	emcmotStatus->outputScale[axis] = OUTPUT_SCALE;
	emcmotStatus->outputOffset[axis] = OUTPUT_OFFSET;
	emcmotStatus->inputScale[axis] = INPUT_SCALE;
	emcmotDebug->inverseInputScale[axis] = 1.0 / INPUT_SCALE;
	emcmotStatus->inputOffset[axis] = INPUT_OFFSET;
	emcmotDebug->inverseOutputScale[axis] = 1.0 / OUTPUT_SCALE;
	emcmotStatus->axVscale[axis] = 1.0;
	emcmotConfig->axisLimitVel[axis] = 1.0;
	emcmotConfig->axisAcc[axis] = 1.0;
	emcmotDebug->bigVel[axis] = 1.0;
	SET_AXIS_ENABLE_FLAG(axis, 0);
	SET_AXIS_ACTIVE_FLAG(axis, 0);	/* default is not to use it; need an
					   explicit activate */
	SET_AXIS_NSL_FLAG(axis, 0);
	SET_AXIS_PSL_FLAG(axis, 0);
	SET_AXIS_NHL_FLAG(axis, 0);
	SET_AXIS_PHL_FLAG(axis, 0);
	SET_AXIS_INPOS_FLAG(axis, 1);
	SET_AXIS_HOMING_FLAG(axis, 0);
	SET_AXIS_HOMED_FLAG(axis, 0);
	SET_AXIS_FERROR_FLAG(axis, 0);
	SET_AXIS_FAULT_FLAG(axis, 0);
	SET_AXIS_ERROR_FLAG(axis, 0);
	emcmotConfig->axisPolarity[axis] = (EMCMOT_AXIS_FLAG) 0xFFFFFFFF;
	/* will read encoders directly, so don't set them here */


	if (-1 ==
	    tpCreate(&emcmotDebug->freeAxis[axis], FREE_AXIS_QUEUE_SIZE,
		     emcmotDebug->freeAxisTcSpace[axis])) {
	    diagnostics("can't create axis emcmotDebug->queue %d\n", axis);
	    goto cleanup_mem;	/* Another exit point that needs to do a cleanup */
	}
	tpInit(&emcmotDebug->freeAxis[axis]);
	tpSetCycleTime(&emcmotDebug->freeAxis[axis],
		       emcmotConfig->trajCycleTime);
	/* emcmotDebug->freePose is inited to 0's in decl */
	tpSetPos(&emcmotDebug->freeAxis[axis], emcmotDebug->freePose);
	tpSetVmax(&emcmotDebug->freeAxis[axis], emcmotStatus->vel);
	tpSetAmax(&emcmotDebug->freeAxis[axis], emcmotStatus->acc);
	pidInit(&emcmotConfig->pid[axis]);
	pidSetGains(&emcmotConfig->pid[axis], pid);
	cubicInit(&emcmotDebug->cubic[axis]);
    }

    /* FIXME-- add emcmotError */

    /* init min-max-avg stats */
    mmxavgInit(&emcmotDebug->tMmxavg, emcmotDebug->tMmxavgSpace,
	       MMXAVG_SIZE);
    mmxavgInit(&emcmotDebug->sMmxavg, emcmotDebug->sMmxavgSpace,
	       MMXAVG_SIZE);
    mmxavgInit(&emcmotDebug->nMmxavg, emcmotDebug->nMmxavgSpace,
	       MMXAVG_SIZE);
    mmxavgInit(&emcmotDebug->yMmxavg, emcmotDebug->yMmxavgSpace,
	       MMXAVG_SIZE);
    mmxavgInit(&emcmotDebug->fMmxavg, emcmotDebug->fMmxavgSpace,
	       MMXAVG_SIZE);
    mmxavgInit(&emcmotDebug->fyMmxavg, emcmotDebug->fyMmxavgSpace,
	       MMXAVG_SIZE);
    emcmotDebug->start_time = etime();

    /* init motion emcmotDebug->queue */
    if (-1 ==
	tpCreate(&emcmotDebug->queue, DEFAULT_TC_QUEUE_SIZE,
		 emcmotDebug->queueTcSpace)) {
	diagnostics("can't create motion emcmotDebug->queue\n");
	goto cleanup_mem;	/* Another exit point that needs to do a cleanup */
    }
    tpInit(&emcmotDebug->queue);
    tpSetCycleTime(&emcmotDebug->queue, emcmotConfig->trajCycleTime);
    tpSetPos(&emcmotDebug->queue, emcmotStatus->pos);
    tpSetVmax(&emcmotDebug->queue, emcmotStatus->vel);
    tpSetAmax(&emcmotDebug->queue, emcmotStatus->acc);

    /* init the time and rate using functions to affect traj, the
       pids, and the cubics properly, since they're coupled */
    setTrajCycleTime(TRAJ_CYCLE_TIME);
    setServoCycleTime(SERVO_CYCLE_TIME);

    emcmotStatus->tail = 0;

    diagnostics("Finished configuring emcmotStruct\n");

    /* Initialize the hardware here and clean up if it fails. */
    if (-1 == extMotInit((const char *) EMCMOT_INIFILE)) {
	diagnostics("can't initialize motion hardware\n");
	goto cleanup_mem;
    }

    if (-1 == extDioInit((const char *) EMCMOT_INIFILE)) {
	diagnostics("can't initialize DIO hardware\n");
	goto cleanup_hw;
    }

    if (-1 == extAioInit((const char *) EMCMOT_INIFILE)) {
	diagnostics("can't initialize AIO hardware\n");
	goto cleanup_dio;
    }

    extEncoderSetIndexModel(EXT_ENCODER_INDEX_MODEL_MANUAL);

/* FIXME - Pointless setting values here as they could be totally wrong
   and won't be correct until the ini file has been parsed */
    extAmpEnable(axis, !GET_AXIS_ENABLE_POLARITY(axis));

    rt_linux_use_fpu(1);
    rt_set_periodic_mode();

    servoCycleTicks =
	nano2count((int) (1e9 * emcmotConfig->servoCycleTime));
    diagnostics("emcmot: requested servoCycleTime=%d ticks\n",
		servoCycleTicks);

    iperiod = servoCycleTicks;
    iperiod_rtime = (RTIME) servoCycleTicks;

    servoCycleTicks = realtime_startup(servoCycleTicks);
    if (servoCycleTicks < 0) {
	diagnostics("emcmot: realtime_startup failed!\n");
	return (-1);		/* FIXME - Clean up shmem and tasks... */
    }
    diagnostics("emcmot: real servoCycleTime=%d ticks\n", servoCycleTicks);

    iperiod = servoCycleTicks;
    iperiod_rtime = (RTIME) servoCycleTicks;

    diagnostics("emcmot: initializing emcmotTask\n");
    if (0 != (module_error_code = rt_task_init(&emcmotTask,	/* RT_TASK */
					       emcmotController,	/* task function */
					       1,	/* arg */
					       EMCMOT_TASK_STACK_SIZE,	/* stack */
					       EMCMOT_TASK_PRIORITY,	/* priority */
					       1,	/* uses fpu */
					       emcmotsig))	/* signal function */
	) {
	diagnostics
	    ("rt_task_init failed for emcmotTask module_error_code=%d\n",
	     module_error_code);
	return -1;		/* FIXME - Clean up shmem and tasks... */
    }

    now = rt_get_time();
    setServoCycleTime(count2nano(iperiod) * 1e-9);

    diagnostics
	("emcmot: making emcmotTask periodic at a rate of %d ticks or %d nanoseconds \n",
	 servoCycleTicks, (int) count2nano(servoCycleTicks));

    if (0 !=
	(module_error_code =
	 rt_task_make_periodic(&emcmotTask, now + (RTIME) servoCycleTicks,
			       (RTIME) servoCycleTicks)
	)) {
	diagnostics
	    ("rt_task_make_periodic failed for emcmotTask module_error_code=%d\n",
	     module_error_code);
	return -1;		/* FIXME - Clean up shmem and tasks... */
    }
    emcmotTask_created = 1;

    diagnostics("emcmot: init_module finished\n");

    return 0;

/* Cleanup routines for when module initialization fails.. */

  cleanup_dio:
    /* Disable the DIO */
    extDioQuit();
  cleanup_hw:
    /* Shut down the hardware interface */
    extMotQuit();
  cleanup_mem:
    /* free shared memory */
    if (emcmotStruct != 0) {
	diagnostics("emcmot: releasing rtai shared memory.\n");
	rtai_kfree(SHMEM_KEY);
    }
  cleanup_proc:
    /* remove the proc fs */
    proc_clean();
    return -1;
}


void cleanup_module(void)
{
    int axis;

    diagnostics("emcmot: cleanup started.\n");

    if (emcmotTask_created) {
	diagnostics("emcmot: deleting emcmotTask.\n");
	rt_task_delete(&emcmotTask);
	emcmotTask_created = 0;
    }

    realtime_shutdown();

    /* There is no point in clearing the trajectory queue since the
       planner should be dead by now anyway. */
    if (emcmotStruct != 0 && emcmotDebug != 0 && emcmotConfig != 0) {
	diagnostics("emcmot: disabling amps\n");
	for (axis = 0; axis < EMCMOT_MAX_AXIS; axis++) {
	    extAmpEnable(axis, !GET_AXIS_ENABLE_POLARITY(axis));
	}

	/* quit board */
	diagnostics
	    ("emcmot: quitting analog, digital and motor interfaces\n");
	extAioQuit();
	extDioQuit();
	extMotQuit();
    }

    /* free shared memory */
    if (emcmotStruct != 0) {
	diagnostics("emcmot: releasing rtai shared memory.\n");
	rtai_kfree(SHMEM_KEY);
    }

    /* FIXME-- testing */
    diagnostics("emcmot: debounce counts: ");
    for (axis = 0; axis < EMCMOT_MAX_AXIS; axis++) {
	diagnostics("%d ", debouncecount[axis]);
    }
    diagnostics("\n");

    /* Clean up the proc fs */
    proc_clean();

    diagnostics("emcmot: cleanup finished.\n");
}
