/*
  emcmotutil.c

  Utility functions shared between motion and other systems

  Modification history:

  14-Aug-2000  FMP added check of errlog ptr for 0
  13-Mar-2000 WPS added unused attribute to pid_h to avoid 'defined but
  not used' compiler warning.
  12-Feb-1998  FMP created
  */

#include "emcmotcfg.h"		/* EMCMOT_ERROR_NUM,LEN */
#include "emcmot.h"		/* these decls */

/* ident tag */
#ifndef __GNUC__
#ifndef __attribute__
#define __attribute__(x)
#endif
#endif

static char __attribute__ ((unused)) ident[] =
    "$Id: emcmotutil.c,v 1.3 2005/08/12 11:37:42 paul Exp $";

int emcmotErrorInit(EMCMOT_ERROR * errlog)
{
    if (errlog == 0) {
	return -1;
    }

    errlog->head = 0;
    errlog->start = 0;
    errlog->end = 0;
    errlog->num = 0;
    errlog->tail = 0;

    return 0;
}

int emcmotErrorPut(EMCMOT_ERROR * errlog, const char *error)
{
    char *p1;
    const char *p2;
    int i;

    if (errlog == 0 || errlog->num == EMCMOT_ERROR_NUM) {
	/* full */
	return -1;
    }

    errlog->head++;

    // strncpy(errlog->error[errlog->end], error, LINELEN);
    // replaced strncpy with manual copy
    p1 = errlog->error[errlog->end];
    p2 = error;
    i = 0;
    while (*p2 && i < LINELEN) {
	*p1 = *p2;
	p1++;
	p2++;
	i++;
    }
    *p1 = 0;

    errlog->end = (errlog->end + 1) % EMCMOT_ERROR_NUM;
    errlog->num++;

    errlog->tail = errlog->head;

    return 0;
}

int emcmotErrorGet(EMCMOT_ERROR * errlog, char *error)
{
    char *p1;
    const char *p2;
    int i;
    if (errlog == 0 || errlog->num == 0) {
	/* empty */
	return -1;
    }

    errlog->head++;

    //  strncpy(error, errlog->error[errlog->start], LINELEN);  
    // replaced strncpy with manual copy
    p1 = error;
    p2 = errlog->error[errlog->start];
    i = 0;
    while (*p2 && i < LINELEN) {
	*p1 = *p2;
	p1++;
	p2++;
	i++;
    }
    *p1 = 0;


    errlog->start = (errlog->start + 1) % EMCMOT_ERROR_NUM;
    errlog->num--;

    errlog->tail = errlog->head;

    return 0;
}
