/********************************************************************
*
* Description: emcservo.c
*
*	Stubs for Stepper setup routines that are unused in servo
*	systems.
*
*    
* Author: Paul Corner
* License: GPL Version 2
* System: Linux
*    
* Last change: 
* $Revision: 1.2 $
* $Author: paul $
* $Date: 2005/08/12 11:37:42 $
*
* Copyright (c) 2004  All rights reserved.
*
*History
* 24Feb06   Matt Timmermans
*	    added realtime_* calls to set up timer stuff for emcmodule.c
* 
********************************************************************/

#include "emcmot.h"
#include "diagnostics.h"

MODULE_VERSION("$Id: emcservo.c,v 1.2 2005/08/12 11:37:42 paul Exp $");

int FIND_IO_BASE_ADDRESS;

static int currentServoCycleTicks = 1000;

int realtime_startup(int servoCycleTicks)
{
    diagnostics("EMCSERVO - compiled on %s at %s\n", __DATE__, __TIME__);

    if (servoCycleTicks < 2) {
	diagnostics("emcservo: bad cycle time setting (%d)\n",
		    servoCycleTicks);
	return (-1);
    }

    diagnostics("emcservo: starting RT timer: %d\n",
		(int) servoCycleTicks);
    start_rt_timer(servoCycleTicks);
    currentServoCycleTicks = servoCycleTicks;
    return (servoCycleTicks);
}


void realtime_shutdown(void)
{
    diagnostics("emcservo: stopping RT timer\n");
    stop_rt_timer();
}

int realtime_adjust(int servoCycleTicks)
{
    if (servoCycleTicks < 10) {
	servoCycleTicks = 10;
    }
    if (servoCycleTicks != currentServoCycleTicks) {
	diagnostics("emcservo: stopping RT timer\n");
	stop_rt_timer();
	diagnostics("emcservo: starting RT timer: %d\n",
		    (int) servoCycleTicks);
	start_rt_timer(servoCycleTicks);
	diagnostics("emcservo: timer changed\n");
	currentServoCycleTicks = servoCycleTicks;
    }

    return (servoCycleTicks);
}

void scaleOutputs(int max)
{
    int axis = 0;

    if (max > EMCMOT_MAX_AXIS)
	max = EMCMOT_MAX_AXIS;

    for (axis = 0; axis < max; axis++) {
	emcmotDebug->rawOutput[axis] =
	    (emcmotStatus->output[axis] -
	     emcmotStatus->outputOffset[axis]) *
	    emcmotDebug->inverseOutputScale[axis];
    }
}
