/********************************************************************
* Description: hmi_base.hh
*
*
* Author: Morphix User <>
* Created on: Mon Jul 25 15:17:33 BST 2005
* Computer: Babylon-117 
* System: Linux
*    
* Copyright (c) 2005 Morphix User  All rights reserved.
*
********************************************************************/
#ifndef HMI_BASE_HH
#define HMI_BASE_HH

#include "config.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <math.h>

#include "rcs.hh"
#include "posemath.h"
#include "emc.hh"
#include "canon.hh"
#include "emcglb.h"
#include "emccfg.h"
#include "inifile.hh"
#include "emcmotlog.h"

#define RETRY_TIME 10.0		// seconds to wait for subsystems to come up
#define RETRY_INTERVAL 1.0	// seconds between wait tries for a subsystem
#define EMC_COMMAND_DELAY   0.1	// how long to sleep between checks
#define CLOSE(a,b,eps) ((a)-(b) < +(eps) && (a)-(b) > -(eps))
#define LINEAR_CLOSENESS 0.0001
#define ANGULAR_CLOSENESS 0.0001
#define INCH_PER_MM (1.0/25.4)
#define CM_PER_MM 0.1
#define GRAD_PER_DEG (100.0/90.0)
#define RAD_PER_DEG TO_RAD	// from posemath.h

typedef enum {
    EMC_WAIT_NONE = 1,
    EMC_WAIT_RECEIVED,
    EMC_WAIT_DONE
} _emc_wait;

typedef enum {
    EMC_UPDATE_NONE = 1,
    EMC_UPDATE_AUTO
} _emc_update;

typedef enum {
    LINEAR_UNITS_CUSTOM = 1,
    LINEAR_UNITS_AUTO,
    LINEAR_UNITS_MM,
    LINEAR_UNITS_INCH,
    LINEAR_UNITS_CM
} _linear_units;

typedef enum {
    ANGULAR_UNITS_CUSTOM = 1,
    ANGULAR_UNITS_AUTO,
    ANGULAR_UNITS_DEG,
    ANGULAR_UNITS_RAD,
    ANGULAR_UNITS_GRAD
} _angular_units;

namespace Hmi {

    extern char error_string[LINELEN];
    extern char operator_text_string[LINELEN];
    extern char operator_display_string[LINELEN];
    extern int emcCommandSerialNumber;
    extern int saveEmcCommandSerialNumber;
    extern int programStartLine;
    extern char lastProgramFile[LINELEN];
    extern int axisJogging;
    extern double emcTimeout;
    extern int jogPol[EMC_AXIS_MAX];
    extern _emc_wait emcWaitType;
    extern _emc_update emcUpdateType;
    extern _linear_units linearUnitConversion;
    extern _angular_units angularUnitConversion;

    extern int emcTaskNmlGet(char *process);
    extern int emcErrorNmlGet(char *process);
    extern int tryNml(char *process);
    extern int updateStatus(void);
    extern int updateError(void);
    extern int emcCommandWaitReceived(int serial_number);
    extern int emcCommandWaitDone(int serial_number);
    extern int nmlQuit(void);
    extern double convertLinearUnits(double u);
    extern double convertAngularUnits(double u);
    extern int sendDebug(int level);
    extern int sendEstop(void);
    extern int sendEstopReset(void);
    extern int sendMachineOn(void);
    extern int sendMachineOff(void);
    extern int sendManual(void);
    extern int sendAuto(void);
    extern int sendMdi(void);
    extern int sendOverrideLimits(int axis);
    extern int sendJogStop(int axis);
    extern int sendJogCont(int axis, double speed);
    extern int sendJogIncr(int axis, double speed, double incr);
    extern int sendMistOn(void);
    extern int sendMistOff(void);
    extern int sendFloodOn(void);
    extern int sendFloodOff(void);
    extern int sendLubeOn(void);
    extern int sendLubeOff(void);
    extern int sendSpindleForward(void);
    extern int sendSpindleReverse(void);
    extern int sendSpindleOff(void);
    extern int sendSpindleIncrease(void);
    extern int sendSpindleDecrease(void);
    extern int sendSpindleConstant(void);
    extern int sendBrakeEngage(void);
    extern int sendBrakeRelease(void);
    extern int sendAbort(void);
    extern int sendHome(int axis);
    extern int sendFeedOverride(double override);
    extern int sendTaskPlanInit(void);
    extern int sendProgramOpen(char *program);
    extern int sendProgramRun(int line);
    extern int sendProgramPause(void);
    extern int sendProgramResume(void);
    extern int sendProgramStep(void);
    extern int sendMdiCmd(char *mdi);
    extern int sendLoadToolTable(const char *file);
    extern int sendToolSetOffset(int tool, double length, double diameter);
    extern int sendAxisCycleTime(int axis, double cycleTime);
    extern int sendAxisSetOutputScale(int axis, double scale, double offset);
    extern int sendAxisSetFerror(int axis, double ferror);
    extern int sendAxisSetGains(int axis, double p, double i, double d,
				double ff0, double ff1, double ff2,
				double backlash, double bias,
				double maxError, double deadband);
    extern int sendAxisSetOutput(int axis, double output);
    extern int sendAxisEnable(int axis, int val);
    extern int sendAxisLoadComp(int axis, const char *file);
    extern int sendAxisAlter(int axis, double alter);
    extern int sendLogOpen(char *file, int type, int size, int skip,
			   enum EMCLOG_TRIGGER_TYPE triggerType,
			   enum EMCLOG_TRIGGER_VAR triggerVar,
			   double triggerThreshold, int which);
    extern int sendLogStart(void);
    extern int sendLogStop(void);
    extern int sendLogClose(void);
    extern int sendSetTeleopEnable(int enable);
    extern int sendSetProbeIndex(int index);
    extern int sendSetProbePolarity(int polarity);
    extern int sendClearProbeTrippedFlag(void);
    extern int sendProbe(double x, double y, double z);
};

#endif
