#!/bin/sh
# the next line restarts using wish\
exec wish "$0" "$@"

# The gui for conversational programming
# Sandor Szabo, NIST, 12/6/99
# (RH) There is still some legacy garbage in here after my mods.

# make sure cpUtils.tcl has been sourced once
if ![info exists cpUtilsSourced] {
    source cpUtils.tcl
    puts "Sourced cpUtils.tcl"
}

# Flag to indicate cp text has been modified.  When pressing done,
# will prompt use with option to save.
set needToSave 0

# this variable is used to determine if currently editing an
# existing mop
set editExistingMop 0

# Always holds the text currently in a file cp text window is showing
set cpFileText ""

# Always holds the text currently in cp text window.  When editing,
# not the same as cpFileText until saved.
set cpCurrentText ""

# Name of current cp file
set cpFileName ""

;;######################################################################
;# CpRenumberCode

# After inserting gcode returned by mop editor, need to renumber each
# line.  This routine renumbers the entire cp program.  Really only
# need to renumber from start of insert to end of cp program.

proc CpRenumberCode { } {
    global rtnTextToCp cpCodeT sp
    set end [$cpCodeT index end]
    set pat {N[0-9]*}
    set beginNum "N"
    set lineNum 10
    for {set i 1} {$i < $end} {incr i} {
	# get line minus any whitespace at beginning
	set line [string trimleft [$cpCodeT get $i.0 $i.end]]
	# Test if first character indicates line
	set patMatch [string index $line 0]
	if {$patMatch == $beginNum} {
	    # Line starts with line number, find location of space
	    # following line number
	    set afterNumLoc [string first $sp $line]
	    # get remainder of line
	    set oldLine [string trimleft \
		    [string range $line $afterNumLoc end]]
	    # create new line number for this line
	    set thisLine $beginNum
	    set thisLine [append thisLine $lineNum]
	    incr lineNum 10
	    # create new line = new number and remainder of old line
	    set thisLine [append thisLine $sp $oldLine]
	    # get rid of old line in text widget and insert new line
	    $cpCodeT delete $i.0 $i.end
	    $cpCodeT insert $i.0 $thisLine
	}
    }
}
;;######################################################################
;# CpUpdateTextWindow

# When a mop editor is done, the gcode text is inserted into the cp program.
# proc CpUpdateTextWindow { varName index op } {

proc CpUpdateTextWindow { } {
    global rtnTextToCp cpCodeT
    global needToSave
    if { $rtnTextToCp == "" } {
	# no code to update
	return
    }
    set curInsert [$cpCodeT index insert]

    $cpCodeT insert $curInsert $rtnTextToCp
    # Adjust window to where insert cursor is
    $cpCodeT see insert

    CpRenumberCode
    set needToSave 1
}


;;######################################################################
;# CpSaveCodeT

proc CpSaveCodeT { } {
    global cpFileName cpFile cpCodeT cpFileText cpTest
    if { $cpFileName != "" && $cpFileName != 0 } {
	set cpFile [open $cpFileName {CREAT TRUNC RDWR}]
	puts -nonewline $cpFile [$cpCodeT get 1.0 end]
	close $cpFile
	if $cpTest {puts "->CpSaveCodeT: saved $cpFileName"}
    } else {
	if $cpTest {puts "->CpSaveCodeT: No cpFileName"}
    }
}

;;######################################################################
;# CpAskToSave

#

proc CpAskToSave { msg } {

    # If reached this point, then ask if need to save
    return [tk_messageBox -type yesnocancel -default yes \
	    -message  $msg \
	    -icon question]

}

;;######################################################################
;# CpNeedToSave

# Returns a 1 if there have been changes to text in cp text widget.
# Too bad the text widget doesn't tell you this automatically.

proc CpNeedToSave { } {
    global cpFileName cpFile cpCodeT cpFileText cpCurrentText cpTest
    global dir file pattern showdot

    # The easy test first.  If there is nothing in text window, then
    # there is nothing to save.
    if { [$cpCodeT index 1.end] == "1.0" && \
	    [$cpCodeT index end] == "2.0" && \
	    $cpFileText == ""} {
	if $cpTest {puts "->nothing in cpCodeT, no need to save"}
	return 0
    }

    # There's something in text window, but does it differ from file?
    set cpCurrentText [$cpCodeT get 1.0 [GetEndIndex $cpCodeT]]
    if {[string compare $cpCurrentText $cpFileText] == 0} {
	if $cpTest {puts "->cpCurrentText == cpFileText, no need to save"}
	return 0
    }

    if $cpTest {puts "->cpCurrentText != cpFileText, need to save"}
    return 1
}

;;######################################################################
;# CpOpenFile

# Opens a file, stores contents in cpFileText and loads up text window
proc CpOpenFile { } {
    global cpFileName cpFile cpCodeT cpFileText

    # Before openning a file, see if there is a need to close an
    # already open file.
    CpCloseFile

    set cpFileName [CpFileSelect "Select A File To Open"]
    if {$cpFileName != "" && $cpFileName != 0} {
	$cpCodeT delete 1.0 end
	set cpFile [open $cpFileName {RDWR CREAT}]
	set cpFileText [read $cpFile]
	$cpCodeT insert end $cpFileText
	$cpCodeT mark set insert 1.0
	close $cpFile
    }
}

;;######################################################################
;# CpCloseFile

# Close a file.

proc CpCloseFile { } {
    global cpFileName cpFile cpCodeT cpFileText cpTest

    if [CpNeedToSave] {
	
	# May need to save but have never selected a file yet
	if { $cpFileName == "" || $cpFileName == 0} {
	    set cpFileName [CpFileSelect "Select A File To Save To"]
	    if { $cpFileName == 0 } {
		# User cancelled
		set cpFileName ""
		return {}
	    } else {
		CpSaveCodeT
	    }
	} else {
	    # have a file name, but double check it is the right one
	    set choice [CpAskToSave "Do you want to save the changes to $cpFileName before closing?"]
	    switch -- $choice {
		"yes" {
		    CpSaveCodeT
		}
		"no" {
		    if $cpTest {puts "->No to save in $cpFileName"}
		}
		"cancel" {
		    if $cpTest {puts "->Cancel to save"}
		    return {}
		}
	    }
	}
    }
    CpClearFile
    set cpFileText ""
    set cpCurrentText ""
    set cpFileName ""
}

;;######################################################################
;# CpClearFile

# Clear a file.

proc CpClearFile { } {
    global cpFileName cpFile cpCodeT cpFileText
    $cpCodeT delete 1.0 end
}

;;######################################################################
;# CpDoneFile

# What do do when the done button is pressed.

proc CpDoneFile { } {
    global cpFileName cpFile cpCodeT cpFileText cpTest
    global cp

    # determine if save is needed
    if [CpNeedToSave] {
	
	set choice [CpAskToSave "Save before quiting?"]
	switch -- $choice {
	    "yes" {

		# Make sure there is a file name.  Not true if
		# starting from scratch.
		if { $cpFileName == "" || $cpFileName == 0 } {
		    set cpFileName [CpFileSelect "Select A File"]
		    if { $cpFileName == 0 } {
			# User cancelled
			set cpFileName ""
			return {}
		    }
		    CpSaveCodeT
		}
	    }
	    "no" {
		if $cpTest {puts "->No to save in $cpFileName"}
	    }
	    "cancel" {
		if $cpTest {puts "->Cancel to save"}
		return {}
	    }
	}
    }

    CpClearFile
    set cpFileText ""
    set cpCurrentText ""
    set cpFileName ""
    destroy $cp
}


;;######################################################################
;# CpSaveAsFile

# Save a file.

proc CpSaveAsFile { } {
    global cpFileName cpFileText cpCurrentText
    global dir file pattern showdot

    # Get name of file to save as
    set cpFileName [CpFileSelect "Select A File"]
    if {$cpFileName == 0} {
	# cancelled
	set cpFileName ""
	return {}
    }
    CpSaveCodeT
    set cpFileText $cpCurrentText
}

;;######################################################################
;# CpSaveFile

# Save a file.

proc CpSaveFile { } {
    global cpFileName cpFileText cpCurrentText
    global dir file pattern showdot

    if [CpNeedToSave] {

	# May need to save but have never selected a file yet
	if { $cpFileName == "" } {
	    set cpFileName [CpFileSelect "Select A File"]
	}
	if {$cpFileName == 0} {
	    # cancelled
	    set cpFileName ""
	    return {}
	}
	CpSaveCodeT
	set cpFileText $cpCurrentText
    }
}

;;######################################################################
;# CpMopNext - Move cursur forward to next mop instruction

# Move cursur from current position to the next occurance of a line
# beginning with the string defined by mopStart.

proc CpMopNext { direction } {
    global cpCodeT
    global mopStart
    catch {
    if { $direction == "forward" } {
	set next [$cpCodeT search -forward $mopStart \
		[expr [$cpCodeT index insert] + 0.1]]
    } else {
	set next [$cpCodeT search -backward $mopStart \
		[expr [$cpCodeT index insert] - 0.1]]
    }
    $cpCodeT mark set insert $next
    $cpCodeT see $next
    return $next
    }
}

;;######################################################################
;# CpMopDelete - Deletes mop where cursor currently located.

# Deletes all lines from $mopStart to $mopStop. Cursor maybe anywhere in mop.

proc CpMopDelete {  } {
    global cpCodeT
    global mopStart mopStop
    # Search backward for mopStart. Since insert cursor may be at
    # beginning of line, start search at end of line.
    catch {
    set start [$cpCodeT search -backward $mopStart \
     	    [$cpCodeT index "insert lineend"]]
    set stop  [$cpCodeT search -forward $mopStop \
	    $start]
    $cpCodeT delete $start [expr $stop + 1.0]
    $cpCodeT mark set insert $start
    $cpCodeT see $start
    return $start
    }
}

;;######################################################################
;# CpMopEdit - Edits mop where cursor currently located.

# Uses MOP_NAME as key to extract parameters and to call the right mop
# editor.  Sets globals curMopStart and curMopStop to boundaries of
# mop, in case this needs to be deleted if the mop is modified.

proc CpMopEdit {  } {
    global cpCodeT
    global mopStart mopStop
    global curMopStart curMopStop
    global editExistingMop
    # Search backward for mopStart. Since insert cursor may be at
    # beginning of line, start search at end of line.
    catch {
    set curMopStart [$cpCodeT search -backward $mopStart \
	    [$cpCodeT index "insert lineend"]]
    set curMopStop  [$cpCodeT search -forward $mopStop \
	    $curMopStart]
    set mopText [$cpCodeT get $curMopStart $curMopStop]
    set mopName [ReadXMLstyleArrayArgs  $mopText]
    #  Use this var to label moplist $mopName
    # let MopDone know that currently editing an existing mop
    set editExistingMop 1
    # run command associated with mop
    eval [append mopName "Cmd"]
    }
}

;;######################################################################
# MopDone - (rh)execute this whenever a mop done or insert button is hit.

proc MopDone { mopArgs } {
    global rtnTextToCp cpCodeT mopCodeT mop mopStop
    global curMopStart curMopStop
    global editExistingMop

    # Check if there is any code to insert.
    if { [$mopCodeT index end] != 2.0 } {

	if { $editExistingMop } {
	    # Since code to insert remove original mop.
	    # Don't do this when adding new mop.
	    $cpCodeT delete $curMopStart [expr $curMopStop + 1.0]
	    $cpCodeT mark set insert $curMopStart
	    $cpCodeT see $curMopStart
	}
	set editExistingMop 0

	set rtnTextToCp [WriteXMLstyleArrayArgs $mopArgs]
	append rtnTextToCp [$mopCodeT get 1.0 [GetEndIndex $mopCodeT]]
	append rtnTextToCp [format "$mopStop\n"]
	CpUpdateTextWindow
    }
}

;;######################################################################
;# MopRun - execute when run button is pressed.  Used in mopGui.tcl

proc MopRun { file } {
    global mopCodeT
	$mopCodeT delete 1.0 end; \
	sourcefile $file
}

;;######################################################################
;#  This proc is all that remains of mopGui.tcl that needs to periodically run
# called from individual hole or whatever Cmds.  Roll into single mop run proc

proc mopGuiReset {} {
    set mophelpfileptr ""
    set mophelpwindowptr ""
    gridScreen "mop"
}

;;#######################################################################
;# Create window for cpGui.

# Use this during debugging.  Can run in a separate window, see output
# and delete window to try again.  You need to double click on wish
# and source this file.

# set cp [toplevel .cp]

# OR, use this one when running as main window - just double
# click on cpGui.

set cp .

wm title $cp "Conversational Program"

;;#######################################################################
;# Generic menu for cp (rh mod Nov 2000)

#  The menubuttons are hard coded but could be a loop with a menulist also
# or could use the new menubar command

set cpButs [frame $cp.cpButs ]
set fmenu [menubutton $cpButs.filemenu -text "File   " -menu $cpButs.filemenu.m  \
    -underline 0 ]
set emenu [menubutton $cpButs.editmenu -text "Edit   " -menu $cpButs.editmenu.m  \
    -underline 0 ]
set vmenu [menubutton $cpButs.viewmenu -text "View   " -menu $cpButs.viewmenu.m  \
    -underline 0 ]
set amenu [menubutton $cpButs.addmopmenu -text "Select-Mop" -menu $cpButs.addmopmenu.m  \
    -underline 0 ]
pack $fmenu $emenu $vmenu $amenu -side left

set cpfilemenulist { \
    {Open 0 CpOpenFile} \
    {Close 0 CpCloseFile} \
    {Save 0 CpSaveFile} \
    {"Save As" 1 CpSaveAsFile} \
    {separator 0 0} \
    {Done 0 CpDoneFile}}

set cpeditmenulist { \
    {"Clear All" 0 CpClearFile}}

set cpviewmenulist { \
    {Moplist 0 {} }}

# this will eventually be a find file routine for auto menuing
# until then add routine names in set cpaddmopmenulist
# Display name <space> underline number<space> command
# Need to quote multi-word entries.

set cpaddmopmenulist {
    {"Mill Circle" 5 "mopCmd hole"} \
    {"Mill Pocket" 5 "mopCmd pocket"} \
    {"Drill Circular Array" 7 "mopCmd caholes"} \
    {"Drill Rectangle Array" 6 "mopCmd raholes"} \
    {"Drill Grill Array" 6 "mopCmd grill"} \
    {"Engrave Text String" 0 "mopCmd engrave"} \
    {"Engrave Round Bezel" 0 "mopCmd rbezel"}}

proc makeMenuButton {menuname menulist} {
    set i 0
    menu $menuname -tearoff 0
    foreach item $menulist {
        incr i
        set thisname [lindex $item 0]
        set thisnemonic [lindex $item 1]
        set thiscommand [lindex $item 2]
        if {[string tolower $thisname] == "separator" } {
            $menuname add separator
        } else {
            $menuname add command -label $thisname -command "$thiscommand" -underline $thisnemonic
        }
    }
}

makeMenuButton $fmenu.m $cpfilemenulist
makeMenuButton $emenu.m $cpeditmenulist
makeMenuButton $vmenu.m $cpviewmenulist
makeMenuButton $amenu.m $cpaddmopmenulist

# need to make automatic menubutton with filesearch for "*Help.txt"
# that way we can see all help from one place

set cphelpfileptr ""
set cphelpwindowptr ""

button $cpButs.context -text " ???? " -relief flat -padx 2 -width 6 -command { \
	EditableHelpWindow [toplevel $mop.help] $mopHelpFile \
	mophelpfileptr mophelpwindowptr}

button $cpButs.help -text "Help"  -relief flat -padx 2 -width 6 -command { \
	EditableHelpWindow [toplevel $cp.help] "cpHelp.txt" \
	cphelpfileptr cphelpwindowptr}

pack $cpButs.help $cpButs.context -in $cpButs -padx 2 -side right


;;#######################################################################
;# These buttons work for the top text window

set cpEditButs [frame $cp.cpEditButs -relief flat -borderwidth 4 ]

button $cpEditButs.forward -text FORWARD  -borderwidth 2  \
	-command { \
	CpMopNext "forward" \
}

button $cpEditButs.backward -text BACKWARD  -borderwidth 2 \
	-command { \
	CpMopNext "backward" \
}

button $cpEditButs.end -text END  -borderwidth 2  \
	-command { \
	$cpCodeT mark set insert [$cpCodeT index end];\
	$cpCodeT see end\
}

button $cpEditButs.delete -text DELETE  -borderwidth 2  \
	-command { \
	CpMopDelete \
}

button $cpEditButs.edit -text EDIT  -borderwidth 2  \
	-command { \
	CpMopEdit \
}

pack $cpEditButs.forward $cpEditButs.backward $cpEditButs.end \
	$cpEditButs.delete $cpEditButs.edit -in $cpEditButs -side top \
	-fill x

;;#######################################################################
;# Gcode text window for all nc code

set cpCode [frame $cp.cpCode -relief flat ]
set cpCodeT [TextWithScrollbar $cpCode]

;;######################################################################
# (rh mod nov 2k - moved mopGui file in here)
## The generic portion of all mop gui's (a text widget for gcode, a
## RUN button, etc.)

set mopButs [frame $cp.mopButs -relief flat -borderwidth 4]
button $mopButs.run -text RUN  -borderwidth 2 -command { \
	MopRun $mopRunFile
}

button $mopButs.clear -text CLEAR  -borderwidth 2 -command { \
	$mopCodeT delete 1.0 end}

button $mopButs.cancel -text CANCEL -borderwidth 2  -command {\
	$mopCodeT delete 1.0 end;\
	destroy mop $mopArgs
}

button $mopButs.insert -text INSERT -borderwidth 2 -command {\
	MopDone $mopName
	$mopCodeT delete 1.0 end
}

button $mopButs.done -text DONE  -borderwidth 2 -command { \
	MopDone $mopName
	destroy $mopArgs
}

pack $mopButs.run $mopButs.clear $mopButs.insert $mopButs.done $mopButs.cancel  \
    -in $mopButs -fill x -side top

;;#######################################################################
;# Gcode text window

set mopCode [frame $cp.mopCode -relief flat]
set mopCodeT [TextWithScrollbar $mopCode]
set mopText $mopCodeT

;;#######################################################################
;# Place main frames in toplevel

proc gridScreen {which} {
    global cpButs cpCode cpEditButs cpCode mop mopArgs mopCode mopButs
    switch -- $which {
        "base" {
            grid $cpButs -row 0 -column 0 -sticky news
            grid configure $cpButs -columnspan 2
            grid $cpCode  -row 1 -column 0 -sticky news
            grid $cpEditButs -row 1 -column 1 -sticky news
             }
        "mop" {
            grid forget $cpButs
            grid forget $cpCode
            grid forget $cpEditButs
            grid $cpButs -row 0 -column 0 -sticky news
            grid configure $cpButs -columnspan 2
            grid $cpCode  -row 1 -column 0 -sticky news
            grid $cpEditButs -row 1 -column 1 -sticky nsew
            grid $mopCode -row 2 -column 0 -sticky nsew
            grid $mopButs -row 2 -column 1 -sticky nsew
            grid $mopArgs -row 3 -column 0 -sticky nsew
            grid configure $mopArgs -columnspan 2
            grid $mop -sticky nsew
            }
        default {
            }
    }
}

gridScreen "base"

;;#######################################################################

# These are the button commands for mops.  You need to add one for
# each new mop.

set mop [frame $cp.cpMop ]

proc mopCmd { name } {
    global rtnTextToCp mop cp
    set rtnTextToCp "";
    sourcefile ${name}Gui.tcl
    mopGuiReset
}

proc pocketCmd { } {
    global rtnTextToCp mop cp
    set rtnTextToCp "";
    set mop [frame $cp.cpMop ]
    sourcefile pocketGui.tcl
    mopGuiReset
}

proc CAHolesCmd { } {
    global rtnTextToCp mop cp
    set rtnTextToCp "";
    set mop [frame $cp.cpMop ]
    sourcefile caholesGui.tcl
    mopGuiReset
}

proc RAHolesCmd { } {
    global rtnTextToCp mop cp
    set rtnTextToCp "";
    set mop [frame $cp.cpMop ]
    sourcefile raholesGui.tcl
    mopGuiReset
}

proc GrillCmd { } {
    global rtnTextToCp mop cp
    set rtnTextToCp "";
    set mop [frame $cp.cpMop ]
    sourcefile grillGui.tcl
    mopGuiReset
}

proc EngraveCmd { } {
    global rtnTextToCp mop cp
    set rtnTextToCp "";
    set mop [frame $cp.cpMop ]
    sourcefile engraveGui.tcl
    mopGuiReset
}

proc RBezelCmd {} {
    global rtnTextToCp mop cp
    set rtnTextToCp "";
    set mop [frame $cp.cpMop ]
    sourcefile RBezelGui.tcl
    mopGuiReset
}
