# This file contains utility code used by other programs
# Sandor Szabo, NIST, 12/3/99

# set to 1 when doing testing
set cpTest 1

# flag used to tell if this file has been sourced
set cpUtilsSourced ""

# pointer used to return text to cp editor that was generated from mop editor
set rtnTextToCp ""

# Use to source files
proc sourcefile { sourceFile } {
    puts "begin sourcing $sourceFile"
    if [catch {uplevel #0 source $sourceFile} errorStr] {
	puts "Error when sourcing $sourceFile: $errorStr"
    }
}

# number of spaces separating gcode arguments.
set sp "  "

# Utility that brings up a file selection window.  Returns file name
# selected by user.
sourcefile "fileselect.tcl"

# Globals for consistent widgets
set widgetGlobals(widgetCount) 0;
set widgetGlobals(backgroundFrame)  grey;
set widgetGlobals(backgroundText) lightgrey;
set widgetGlobals(backgroundButton) grey1;
set widgetGlobals(backgroundMenubutton) grey1;
set widgetGlobals(backgroundScrollbar) grey1;
set widgetGlobals(backgroundLabel) grey;
set widgetGlobals(backgroundEntry) grey1;

set widgetGlobals(foregroundText) black;
set widgetGlobals(foregroundButton) black;
set widgetGlobals(foregroundMenubutton) black;
set widgetGlobals(foregroundLabel) black;
set widgetGlobals(foregroundEntry) black;

set widgetGlobalList "widgetGlobals tk_version errorInfo tk_patchLevel argv tcl_version";

;;;######################################################################
;;;# TextWithScrollbar {framename OptionalArgs}
;;#  creates a text widget with a scrollbar in a given frame
;;#  framename - Frame to create the text widget in.
;;#  OptionalArgs - any args valid for a text widget

proc TextWithScrollbar {framename args} {
    global widgetGlobalList;  eval "global $widgetGlobalList";
    
    set t [eval {text $framename.t$widgetGlobals(widgetCount) -setgrid true \
	    -wrap word -width 80 -height 15 \
	    -background $widgetGlobals(backgroundText) \
	    -foreground $widgetGlobals(foregroundText) \
	    -yscrollcommand "$framename.sy$widgetGlobals(widgetCount) set"  \
	    -relief sunken } $args];
    
    scrollbar $framename.sy$widgetGlobals(widgetCount) -orient vert \
	    -command "$framename.t$widgetGlobals(widgetCount) yview ";
    
    pack $framename.sy$widgetGlobals(widgetCount) -side right -fill y ;
    pack $framename.t$widgetGlobals(widgetCount) \
	    -side left -fill both -expand y;
    incr widgetGlobals(widgetCount);
    return $t;
}

;;######################################################################
;# EditableHelpWindow { frame helpfilename helpfileptr}

# Creates a text window in frame and loads it with a help file. User
# can add comments directly to help file and save them without using a
# separate editor.  First define some utility routines.

# How to set a global variable from a button command.  See Welch
# section on "Button Commands and Scope Issues"
proc Update {varname val} {
    upvar #0 $varname x
    set x $val
    puts "Update: $varname = $x"
}

# Button command routine used to save file.  filename is the name of
# the file.  Since there may be many windows open, the name of the
# global variable containing the id of a particular open file is
# passed in fileptr. windowptr is the text widget containing the new
# text.

proc SaveFile { filename fileptr windowptr } {
    # fileptr is a global containing a file id. Set fileID to the file id
    # in fileptr.
    upvar #0 $fileptr fileID
    upvar #0 $windowptr windowID
    puts "SaveFile: $filename $fileptr $fileID $windowptr $windowID"


    # Since new contents of file may be smaller than old contents, need to 
    # zero out old file.  The only way to do this in tcl is to open a file
    # using TRUNC.  First close old file.  Then reopen and clear using 
    # ClearFile.  Put new file id in global fileptr.
    close $fileID
    set fid [open $filename {CREAT TRUNC RDWR}]
    Update $fileptr $fid

    # Now write out 
    puts -nonewline $fid [$windowID get 0.0 end]
    flush $fid
}

# cd e:/PROJECTS/ConverProg/src/; source makeboreGui.tcl


# Now we can define the proc.
 
proc EditableHelpWindow { framename helpfilename helpfileptr helpwindowptr} {

    puts "EditableHelpWindow: $framename $helpfilename $helpfileptr $helpwindowptr" 

    # open help file and put file id in global ptr
    set helpfileID [open $helpfilename {RDWR CREAT}]
    Update $helpfileptr $helpfileID

    # read in help text
    set helptext [read $helpfileID]

    # open the help window
    set helpwindow [frame $framename.helpwindow -relief raised -borderwidth 4]
    set helpwindowT [TextWithScrollbar $helpwindow]
    Update $helpwindowptr $helpwindowT
    $helpwindowT insert end $helptext

    # create frame for help buttons
    set hButs [frame $framename.hButs -relief raised -borderwidth 4]

    # add a button to save help text
    button $hButs.save -text SAVE -borderwidth 4 -padx 2 -width 6 \
	    -command [list \
	    SaveFile $helpfilename $helpfileptr $helpwindowptr]
    puts "hButs.save_command: [$hButs.save cget -command]"


    # add a button to exit the window
    button $hButs.done -text DONE  -borderwidth 4 -padx 2 -width 6 \
	    -command "destroy $framename "
    label $hButs.warning -text "DONE does not save any changes"
    

    pack $hButs.save $hButs.done $hButs.warning -in $framename.hButs -padx 2 -side left

    grid $helpwindow -sticky news
    grid $framename.hButs -sticky news

}

;;######################################################################
;# GetEndIndex $textWid

# Return the index (line.char) of the last char in the last line of
# the text widget.  Typically used in following manner to get all the
# text but not the last \n pointed to by end.
# set justText [$textWid get 1.0 [GetEndIndex $textWid]]
# In the above example, GetEndIndex might return 8.9 whereas just
# using "end" would return 9.0

proc GetEndIndex { textWid } {
    set pat {[0-9]*.}
    # First get line index of last line containing text.
    regexp {[0-9]*} [expr [$textWid index end] - 1] endLine
    # Then get last char index in endLine
    return [$textWid index $endLine.end]
    
}

;;######################################################################
;# CpFileSelect msg

# Brings up basic file select window with some extra features.  Msg is
# displayed at top of window.

proc CpFileSelect { msg } {
    global dir file pattern showdot
    set file ""
    # if dir not ".", then all entries come up blank (havn't figured out why)
    set dir "."
    set rtn [fileselect .fsel $msg dir file pattern showdot]
    
    # Don't want to return from fileselect if user hasn't picked a file
    while {$rtn == -1} {
	set dir "."
	set rtn [fileselect .fsel "Need to select a file"\
		dir file pattern showdot]
	puts $file; 
    }

    # Can only return a 0 (for cancel) or the file name
    return $rtn
}

;;######################################################################
# ReadXMLstyleArrayArgs { mopText }
# Given a list {mopText} with the following XML taged arguments:
#
#(<MOP_NAME> arrayName </MOP_NAME>)
#(<MOP_ARG> Arg1 Arg1Val </MOP_ARG>)
#(<MOP_ARG> Arg2 Arg2Vala Arg2Valb </MOP_ARG>)
#(<MOP_ARG> ArgN ArgNVal </MOP_ARG>)
#
# Note: take away first # in each line above.  
# I.e., each line in the list starts with one '#'
#
# This routine will automatically set the global array "arrayName"
# with the following values
# arrayName(Arg1) = Arg1Val
# arrayName(Arg2) = Arg1Vala Arg1Valb 
# and so on.
# Routine returns the "arrayName"

proc ReadXMLstyleArrayArgs { mopText } {
    set remains $mopText
    
    set start [lsearch -exact $remains "(<MOP_NAME>"]
    set stop  [lsearch -exact $remains "</MOP_NAME>)"]
    if {$start == -1 || $stop == -1} { error "need a mop name" }
    set mopName [lindex $remains [expr $start + 1]]
    upvar #0 $mopName refToMopArray
    set refToMopArray(mopName) $mopName
    
    while 1 {
	set start [lsearch -exact $remains "(<MOP_ARG>"]
	set stop  [lsearch -exact $remains "</MOP_ARG>)"]
	if {$start == -1 || $stop == -1} { break }
	set argName  [lindex $remains [expr $start + 1]]
	set argVal   [lrange $remains [expr $start + 2] [expr $stop - 1]]
	puts "$argName $argVal"
	set refToMopArray($argName) $argVal
	set remains  [lrange $remains [expr $stop + 1]  end]
    }
    return $mopName
}

;;######################################################################
# WriteXMLstyleArrayArgs { mopArray }
# This routine is the inverse of ReadXMLstyleArrayArgs.  It starts
# with an array "mopArray" of arguments and returns an XML-styled text
# list.  Each arguments appears on a separate line.  This makes it
# easy to insert in a text widget.
#
set mopStart "(<MOP>)"
set mopStop "(</MOP>)"

proc WriteXMLstyleArrayArgs { mopArray } {
    global mopStart mopStop
    upvar #0 $mopArray refToMopArray
    set val [format "%s\n" $mopStart]
    append val [format "(<MOP_NAME> %s </MOP_NAME>)\n" $refToMopArray(mopName)]

    foreach index [array names refToMopArray] {
	set argName $index
	set argVal  $refToMopArray($index)
	#puts "$mopArray\($argName\) = $argVal"
	append val [format "(<MOP_ARG> %s %s </MOP_ARG>)\n" $argName $argVal]
    }
#    append val [format "%s\n" $mopStop]
    return $val
}
;;#######################################################################
# Output RealNum with SigFrac significant digits
proc OutNum { RealNum SigFrac } {
    
    set i ""
    set j ""
    set format ""
    set end [expr abs($RealNum)]
    
    for { set i 1; set j 0 }\
	    { $j < 6 && $i <= $end }\
	    { set i [expr $i * 10]; incr j 1 } { 
    }

    ;# J <= # int digits + 1 for decimal pt + #frac digits
    set j [expr $j + $SigFrac - 1]
    if { $RealNum < 0.0 } {
	incr j 1 ;#add a position for the minus sign
    }

    set f [format "%%%d.%dlf" $j $SigFrac] ;# two %% prints one %
    set result [ format $f $RealNum ]
}

;;#######################################################################
# return a line number and increment
proc GetLineNo {} {
    global LineNo
    set val "N"
    set val [append val "$LineNo"]
    incr LineNo 10
    return $val
}

;;#######################################################################
# Output the gcode line to a mop text widget.  If mopText == "" then just
# write to stdout.  This is useful for first creating the gcode
# generator without having to build the text widget.
proc OutputGcode { mopText line } {

    if { $mopText == "" } {
	puts $line
    } else {
	$mopText insert end "$line\n"
    }
}
