# set global variables 
# cd e:/PROJECTS/ConverProg/src/; source fileselect.tcl
#
# --------------------
if { ![info exists dir] } {set dir "."}
if { ![info exists file] } {set file ""}
if { ![info exists pattern] } {set pattern "*.ngc"}
if { ![info exists showdot] } {set showdot 0}

# file selector box
# -----------------
# w - the window
# title - window title
# dir1 - open with this directory showing
# file1 - the returned file name selected by user.
# pattern1 - The filter pattern, if "", then pattern set to "*"
# showdot - ?
# set filename [fileselect .fsel "Select A File" dir file pattern showdot]

proc fileselect {w title dir1 file1 pattern1 showdot1} {
    upvar $dir1 dir
    upvar $file1 file
    upvar $pattern1 pattern
    upvar $showdot1 showdot
    global fileselect

    # local functions
    # ---------------
    proc settabstops {toplevel list} {
        for {set i 0} {$i < [llength $list]} {incr i} {
            set tab1 [lindex $list $i]
            set tab2 [lindex $list [expr ( $i + 1 ) % [llength $list]]]
            bindtags $tab1 [list $toplevel [winfo class $tab1] $tab1]
            bind $tab1 <Tab> "focus $tab2"
            bind $tab2 <Shift-Tab> "focus $tab1"
        }
    }

    proc dirnormalize {dir} {
        if {[string compare $dir ""] == 0} {set dir "."}
        set list [split $dir "/"]
        switch [lindex $list 0] {
            ""      { set d [list ""] }
            default { set d /usr/local/emc/programs }
        }
        foreach i $list {
            switch $i {
                ""      -
                "."     { }
                ".."    { if {[string compare [lindex $d end] ""] != 0} {
                             set d [lrange $d 0 [expr [llength $d] - 2]]
                          }
                        }
                default { lappend d $i }
            }
        }
        set s ""
        foreach i $d { append s $i/ }
        return $s
    }


    proc readdir {w dir file pattern} {
        global fileselect

        set dir [dirnormalize $dir]
        if {![file readable $dir]} {
            bell
            return
        }
        set file [file tail $file]

        if {[string compare $pattern ""] == 0} { set pattern "*" }

        set fileselect(path) $dir
        set fileselect(file) $file
        set fileselect(pattern) $pattern
        set fileselect(selection) $dir$file
        $w.filter.entry delete 0 end
        $w.filter.entry insert 0 "$dir$pattern"

        set file_list [lsort [glob -nocomplain "$dir{.,}*"]]
        $w.lists.frame1.list delete 0 end
        if {$fileselect(showdot)} {
            foreach i $file_list {
                if {[file isdirectory $i]} { $w.lists.frame1.list insert end $i }
            }
        } else {
            $w.lists.frame1.list insert end "$dir."
            $w.lists.frame1.list insert end "$dir.."  
            foreach i $file_list {
                if {[file isdirectory $i]} {
                    if { ![string match "$dir\.*" $i] } {
                        $w.lists.frame1.list insert end $i
                    }
                }
            }
        }
        
        if {[$w.lists.frame1.list size] > 0} {
            $w.lists.frame1.list xview \
                    [string last "/" [$w.lists.frame1.list get 0]]
        }


        if {$fileselect(showdot)} {
            set file_list [lsort [glob -nocomplain "$dir{.,}$pattern"]]
        } else {
            set file_list [lsort [glob -nocomplain "$dir$pattern"]]
        }
        $w.lists.frame2.list delete 0 end
        foreach i $file_list {
            if {![file isdirectory $i]} {
                $w.lists.frame2.list insert end [file tail $i]
            }
        }
        if {[$w.lists.frame2.list size] > 0} {
            $w.lists.frame2.list xview \
                    [string last "/" [$w.lists.frame2.list get 0]]
        }
    }

    # event actions
    # -------------
    proc cmd_showdot {w} {
        global fileselect

        readdir $w $fileselect(path) $fileselect(file) $fileselect(pattern)
    }

    proc cmd_filesel_filter {w} {
        set dir [file dirname [$w.filter.entry get]]
        set pattern [file tail [$w.filter.entry get]]
        readdir $w $dir "" $pattern
    }

    proc cmd_filesel_left_dblb1 {w list} {
        global fileselect

        if {[$list size] > 0 } {
            readdir $w "[$list get [$list curselection]]/" "" \
                    $fileselect(pattern)
        }
    }

    proc cmd_filesel_right_b1 {w list} {
        global fileselect

        if {[$list size] > 0} {
            set fileselect(file) [$list get [$list curselection]]
            set fileselect(selection) "$fileselect(path)$fileselect(file)"
        }
    }

    proc cmd_filesel_right_dblb1 {w list} {
        if {[$list size] > 0} {
            $w.buttons.ok flash
            $w.buttons.ok invoke
        }
    }

    # GUI
    # ---

    catch {destroy $w}
    toplevel $w
    wm protocol $w WM_DELETE_WINDOW { }
    wm transient $w [winfo toplevel [winfo parent $w]]
    wm title $w $title

    frame $w.title -relief raised -borderwidth 1
    frame $w.top -relief raised -borderwidth 1
    frame $w.bottom -relief raised -borderwidth 1
    pack $w.title $w.top $w.bottom -fill both -expand 1

    label $w.title.label -text $title
    pack $w.title.label -in $w.title -expand 1 -pady 5

    frame $w.filter
    label $w.filter.label -text "Filter" -anchor w
    checkbutton $w.filter.showdot -text "Show Dot Files" \
            -command "cmd_showdot $w" -variable fileselect(showdot)
    entry $w.filter.entry
    pack $w.filter.entry -side bottom -fill x
    pack $w.filter.label -side left
    pack $w.filter.showdot -side right

    frame $w.selection
    label $w.selection.label -text "Selection" -anchor w
    entry $w.selection.entry -textvariable fileselect(selection)
    pack $w.selection.label $w.selection.entry -fill x

    frame $w.lists

    frame $w.lists.frame1
    label $w.lists.frame1.label -text "directories" -anchor w
    scrollbar $w.lists.frame1.yscroll -relief sunken \
            -command "$w.lists.frame1.list yview"
    scrollbar $w.lists.frame1.xscroll -relief sunken -orient horizontal \
            -command "$w.lists.frame1.list xview"
    listbox $w.lists.frame1.list -width 30 -height 10 \
            -yscroll "$w.lists.frame1.yscroll set" \
            -xscroll "$w.lists.frame1.xscroll set" \
            -relief sunken -setgrid 1 -selectmode single
    pack $w.lists.frame1.label -side top -fill x
    pack $w.lists.frame1.yscroll -side right -fill y
    pack $w.lists.frame1.xscroll -side bottom -fill x
    pack $w.lists.frame1.list -expand yes -fill y

    frame $w.lists.frame2
    label $w.lists.frame2.label -text "Files" -anchor w
    scrollbar $w.lists.frame2.yscroll -relief sunken \
            -command "$w.lists.frame2.list yview"
    scrollbar $w.lists.frame2.xscroll -relief sunken -orient horizontal \
            -command "$w.lists.frame2.list xview"
    listbox $w.lists.frame2.list -width 30 -height 10 \
            -yscroll "$w.lists.frame2.yscroll set" \
            -xscroll "$w.lists.frame2.xscroll set" \
            -relief sunken -setgrid 1 -selectmode single
    pack $w.lists.frame2.label -side top -fill x
    pack $w.lists.frame2.yscroll -side right -fill y
    pack $w.lists.frame2.xscroll -side bottom -fill x
    pack $w.lists.frame2.list -expand yes -fill y

    frame $w.lists.fill
    pack $w.lists.frame1 -side left
    pack $w.lists.frame2 -side right
    pack $w.lists.fill -padx 10

    frame $w.buttons
    button $w.buttons.ok -text "Ok" -width 10
    button $w.buttons.filter -text "Filter" -width 10
    button $w.buttons.cancel -text "Cancel" -width 10
    pack $w.buttons.ok $w.buttons.filter $w.buttons.cancel -side left -expand 1

    pack $w.filter $w.lists $w.selection -side top -padx 20 -fill x -pady 5 \
            -in $w.top
    pack $w.buttons -expand 1 -fill both -pady 10 -in $w.bottom

    # event bindings
    # --------------

    $w.buttons.ok config -command "set fileselect(button) 0"
    $w.buttons.cancel config -command "set fileselect(button) 2"
    $w.buttons.filter config -command "cmd_filesel_filter $w"

    # directories window bindings
    bind $w.lists.frame1.list <Double-Button-1> "cmd_filesel_left_dblb1 $w %W"
    bind $w.lists.frame1.list <Double-space> "cmd_filesel_left_dblb1 $w %W"
    # File window bindings
    bind $w.lists.frame2.list <Button-1> "cmd_filesel_right_b1 $w %W"
    bind $w.lists.frame2.list <space> "cmd_filesel_right_b1 $w %W"
    bind $w.lists.frame2.list <Double-Button-1> "cmd_filesel_right_dblb1 $w %W"
    bind $w.lists.frame2.list <Double-space> "cmd_filesel_right_dblb1 $w %W"

    bind $w.filter.entry <Return> \
            "$w.buttons.filter flash; $w.buttons.filter invoke"
    bind $w.selection.entry <Return> \
            "$w.buttons.ok flash; $w.buttons.ok invoke"
    bind $w <Escape> "$w.buttons.cancel flash; $w.buttons.cancel invoke"

    settabstops $w [list $w.filter.showdot $w.filter.entry \
            $w.lists.frame1.list $w.lists.frame2.list $w.selection.entry \
            $w.buttons.ok $w.buttons.filter $w.buttons.cancel]

    # initialization
    # --------------

    set fileselect(path) $dir
    set fileselect(file) ""
    set fileselect(pattern) $pattern
    set fileselect(showdot) $showdot
    set fileselect(button) 0
    set fileselect(selection) ""

    readdir $w $dir $file $pattern
    wm withdraw $w
    update idletasks
    set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
            - [winfo vrootx [winfo parent $w]]]
    set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
            - [winfo vrooty [winfo parent $w]]]
    wm geom $w +$x+$y
    wm deiconify $w
    set old_focus [focus]
    grab $w
    focus $w.selection.entry
    tkwait variable fileselect(button)
    grab release $w
    catch {focus $old_focus}
    destroy $w
    if {$fileselect(button) == 0} {
	# user hit OK
        set dir $fileselect(path)

	# If user selected a file from Files window, then $fileselect(file)
	# holds the file name.  If user typed in the file name at end of path,
	# then need to extract from $fileselect(selection).
	set lastSlash [string last "/" $fileselect(selection)]
	if {$lastSlash != -1} {
	    # there is a path name to deal with
	    set fileselect(file) [string range \
		    $fileselect(selection) [expr $lastSlash + 1] end]
	} else {
	    set fileselect(file) $fileselect(selection)
	}
	set file $fileselect(file)
        set pattern $fileselect(pattern)
        set showdot $fileselect(showdot)
	# No test for valid file selection, may just be a directory
	if {$file == ""} {
	    return -1 
	} else {
	    return $fileselect(selection)
	}
    } else {
	# user hit Cancel
        return 0
    }
}
