# makeCAHoles.tcl
# A conversion of the c program written by Jon Elson
# See http://artsci.wustl.edu/~jmelson/makebore.
# Modified by Lawrence Glaister VE7IT Nov 2000

# make sure cpUtils.tcl has been sourced once
if ![info exists cpUtilsSourced] {
    source cpUtils.tcl
    puts "Sourced cpUtils.tcl"
}


# global variables
set X          ""
set Y          ""
set CurX       ""
set CurY       ""
set I          ""
set LineNo     10
set CircRadius  [expr $caholes(CircDia) / 2.0]
set TwoPi      6.283185308
set Angle      [expr $TwoPi / $caholes(NumPattern)]
set CurAngle   0.0


# Note, use append to build gcode argument to OutputCgode because 
# of better control of whitespaces.
# Lin Interp. and go to initial pos

# get drill to safe height
OutputGcode $mopText  [set gcode ""; append gcode\
    [GetLineNo] $sp\
    G0 $sp \
    Z [OutNum $caholes(SafeZ) 4] $sp (set_to_safe_height)] 

# set drilling speed
OutputGcode $mopText  [set gcode ""; append gcode\
    [GetLineNo] $sp\
    F [OutNum $caholes(FeedRate) 1] $sp (set_drilling_rate) ]     
    
for { set I 0 }\
	{ $I < $caholes(NumHoles) }\
	{ incr I} { 
    
    set CurY [expr $caholes(CenterY) + $CircRadius * cos($CurAngle)]
    set CurX [expr $caholes(CenterX) + $CircRadius * sin($CurAngle)] 

#    puts "drilling hole $I at $CurX,$CurY" 
   
    # fast seek to hole posn
    OutputGcode $mopText  [set gcode ""; append gcode\
	    [GetLineNo] $sp\
	    G0 $sp\
	    X [OutNum $CurX 4] $sp\
	    Y [OutNum $CurY 4]]

    # drill hole
    OutputGcode $mopText  [set gcode ""; append gcode\
	    [GetLineNo] $sp\
        G1 $sp \
        Z [OutNum -$caholes(HoleDepth) 4]]
        
    # retract drill
    OutputGcode $mopText  [set gcode ""; append gcode\
	    [GetLineNo] $sp\
	    G0 $sp \
        Z [OutNum $caholes(SafeZ) 4]]    
        
    set CurAngle [expr $CurAngle + $Angle ]
    }
    
