# makeEngrave.tcl
# Lawrence Glaister VE7IT Nov 2000, lg@jfm.bc.ca
# engraves a text string
# as of Dec 2000, only 1 uppercase single stroke font is supported
# the intention is to give full True type font support in the future
# if you need symbols added to this package, the data format for the
# fonts is a 1" high character centered on 0,0.
# if you add new symbols etc, please email them to me or update the 
# source where you downloaded this code.

# make sure cpUtils.tcl has been sourced once
if ![info exists cpUtilsSourced] {
    source cpUtils.tcl
    puts "Sourced cpUtils.tcl"
}

# this routine takes 1 line of the characters gcode and rotates/translates
# it into the proper position and then outputs the gcode.
proc processline { CentX CentY Angle Mult FlipX FlipY CmdLine } {
    upvar mopText mopText
    upvar sp sp
       
    # parse line consisting of cmd, x val y val { i val j val }
    # error handling could be improved, but the font is a well behaved
    # internal structure, so take the easy road
    set cmd ""
    set x 0.0
    set y 0.0
    set i 0.0
    set j 0.0
    set cmd [lindex $CmdLine 0] 

    if {[ lindex $CmdLine 1 ] == "X"} { set x [ lindex $CmdLine 2 ] }
    if {[ lindex $CmdLine 3 ] == "Y"} { set y [ lindex $CmdLine 4 ] } 
    if {[ llength $CmdLine ] >= 9 } {
        if {[ lindex $CmdLine 5 ] == "I"} { set i [ lindex $CmdLine 6 ] }
        if {[ lindex $CmdLine 7 ] == "J"} { set j [ lindex $CmdLine 8 ] } 
    }
    
    # this is the head hurting part... rotate
    set xx [expr $x * cos($Angle) -  $y * sin($Angle) ]
    set yy [expr $x * sin($Angle) +  $y * cos($Angle) ]


#    set x  [expr $xx * $Mult * $Rev + $CentX * $Rev]

    # scale and translate
    set x  [expr $xx * $Mult * $FlipX + $CentX * $FlipX]
    set y  [expr $yy * $Mult * $FlipY + $CentY * $FlipY]

    # now build the output gcode
    if { $cmd == "G00" || $cmd == "G01" } {   
        OutputGcode $mopText  [set gcode ""; append gcode\
            [GetLineNo] $sp  \
            $cmd $sp X  $sp  \
            [OutNum $x 4] $sp\
            Y $sp [OutNum $y 4]]       
    } elseif { $cmd == "G02" || $cmd == "G03" } {
        # scale, rotate, translate arcs
        set ii [expr $i * cos($Angle) - $j * sin($Angle) ]
        set jj [expr $i * sin($Angle) + $j * cos($Angle) ]
        set i  [expr $ii * $Mult * $FlipX]
        set j  [expr $jj * $Mult * $FlipY]
        # if reverse printing, arcs need flipping direction of cutting
        if { $FlipX == -1.0 && $FlipY ==  1.0  || \
             $FlipX ==  1.0 && $FlipY == -1.0  } {
            if { $cmd == "G02" } { set cmd "G03" } else { set cmd "G02" }
        }
        OutputGcode $mopText  [set gcode ""; append gcode\
            [GetLineNo] $sp  \
            $cmd $sp X  $sp  \
            [OutNum $x 4] $sp\
            Y $sp [OutNum $y 4] $sp \
            I $sp [OutNum $i 4] $sp \
            J $sp [OutNum $j 4]]       
    } else {
        puts "unknown command in font definition >>$CmdLine<< " 
    }
}

# global variables
set X          0.0
set Y          0.0
set CurX       0.0
set CurY       0.0
set LineNo     1000
set TwoPi      6.283185307
set DegToRad   [expr $TwoPi/360.0]

# fonts are defined as a string indexed array of lists
# up will get replaced with safeZ clearance set
# dwn will get replaced with cutting depth
# font co-ordinates have 0,0 in the center of the letter
# these get scaled, translated and rotated throught the magic of software
# The first line in each char defn is a comment with char width info used
# for kerning.
# fonts are defined on a nominal 1" x width grid with 0,0 at center
# This is handy, as the input param height can be used directly as
# a scaler for sizing the output.
set mysp { }

set font1($mysp) {
{1.0}
{G00 X  0.0000 Y  0.0000}
}

set font1(.) {
{0.5}
{G00 X 0.0000 Y  -0.5000}
{dwn}
}

set font1(-) {
{0.7}
{G00 X -0.2500 Y  0.0000}
{dwn}
{G01 X  0.2500 Y  0.0000}
}

set font1(=) {
{0.7}
{G00 X -0.2500 Y  0.1500}
{dwn}
{G01 X  0.2500 Y  0.1500}
{up}
{G00 X  0.2500 Y -0.1500}
{dwn}
{G01 X -0.2500 Y -0.1500}
}

set font1(+) {
{0.7}
{G00 X -0.2500 Y  0.0000}
{dwn}
{G01 X  0.2500 Y  0.0000}
{up}
{G00 X  0.0000 Y -0.2500}
{dwn}
{G01 X  0.0000 Y  0.2500}
}

set font1(0) {
{1.0}
{G00 X  0.3500 Y -0.2500}
{dwn}
{G01 X  0.3500 Y  0.2500}
{G03 X  0.1000 Y  0.5000 I -0.2500 J  0.0000}
{G01 X -0.1000 Y  0.5000}
{G03 X -0.3500 Y  0.2500 I  0.0000 J -0.2500}
{G01 X -0.3500 Y -0.2500}
{G03 X -0.1000 Y -0.5000 I  0.2500 J  0.0000}
{G01 X  0.1000 Y -0.5000}
{G03 X  0.3500 Y -0.2500 I  0.0000 J  0.2500}
}

set font1(1) {
{0.6}
{G00 X -0.1500 Y  0.3500}
{dwn}
{G03 X  0.1000 Y  0.5000 I -0.0250 J  0.3250}
{G01 X  0.1000 Y -0.5000}
}

set font1(2) {
{1.0}
{G00 X -0.3500 Y  0.2500}
{dwn}
{G01 X -0.3500 Y  0.3000}
{G02 X -0.1500 Y  0.5000 I  0.2000 J  0.0000}
{G01 X  0.1500 Y  0.5000}
{G02 X  0.3500 Y  0.3000 I  0.0000 J -0.2000}
{G01 X  0.3500 Y  0.2661}
{G02 X  0.2492 Y  0.0924 I -0.2000 J  0.0000}
{G01 X -0.2492 Y -0.1924}
{G03 X -0.3500 Y -0.3661 I  0.0992 J -0.1737}
{G01 X -0.3500 Y -0.5000}
{G01 X  0.3500 Y -0.5000} 
}

set font1(3) {
{1.0}
{G00 X -0.0500 Y  0.0000}
{dwn}
{G01 X  0.1500 Y  0.0000}
{G03 X  0.3500 Y  0.2000 I  0.0000 J  0.2000}
{G01 X  0.3500 Y  0.3000}
{G03 X  0.1500 Y  0.5000 I -0.2000 J  0.0000}
{G01 X -0.1500 Y  0.5000}
{G03 X -0.3500 Y  0.3000 I  0.0000 J -0.2000}
{G01 X -0.3500 Y  0.2500}
{up}
{G00 X -0.3500 Y -0.2500}
{dwn}
{G01 X -0.3500 Y -0.3000}
{G03 X -0.1500 Y -0.5000 I  0.2000 J  0.0000}
{G01 X  0.1500 Y -0.5000}
{G03 X  0.3500 Y -0.3000 I  0.0000 J  0.2000}
{G01 X  0.3500 Y -0.2000}
{G03 X  0.1500 Y  0.0000 I -0.2000 J  0.0000}
}

set font1(4) {
{1.0}
{G00 X  0.1500 Y -0.5000}
{dwn}
{G01 X  0.1500 Y  0.5000}
{G01 X -0.3500 Y -0.2500}
{G01 X  0.3500 Y -0.2500}
}

set font1(5) {
{1.0}
{G00 X -0.3500 Y -0.3000}
{dwn}
{G03 X -0.1500 Y -0.5000 I  0.2000 J  0.0000}
{G01 X  0.1500 Y -0.5000}
{G03 X  0.3500 Y -0.3000 I  0.0000 J  0.2000}
{G01 X  0.3500 Y -0.0500}
{G03 X  0.1500 Y  0.1500 I -0.2000 J  0.0000}
{G01 X -0.3500 Y  0.1500}
{G01 X -0.3500 Y  0.5000}
{G01 X  0.3500 Y  0.5000}
}

set font1(6) {
{1.0}
{G00 X  0.3500 Y  0.2500}
{dwn}
{G01 X  0.3500 Y  0.3000}
{G03 X  0.1500 Y  0.5000 I -0.2000 J  0.0000}
{G01 X -0.1500 Y  0.5000}
{G03 X -0.3500 Y  0.3000 I  0.0000 J -0.2000}
{G01 X -0.3500 Y -0.2000}
{G01 X -0.3500 Y -0.3000}
{G03 X -0.1500 Y -0.5000 I  0.2000 J  0.0000}
{G01 X  0.1500 Y -0.5000}
{G03 X  0.3500 Y -0.3000 I  0.0000 J  0.2000}
{G01 X  0.3500 Y -0.2000}
{G03 X  0.1500 Y  0.0000 I -0.2000 J  0.0000}
{G01 X -0.1500 Y  0.0000}
{G03 X -0.3500 Y -0.2000 I  0.0000 J -0.2000}
}

set font1(7) {
{1.0}
{G00 X -0.3500 Y  0.5000}
{dwn}
{G01 X  0.3500 Y  0.5000}
{G03 X -0.1000 Y -0.5000 I  0.7326 J -0.9309}
}

set font1(8) {
{1.0}
{G00 X -0.1985 Y  0.0060}
{dwn}
{G02 X -0.3500 Y  0.2000 I  0.0485 J  0.1940}
{G01 X -0.3500 Y  0.3000}
{G02 X -0.1500 Y  0.5000 I  0.2000 J  0.0000}
{G01 X  0.1500 Y  0.5000}
{G02 X  0.3500 Y  0.3000 I  0.0000 J -0.2000}
{G01 X  0.3500 Y  0.2000}
{G02 X  0.1500 Y  0.0000 I -0.2000 J  0.0000}
{G01 X -0.1500 Y  0.0000}
{G03 X -0.3500 Y -0.2000 I  0.0000 J -0.2000}
{G01 X -0.3500 Y -0.3000}
{G03 X -0.1500 Y -0.5000 I  0.2000 J  0.0000}
{G01 X  0.1500 Y -0.5000}
{G03 X  0.3500 Y -0.3000 I  0.0000 J  0.2000}
{G01 X  0.3500 Y -0.2000}
{G03 X  0.1500 Y  0.0000 I -0.2000 J  0.0000}
}

set font1(9) {
{1.0}
{G00 X -0.3500 Y -0.2500}
{dwn}
{G01 X -0.3500 Y -0.3000}
{G03 X -0.1500 Y -0.5000 I  0.2000 J  0.0000}
{G01 X  0.1500 Y -0.5000}
{G03 X  0.3500 Y -0.3000 I  0.0000 J  0.2000}
{G01 X  0.3500 Y  0.2000}
{G01 X  0.3500 Y  0.3000}
{G03 X  0.1500 Y  0.5000 I -0.2000 J  0.0000}
{G01 X -0.1500 Y  0.5000}
{G03 X -0.3500 Y  0.3000 I  0.0000 J -0.2000}
{G01 X -0.3500 Y  0.2000}
{G03 X -0.1500 Y  0.0000 I  0.2000 J  0.0000}
{G01 X  0.1500 Y  0.0000}
{G03 X  0.3500 Y  0.2000 I  0.0000 J  0.2000}
}

set font1(A) {
{1.0}
{G00 X -0.3500 Y -0.5000}
{dwn}
{G01 X  0.0000 Y  0.5000}
{G01 X  0.3500 Y -0.5000}
{up}
{G00 X  0.2625 Y -0.2500}
{dwn}
{G01 X -0.2625 Y -0.2500}
}

set font1(B) {
{1.0}
{G00 X -0.3500 Y  0.0500}
{dwn}
{G01 X  0.1000 Y  0.0500}
{G02 X  0.3500 Y -0.2000 I  0.0000 J -0.2500}
{G01 X  0.3500 Y -0.2500}
{G02 X  0.1000 Y -0.5000 I -0.2500 J  0.0000}
{G01 X -0.3500 Y -0.5000}
{G01 X -0.3500 Y  0.5000}
{G01 X  0.1000 Y  0.5000}
{G02 X  0.3000 Y  0.3000 I  0.0000 J -0.2000}
{G01 X  0.3000 Y  0.2500}
{G02 X  0.1000 Y  0.0500 I -0.2000 J  0.0000}
}

set font1(C) {
{1.0}
{G00 X  0.3500 Y  0.2000}
{dwn}
{G01 X  0.3500 Y  0.2500}
{G03 X  0.1000 Y  0.5000 I -0.2500 J  0.0000}
{G01 X -0.1000 Y  0.5000}
{G03 X -0.3500 Y  0.2500 I  0.0000 J -0.2500}
{G01 X -0.3500 Y -0.2500}
{G03 X -0.1000 Y -0.5000 I  0.2500 J  0.0000}
{G01 X  0.1000 Y -0.5000}
{G03 X  0.3500 Y -0.2500 I  0.0000 J  0.2500}
{G01 X  0.3500 Y -0.2000}
}

set font1(D) {
{1.0}
{G00 X -0.3500 Y -0.5000}
{dwn}
{G01 X  0.0000 Y -0.5000}
{G03 X  0.3500 Y -0.1500 I  0.0000 J  0.3500}
{G01 X  0.3500 Y  0.1500}
{G03 X  0.0000 Y  0.5000 I -0.3500 J  0.0000}
{G01 X -0.3500 Y  0.5000}
{G01 X -0.3500 Y -0.5000}
}

set font1(E) {
{1.0}
{G00 X -0.3500 Y  0.0000}
{dwn}
{G01 X  0.2200 Y  0.0000}
{up}
{G00 X  0.3500 Y  0.5000}
{dwn}
{G01 X -0.3500 Y  0.5000}
{G01 X -0.3500 Y -0.5000}
{G01 X  0.3500 Y -0.5000}
}

set font1(F) {
{1.0}
{G00 X -0.3500 Y  0.0500}
{dwn}
{G01 X  0.2500 Y  0.0500}
{up}
{G00 X  0.3500 Y  0.5000}
{dwn}
{G01 X -0.3500 Y  0.5000}
{G01 X -0.3500 Y -0.5000}
}

set font1(G) {
{1.0}
{G00 X  0.0000 Y -0.0500}
{dwn} 
{G01 X  0.3500 Y -0.0500}
{G01 X  0.3500 Y -0.2500}
{G02 X  0.1000 Y -0.5000 I -0.2500 J  0.0000}
{G01 X -0.1000 Y -0.5000}
{G02 X -0.3500 Y -0.2500 I  0.0000 J  0.2500}
{G01 X -0.3500 Y  0.2500}
{G02 X -0.1000 Y  0.5000 I  0.2500 J  0.0000}
{G01 X  0.1000 Y  0.5000}
{G02 X  0.3500 Y  0.2500 I  0.0000 J -0.2500}
{G01 X  0.3500 Y  0.2000}
}

set font1(H) {
{1.0}
{G00 X -0.3500 Y  0.0000}
{dwn}
{G01 X  0.3500 Y  0.0000}
{up}
{G00 X  0.3500 Y  0.5000}
{dwn}
{G01 X  0.3500 Y -0.5000}
{up}
{G00 X -0.3500 Y -0.5000}
{dwn}
{G01 X -0.3500 Y  0.5000}
}

set font1(I) {
{0.5}
{G00 X  0.0000 Y  0.5000}
{dwn}
{G01 X  0.0000 Y -0.5000}
}

set font1(J) {
{1.0}
{G00 X  0.3500 Y  0.5000}
{dwn} 
{G01 X  0.3500 Y -0.2000}
{G02 X  0.0500 Y -0.5000 I -0.3000 J  0.0000}
{G01 X -0.0500 Y -0.5000}
{G02 X -0.3500 Y -0.2000 I  0.0000 J  0.3000}
{G01 X -0.3500 Y -0.1000}
}

set font1(K) {
{1.0}
{G00 X  0.3500 Y -0.5000}
{dwn}
{G01 X -0.1065 Y  0.1087}
{up}
{G00 X -0.3500 Y -0.1000}
{dwn}
{G01 X  0.3500 Y  0.5000}
{up}
{G00 X -0.3500 Y  0.5000}
{dwn}
{G01 X -0.3500 Y -0.5000}
}

set font1(L) {
{1.0}
{G00 X -0.3500 Y  0.5000}
{dwn}
{G01 X -0.3500 Y -0.5000}
{G01 X  0.3500 Y -0.5000}
}

set font1(M) {
{1.0}
{G00 X -0.3500 Y -0.5000}
{dwn}
{G01 X -0.3500 Y  0.5000}
{G01 X  0.0000 Y  0.0000}
{G01 X  0.3500 Y  0.5000}
{G01 X  0.3500 Y -0.5000}
}

set font1(N) {
{1.0}
{G00 X -0.3500 Y -0.5000}
{dwn}
{G01 X -0.3500 Y  0.5000}
{G01 X  0.3500 Y -0.5000}
{G01 X  0.3500 Y  0.5000}
}

set font1(O) {
{1.0}
{G00 X  0.3500 Y -0.2500}
{dwn}
{G01 X  0.3500 Y  0.2500}
{G03 X  0.1000 Y  0.5000 I -0.2500 J  0.0000}
{G01 X -0.1000 Y  0.5000}
{G03 X -0.3500 Y  0.2500 I  0.0000 J -0.2500}
{G01 X -0.3500 Y -0.2500}
{G03 X -0.1000 Y -0.5000 I  0.2500 J  0.0000}
{G01 X  0.1000 Y -0.5000}
{G03 X  0.3500 Y -0.2500 I  0.0000 J  0.2500}
}

set font1(P) {
{1.0}
{G00 X -0.3500 Y  0.0000}
{dwn}
{G01 X  0.1500 Y  0.0000}
{G03 X  0.3500 Y  0.2000 I  0.0000 J  0.2000}
{G01 X  0.3500 Y  0.3000}
{G03 X  0.1500 Y  0.5000 I -0.2000 J  0.0000}
{G01 X -0.3500 Y  0.5000}
{G01 X -0.3500 Y -0.5000}
}

set font1(Q) {
{1.0}
{G00 X  0.3500 Y -0.2500}
{dwn}
{G01 X  0.3500 Y  0.2500}
{G03 X  0.1000 Y  0.5000 I -0.2500 J  0.0000}
{G01 X -0.1000 Y  0.5000}
{G03 X -0.3500 Y  0.2500 I  0.0000 J -0.2500}
{G01 X -0.3500 Y -0.2500}
{G03 X -0.1000 Y -0.5000 I  0.2500 J  0.0000}
{G01 X  0.1000 Y -0.5000}
{G03 X  0.3500 Y -0.2500 I  0.0000 J  0.2500}
{up}
{G00 X  0.1000 Y -0.2500}
{dwn}
{G01 X  0.3500 Y -0.5000}
}

set font1(R) {
{1.0}
{G00 X -0.3500 Y -0.5000}
{dwn}
{G01 X -0.3500 Y  0.5000}
{G01 X  0.1500 Y  0.5000}
{G02 X  0.3500 Y  0.3000 I  0.0000 J -0.2000}
{G01 X  0.3500 Y  0.2000}
{G02 X  0.1500 Y  0.0000 I -0.2000 J  0.0000}
{G01 X -0.3500 Y  0.0000}
{up}
{G00 X  0.0000 Y  0.0000}
{dwn}
{G01 X  0.3500 Y -0.5000}
}

set font1(S) {
{1.0}
{G00 X -0.3500 Y -0.3000}
{dwn}
{G03 X -0.1500 Y -0.5000 I  0.2000 J  0.0000}
{G01 X  0.1500 Y -0.5000}
{G03 X  0.3500 Y -0.3000 I  0.0000 J  0.2000}
{G01 X  0.3500 Y -0.2000}
{G03 X  0.1500 Y  0.0000 I -0.2000 J  0.0000}
{G01 X -0.1500 Y  0.0000}
{G02 X -0.3500 Y  0.2000 I  0.0000 J  0.2000}
{G01 X -0.3500 Y  0.3000}
{G02 X -0.1500 Y  0.5000 I  0.2000 J  0.0000}
{G01 X  0.1500 Y  0.5000}
{G02 X  0.3500 Y  0.3000 I  0.0000 J -0.2000}
}

set font1(T) {
{1.0}
{G00 X  0.0000 Y -0.5000}
{dwn}
{G01 X  0.0000 Y  0.5000}
{up}
{G00 X -0.3500 Y  0.5000}
{dwn}
{G01 X  0.3500 Y  0.5000}
}

set font1(U) {
{1.0}
{G00 X -0.3500 Y  0.5000}
{dwn} 
{G01 X -0.3500 Y -0.2000}
{G03 X -0.0500 Y -0.5000 I  0.3000 J  0.0000}
{G01 X  0.0500 Y -0.5000}
{G03 X  0.3500 Y -0.2000 I  0.0000 J  0.3000}
{G01 X  0.3500 Y  0.5000}
}

set font1(V) {
{1.0}
{G00 X -0.3500 Y  0.5000}
{dwn}
{G01 X  0.0000 Y -0.5000}
{G01 X  0.3500 Y  0.5000}
}

set font1(W) {
{1.0}
{G00 X -0.3500 Y  0.5000}
{dwn} 
{G01 X -0.2000 Y -0.5000}
{G01 X  0.0000 Y  0.5000}
{G01 X  0.2000 Y -0.5000}
{G01 X  0.3500 Y  0.5000}
}

set font1(X) {
{1.0}
{G00 X -0.3500 Y -0.5000} 
{dwn} 
{G01 X  0.3500 Y  0.5000}
{up} 
{G00 X -0.3500 Y  0.5000}
{dwn} 
{G01 X  0.3500 Y -0.5000}
}

set font1(Y) {
{1.0}
{G00 X -0.3500 Y  0.5000}
{dwn}
{G01 X  0.0000 Y  0.0000}
{G01 X  0.3500 Y  0.5000}
{up}
{G00 X  0.0000 Y  0.0000}
{dwn}
{G01 X  0.0000 Y -0.5000}
}

set font1(Z) {
{1.0}
{G00 X -0.3500 Y  0.5000}
{dwn} 
{G01 X  0.3500 Y  0.5000}
{G01 X -0.3500 Y -0.5000}
{G01 X  0.3500 Y -0.5000} 
}

# Note, use append to build gcode argument to OutputCgode because 
# of better control of whitespaces.
# Lin Interp. and go to initial pos

# get drill to safe height
OutputGcode $mopText  [set gcode ""; append gcode\
    [GetLineNo] $sp\
    G0 $sp \
    Z [OutNum $engrave(SafeZ) 4] $sp (retract_to_safe_height)] 

# set drilling speed
OutputGcode $mopText  [set gcode ""; append gcode\
    [GetLineNo] $sp\
    F [OutNum $engrave(FeedRate) 1] $sp $sp $sp $sp (set_engraving_rate)]   

# internally, we need flip flags to be a multiplier, not a flag
if {$engrave(FlipX) == "Y" || $engrave(FlipX) == "y" } {
    set FlipX -1.0 } else { set FlipX 1.0 }
if {$engrave(FlipY) == "Y" || $engrave(FlipY) == "y" } { 
    set FlipY -1.0 } else { set FlipY 1.0 }


# current font only supports upper case characters
set ustring [string toupper $engrave(EText)]
set strlen [string length $ustring]
puts "Engraving $strlen characters >>>$ustring<<<"    

set curx [expr $engrave(StartX)]
set cury [expr $engrave(StartY)]


for  {set i 0} { $i < $strlen } {incr i} {

    set ch [string index $ustring $i]
 
    OutputGcode $mopText  [set gcode ""; append gcode\
        [GetLineNo] $sp\
        (Engrave>>  $ch <<)]

    # make sure we have a font defined for char, else complain and 
    # engrave a space
    if [catch {set t $font1($ch)}] {
        puts "Sorry, no font definition for character >>$ch<<"
        OutputGcode $mopText  [set gcode ""; append gcode\
            [GetLineNo] $sp\
            (ERROR_-_No_font_definition_for_character_>>$ch<<_ERROR) ]         
        set ch $mysp        
    }   

    
    # process each line of gcode from character defintion
    set first 1
    foreach fline $font1($ch) {
        # handle 3 special cases in char encoding
        # the first line in each defn contains character width
        if { $first == 1 } {
            set chwidth $fline
            # update the centerpoint for the character
            # offset by 1/2 current char width
            set curx [expr $curx + $chwidth/2.0 * cos($engrave(Angle) * \
                        $DegToRad) * $engrave(Height) ]
            set cury [expr $cury + $chwidth/2.0 * sin($engrave(Angle) * \
                        $DegToRad) * $engrave(Height)]   
            set first 0
            continue
        }
        #if line says up, output gcode to retract quill to SafeZ
        if {[string compare $fline "up"] == "0"} {
            OutputGcode $mopText  [set gcode ""; append gcode\
                [GetLineNo] $sp\
                G00 $sp Z $sp\
                [OutNum $engrave(SafeZ) 4]]
            continue
        }
        #if line says dwn, output gcode to extend quill to engraving depth
        if {[string compare $fline "dwn"] == "0"} {
            OutputGcode $mopText  [set gcode ""; append gcode\
                [GetLineNo] $sp\
                G01 $sp Z $sp\
                [OutNum $engrave(CutZ) 4]]
            continue
        }

        # convert and output gcode line by rotation, translation
        processline $curx $cury [expr $engrave(Angle)*$DegToRad] \
                    $engrave(Height) $FlipX $FlipY $fline 
        
    }
    # after each char, we need to retract quill to SafeZ
    OutputGcode $mopText  [set gcode ""; append gcode\
        [GetLineNo] $sp\
        G00 $sp Z $sp\
       [OutNum $engrave(SafeZ) 4]]
    
    # update the centerpoint for the next character
    # offset by 1/2 last char width
    set curx [expr $curx + $chwidth/2 * cos($engrave(Angle) * \
             $DegToRad) * $engrave(Height) ]
    set cury [expr $cury + $chwidth/2 * sin($engrave(Angle) * \
             $DegToRad) * $engrave(Height) ]   
}

