# makehole.tcl
# A conversion of the c program written by Jon Elson
# See http://artsci.wustl.edu/~jmelson/makebore.c
# Sandor Szabo, NIST, 12/2/99

# make sure cpUtils.tcl has been sourced once
if ![info exists cpUtilsSourced] { 
    source cpUtils.tcl
    puts "Sourced cpUtils.tcl"
}

# enum States_type
set initial 0
set Spiraling 1
set finishing 2
set State "";# is of type State_type

# eunum Interp_type
set linear 0
set arc 1
set Interp "";# is of type Interp_type

# global variables
set BoreRadius ""
set CurRadius ""
set NX ""
set NY ""
set NR ""
set CurX ""
set CurY ""
set CurZ ""
set CentDist ""
set ToolRadius ""
set ArcCent ""
set ArcRadius ""
set OldRadius ""
set Quadrant ""
set FinQuadrant ""
set true 1
set false 0
set LineNo 10
set Done ""
set FeedonLine ""

set ToolRadius [expr $hole(ToolDia) / 2.0]
set BoreRadius [expr $hole(BoreDia) / 2.0]

# safety test on Width of cut
if { [expr $BoreRadius - $ToolRadius - $hole(FinishWOC)] <= $hole(WidthOfCut) } {
    puts stderr "Width of Cut too large to spiral out to requested bore."
    # exit
}

for { set CurZ [expr -$hole(DepthPerPass)] }\
	{ $CurZ >= [expr -$hole(BoreDepth)-$hole(DepthPerPass)+0.001] }\
	{ set CurZ [expr $CurZ-$hole(DepthPerPass)] } { 
    
    
    if { $CurZ < [expr -$hole(BoreDepth)] } {
	set CurZ [expr -$hole(BoreDepth)]
	;# State = LastLevel
    }

    # Set linear interpolation, feedrate and move to center
    set CurX $hole(CenterX)
    set CurY $hole(CenterY)
    set Interp $linear;

    # Note, use append to build gcode argument to OutputCgode because 
    # of better control of whitespaces.
    OutputGcode $mopText  [set gcode ""; append gcode\
	    [GetLineNo] $sp\
	    G01 $sp\
	    F [OutNum $hole(FeedRate) 3] $sp\
	    X [OutNum $CurX 4] $sp\
	    Y [OutNum $CurY 4]]

    # Lower tool
    OutputGcode $mopText  [set gcode ""; append gcode\
	    [GetLineNo] $sp\
	    F [OutNum [expr $hole(FeedRate)/6.0] 3] $sp\
	    Z [OutNum $CurZ 4]]

    # Move in X to start of first spiral cut
    set CurRadius [expr $hole(WidthOfCut)/4.0]
    set CurX [expr $CurX + $CurRadius]
    set OldRadius $CurRadius
    OutputGcode $mopText  [set gcode ""; append gcode\
	    [GetLineNo] $sp\
	    F [OutNum [expr $hole(FeedRate)] 3] $sp\
	    X [OutNum $CurX 4]]
    set gcode ""

    # loop to spiral out in quadrants
    set Quadrant 1
    set FinQuadrant 0;
    set State $initial;
    set Done $false
    set FeedonLine $false
    set EventCnt 200
    while { ! $Done  && $EventCnt > 0 } {
	incr EventCnt -1
	if { $State == $finishing } {
	    # puts "State Finishing 1"
	    set CentDist $CurRadius
	    set ArcRadius $CurRadius
	} else {
	    set CentDist [expr $CurRadius + $hole(WidthOfCut) / 4.0]
	    # compute where to put center of arc
	    set ArcCent [expr ($OldRadius*$OldRadius - $CurRadius*$CurRadius)\
		    /(-2.0*$CurRadius)]
	    set ArcRadius [expr $CurRadius - $ArcCent]
	    set OldRadius $CurRadius 
	}
	
	switch -- $Quadrant {
	    1 {
		set NX $hole(CenterX)
		set NY [expr $hole(CenterY) + $CurRadius]
		set NR $ArcRadius
	    }
	    2 {
		set NX [expr $hole(CenterX) - $CurRadius]
		set NY $hole(CenterY)
		set NR $ArcRadius
	    }
	    3 {
		set NX $hole(CenterX)
		set NY [expr $hole(CenterY) - $CurRadius]
		set NR $ArcRadius
	    }
	    4 {
		set NX [expr $hole(CenterX) + $CurRadius]
		set NY $hole(CenterY)
		set NR $ArcRadius
	    }
	}
	;# switch

	# This prevents line numbers during finish cut - not sure why?
	if { $FeedonLine == $false } {
	    append gcode [GetLineNo] $sp
	    set FeedonLine $false;# Curious, this is already false
	}
	
	if { $Interp == $linear } {
	    set Interp $arc
	    append gcode G03 $sp
	} 

	OutputGcode $mopText [append gcode \
		X [OutNum $NX 4] $sp\
		Y [OutNum $NY 4] $sp\
		R [OutNum $NR 4]]
	# gcode needs to be cleared here in order to 
	set gcode ""

	if { $State == $finishing } {
	    # puts "State Finishing 2"
	    incr FinQuadrant
	    if { $FinQuadrant >= 5 } {
		set Done $true;
	    }
	}

	if { $State == $Spiraling } {
	    # puts "State Spiraling"
	    if { $CurRadius <= [expr $BoreRadius \
		    - $ToolRadius - $hole(FinishWOC) - 0.001] } {
		# Still need spiral rough cuts
		set CurRadius [expr $CurRadius \
			+ ($hole(WidthOfCut) / 4.0)]
	    } else {
		# puts "Cur Z = $CurZ, hole(BoreDepth) = $hole(BoreDepth)"
		if { [expr $CurZ-.001] <= [expr -$hole(BoreDepth)] } {
		    # puts "State is finishing";# bore to finish diameter
		    set State $finishing
		    set CurRadius [expr $BoreRadius - $ToolRadius]
		    set FinQuadrant 0
		    
		    # set finish feedrate
		    append gcode [GetLineNo] $sp\
			    F [OutNum $hole(FinishFeed) 3] $sp
		    set FeedonLine $true;
		} else {
		    # Bore to true circle, leaving hole(FinishWOC) */
		    set State $finishing 	
		    set CurRadius [expr $BoreRadius - $ToolRadius - \
			    $hole(FinishWOC)]
		    set FinQuadrant 0
		}
	    }
	}

	if { $State == $initial } {
	    #puts "State Initial"
	    if { $CurRadius > $hole(WidthOfCut) } {
		set State $Spiraling
	    }
	    set CurRadius [expr $CurRadius + ($hole(WidthOfCut) / 4.0)]
	}
	incr Quadrant
	if { $Quadrant > 4 } {
	    set Quadrant 1
	}
    }
    ;# while !Done
} 
;# for CurZ    

# return to center
OutputGcode $mopText  [set gcode ""; append gcode\
	[GetLineNo] $sp\
	G01 $sp\
	X [OutNum $hole(CenterX) 4] $sp\
	Y [OutNum $hole(CenterY) 4]]

# raise spindle above work 
OutputGcode $mopText  [set gcode ""; append gcode\
	[GetLineNo] $sp\
	G01 $sp F50 $sp Z0.1]

# End of Program - this should be inserted when a nc file is closed.
# This must go outside the <MOP> </MOP> wrapper.  Is this really
# needed?

#OutputGcode $mopText  [set gcode ""; append gcode\
#	[GetLineNo] $sp\
#	M02]

