# makehole.tcl
# A conversion of the c program written by Jon Elson
# See http://artsci.wustl.edu/~jmelson/makebore.c
# Sandor Szabo, NIST, 12/2/99

# make sure cpUtils.tcl has been sourced once
if ![info exists cpUtilsSourced] {
    source cpUtils.tcl
    puts "Sourced cpUtils.tcl"
}

# enum States_type
set initial 0
set Spiraling 1
set finishing 2
set State "";# is of type State_type

# global variables
set BoreRadius ""
set CurRadius ""
set NX ""
set NY ""
set NR ""
set CurX ""
set CurY ""
set CurZ ""
set CentDist ""
set ToolRadius ""
set ArcCent ""
set ArcRadius ""
set OldRadius ""
set Quadrant ""
set FinQuadrant ""
set true 1
set false 0
set LineNo 10
set Done ""
set FeedonLine ""

set ToolRadius [expr $pocket(ToolDia) / 2.0]
if { $pocket(Xsize) >= $pocket(Ysize) } {
    set MajAxis "Xaxis"
} else {
    set MajAxis "Yaxis"
}

if { $MajAxis == "Xaxis" } {
    set CurX [expr $pocket(llX) + $ToolRadius + $pocket(FinishWOC)];
    set CurY [expr $pocket(llY) + $pocket(Ysize) / 2.0];
} elseif { $MajAxis == "Yaxis" } {
    set CurX [expr $pocket(llX) + $pocket(Xsize) / 2.0]
    set CurY [expr $pocket(llY) + $ToolRadius + $pocket(FinishWOC)]
} else {
    puts "Error: MajAxis == \"$MajAxis\""
}

# Note, use append to build gcode argument to OutputCgode because 
# of better control of whitespaces.

# Lin Interp. and go to initial pos
OutputGcode $mopText  [set gcode ""; append gcode\
	[GetLineNo] $sp\
	G01 $sp\
	F [OutNum 40.0 1] $sp\
	X [OutNum $CurX 4] $sp\
	Y [OutNum $CurY 4]]

# rapid feed to near top of work 
OutputGcode $mopText  [set gcode ""; append gcode\
	[GetLineNo] $sp\
	Z [OutNum 0.02 4] $sp\
	F40];puts "Entering for"
for { set CurZ [expr -$pocket(DepthPerPass)] }\
	{ $CurZ >= [expr -$pocket(Thickness)-$pocket(DepthPerPass)+0.001] }\
	{ set CurZ [expr $CurZ-$pocket(DepthPerPass)] } { 
    puts "In for";flush stdout
    if { $CurZ < -$pocket(Thickness)} { 
	set CurZ -$pocket(Thickness) 
    }

    # plunge to new level
    OutputGcode $mopText  [set gcode ""; append gcode\
	    [GetLineNo] $sp\
	    Z [OutNum $CurZ 4] $sp\
	    F [OutNum [expr $pocket(FeedRate) / 5.0] 1] ]

    if { $MajAxis == "Xaxis" } {
	# plow down center
	set X [expr $pocket(llX) + $pocket(Xsize) \
		- $ToolRadius - $pocket(FinishWOC)]
	
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		X [OutNum $X 4] $sp\
		F [OutNum [expr $pocket(FeedRate) / 2.0] 1] ]

	set done false
	set Offset 0.0

	while { ! $done } {
	    set Offset [expr $Offset + $pocket(WidthOfCut)]
	    
	    if { $Offset >= [expr $pocket(Ysize)/2.0 \
		    - $ToolRadius - $pocket(FinishWOC)] } {
		set Offset [expr $pocket(Ysize)/2.0 \
			- $ToolRadius - $pocket(FinishWOC)]
		set done $true;
	    }
	    
	    set Y [expr $pocket(llY) + $pocket(Ysize)/2.0 + $Offset]
	    OutputGcode $mopText  [set gcode ""; append gcode\
		    [GetLineNo] $sp\
		    Y [OutNum $Y 4] $sp\
		    F [OutNum $pocket(FeedRate) 1] ]
	    
	    set X [expr $pocket(llX) + $ToolRadius + $pocket(FinishWOC)]
	    OutputGcode $mopText  [set gcode ""; append gcode\
		    [GetLineNo] $sp\
		    X [OutNum $X 4] $sp]
	    
	    set Y [expr $pocket(llY) + $pocket(Ysize)/2.0 - $Offset]
	    OutputGcode $mopText  [set gcode ""; append gcode\
		    [GetLineNo] $sp\
		    Y [OutNum $Y 4]]
	    
	    set X [expr $pocket(llX) + $pocket(Xsize)\
		    - $ToolRadius - $pocket(FinishWOC)]
	    OutputGcode $mopText  [set gcode ""; append gcode\
		    [GetLineNo] $sp\
		    X [OutNum $X 4] $sp]
	    # end while
	}
	# go back to starting point 
	set Y [expr $pocket(llY) + $pocket(Ysize)/2.0]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		Y [OutNum $Y 4] $sp\
		F [OutNum [expr $pocket(FeedRate)*4.0] 1] ]
	
	set X [expr $pocket(llX) + $ToolRadius + $pocket(FinishWOC)]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		X [OutNum $X 4] $sp]
	
    } else {
	# MajAxis == YAxis
	# plow down center
	set Y [expr $pocket(llY) + $pocket(Ysize) \
		- $ToolRadius - $pocket(FinishWOC)]
	
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		Y [OutNum $Y 4] $sp\
		F [OutNum [expr $pocket(FeedRate) / 2.0] 1] ]

	set done $false
	set Offset 0.0

	while { ! $done } {
	    set Offset [expr $Offset + $pocket(WidthOfCut)]
	    
	    if { $Offset >= [expr $pocket(Xsize)/2.0 \
		    - $ToolRadius - $pocket(FinishWOC)] } {
		set Offset [expr $pocket(Xsize)/2.0 \
			- $ToolRadius - $pocket(FinishWOC)]
		set done $true;
	    }

	    set X [expr $pocket(llX) + $pocket(Xsize)/2.0 - $Offset]
	    OutputGcode $mopText  [set gcode ""; append gcode\
		    [GetLineNo] $sp\
		    X [OutNum $X 4] $sp\
		    F [OutNum $pocket(FeedRate) 1] ]
	    
	    set Y [expr $pocket(llY) + $ToolRadius + $pocket(FinishWOC)]
	    OutputGcode $mopText  [set gcode ""; append gcode\
		    [GetLineNo] $sp\
		    Y [OutNum $Y 4] $sp]
	    
	    set X [expr $pocket(llX) + $pocket(Xsize)/2.0 + $Offset]
	    OutputGcode $mopText  [set gcode ""; append gcode\
		    [GetLineNo] $sp\
		    X [OutNum $X 4]]
	    
	    set Y [expr $pocket(llY) + $pocket(Ysize)\
		    - $ToolRadius - $pocket(FinishWOC)]
	    OutputGcode $mopText  [set gcode ""; append gcode\
		    [GetLineNo] $sp\
		    Y [OutNum $Y 4] $sp]
	    # end while
	}
	# go back to starting point 
	set X [expr $pocket(llX) + $pocket(Xsize)/2.0]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		X [OutNum $X 4] $sp\
		F [OutNum [expr $pocket(FeedRate)*4.0] 1] ]

	set Y [expr $pocket(llY) + $ToolRadius + $pocket(FinishWOC)]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		Y [OutNum $Y 4] $sp]
	

    }
    if { $MajAxis == "Xaxis" } {
	# step out to finish dimension 
	set CurX [expr $pocket(llX) + $ToolRadius + 0.25 * $pocket(FinishWOC)]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		F [OutNum $pocket(FinishFeed) 1] $sp\
		X [OutNum $CurX 4] $sp]

	# go into corner
	set CurX [expr $pocket(llX) + $ToolRadius]
	set CurY [expr $pocket(llY) + $ToolRadius]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		X [OutNum $CurX 4] $sp\
		Y [OutNum $CurY 4]]


	set X [expr $pocket(llX) - $ToolRadius + $pocket(Xsize)]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		X [OutNum $X 4]]

	set Y [expr $pocket(llY) - $ToolRadius + $pocket(Ysize)]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		Y [OutNum $Y 4]]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		X [OutNum $CurX 4]]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		Y [OutNum $CurY 4]]
    } else {
	# MajAxis == YAxis
	# step out to finish dimension 
	set CurY [expr $pocket(llY) + $ToolRadius + 0.25 * $pocket(FinishWOC)]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		F [OutNum $pocket(FinishFeed) 1] $sp\
		Y [OutNum $CurY 4] $sp]

	# go into corner
	set CurY [expr $pocket(llY) + $ToolRadius]
	set CurX [expr $pocket(llX) - $ToolRadius + $pocket(Xsize)]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		Y [OutNum $CurY 4] $sp\
		X [OutNum $CurX 4]]


	set Y [expr $pocket(llY) - $ToolRadius + $pocket(Ysize)]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		Y [OutNum $Y 4]]

	set X [expr $pocket(llX) + $ToolRadius]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		X [OutNum $X 4]]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		Y [OutNum $CurY 4]]
	OutputGcode $mopText  [set gcode ""; append gcode\
		[GetLineNo] $sp\
		X [OutNum $CurX 4]]
    }
}
 
;# for CurZ

# raise spindle above work 
OutputGcode $mopText  [set gcode ""; append gcode\
	[GetLineNo] $sp\
	$sp Z0.1]

# End of Program - this should be inserted when a nc file is closed.
# This must go outside the <MOP> </MOP> wrapper.  Is this really
# needed?

#OutputGcode $mopText  [set gcode ""; append gcode\
#	[GetLineNo] $sp\
#	M02]

