# makeRBezel.tcl
# written by
# Lawrence Glaister VE7IT Nov 2000  lg@jfm.bc.ca
# engraves a round control bezel

# make sure cpUtils.tcl has been sourced once
if ![info exists cpUtilsSourced] {
    source cpUtils.tcl
    puts "Sourced cpUtils.tcl"
}


# global variables
set LineNo     1000
set TwoPi      6.283185307
set DegToRad   [expr $TwoPi/360.0]

# Note, use append to build gcode argument to OutputCgode because 
# of better control of whitespaces.
# Lin Interp. and go to initial pos

# get cutter to safe height
OutputGcode $mopText  [set gcode ""; append gcode\
    [GetLineNo] $sp\
    G0 $sp \
    Z [OutNum $rbezel(SafeZ) 4] $sp (retract_to_safe_height)] 

# set engraving speed
OutputGcode $mopText  [set gcode ""; append gcode\
    [GetLineNo] $sp\
    F [OutNum $rbezel(FeedRate) 1] $sp (set_engraving_rate)]  
     
puts "Engraving a round bezel at $rbezel(CenterX),$rbezel(CenterY)"    
# mark center with a point for cases when manually drilling center holes
OutputGcode $mopText  [set gcode ""; append gcode\
    [GetLineNo] $sp  \
    G00 $sp X  $sp  \
    [OutNum $rbezel(CenterX) 4] $sp\
    Y $sp [OutNum $rbezel(CenterY) 4] (RBezel - Mark_Center_Hole)]       
# set to cutting height
OutputGcode $mopText  [set gcode ""; append gcode\
    [GetLineNo] $sp\
    G01 $sp Z $sp\
    [OutNum $rbezel(CutZ) 4]]    
# get cutter to safe height
OutputGcode $mopText  [set gcode ""; append gcode\
    [GetLineNo] $sp\
    G00 $sp \
    Z [OutNum $rbezel(SafeZ) 4] ] 
      

set deltaangle [expr $rbezel(EndAngle) - $rbezel(StartAngle)]
set deltaangle [expr $deltaangle / [expr $rbezel(NumTicks)- 1]]

for { set seg 0 } { $seg < $rbezel(NumTicks) } { incr seg } {

    set angle [expr $rbezel(StartAngle) + $seg * $deltaangle]
    # calculate co-ordinates of inner point on tick mark
    set x [expr $rbezel(ArcRadius) * cos( $angle * $DegToRad ) \
           + $rbezel(CenterX)]
    set y [expr $rbezel(ArcRadius) * sin( $angle * $DegToRad ) \
           + $rbezel(CenterY)]

    # move to inner radius of tick mark
    OutputGcode $mopText  [set gcode ""; append gcode\
        [GetLineNo] $sp  \
        G00 $sp X  $sp  \
        [OutNum $x 4] $sp\
        Y $sp [OutNum $y 4]]       

    # set to cutting height
    OutputGcode $mopText  [set gcode ""; append gcode\
        [GetLineNo] $sp\
        G01 $sp Z $sp\
        [OutNum $rbezel(CutZ) 4]]    
       
    if { [expr $seg %  $rbezel(MajorEvery)] == 0 } { 
        set rad $rbezel(MajorRadius)
    } else { set rad $rbezel(MinorRadius) }

    # compute outer point of tick mark
    set x [expr $rad * cos( $angle * $DegToRad ) + $rbezel(CenterX)]
    set y [expr $rad * sin( $angle * $DegToRad ) + $rbezel(CenterY)]

    # cut to outer radius of tick mark
    OutputGcode $mopText  [set gcode ""; append gcode\
        [GetLineNo] $sp  \
        G01 $sp X  $sp  \
        [OutNum $x 4] $sp\
        Y $sp [OutNum $y 4]]        
   
    # raise engraver
    OutputGcode $mopText  [set gcode ""; append gcode\
        [GetLineNo] $sp\
        G00 $sp Z $sp\
        [OutNum $rbezel(SafeZ) 4]]
             

}

