#!/bin/sh
# the next line restarts using emcsh \
exec plat/nonrealtime/bin/emcsh "$0" "$@"

# TclTk script for tkemc gui

# Notes
# This script uses a text widget for EMC mdi mode program entry.
# Written by Ray Henry <rehenry@up.net>
# You may have to edit line three above for the location of emcsh on your system.

emc_set_wait received
set programEntry [emc_ini PROGRAM_PREFIX DISPLAY]
set mdifilename ${programEntry}newfile.ngc

wm title . "Block MDI Entry"

# create the main window top frame
set top [frame .mdiblock ]

set top1 [frame $top.f1]

label $top1.label -justify center -text "\
    This script runs text as blocks of mdi code.
    Pressing run executes all lines."
frame $top1.sep -borderwidth 4 -relief groove -bg black
pack $top1.label $top1.sep -side top -fill x

set top1l [frame $top1.fl]
button $top1l.button1 -text "Insert Text File : " -anchor e -bd 2 \
    -fg darkred -command {fileDialog}
pack $top1l.button1 -side top -fill x
pack $top1l -side left -pady 4

set top1r [frame $top1.fr]
label $top1r.label1 -textvariable mdifilename -anchor e -relief groove
pack $top1r.label1 -side top -fill x -padx 4 -expand yes
pack $top1r -side left -fill x
pack $top1 -side top  -fill x

set top2 [frame $top.f2]
text $top2.text  -height 8 -width 50 -takefocus 1
pack $top2.text -side top -expand yes -fill both
pack $top2 -side top -expand yes -fill both

set top3 [frame $top.control -borderwidth 4 -relief groove ]
button $top3.button1 -bd 2 -font {Helvetica -12 bold} -text "Save" \
    -command {saveFileAs}
button $top3.button2 -bd 2 -bg darkgreen -fg white -highlightcolor black \
    -font {Helvetica -12 bold} -text "Run Text" -command {runStart}
button $top3.button3 -bd 2 -bg yellow \
    -font {Helvetica -12 bold} -text "ABORT" -command {emc_abort}
button $top3.button4 -bd 2 -text Cancel  -command {runExit}
button $top3.button5 -textvariable stoptext -command {subroutineStop} \
    -bg red -activebackground #f84c50 -width 10
pack $top3.button1 $top3.button2 $top3.button5 $top3.button3 $top3.button4 -side left -fill x
pack $top3 -side left -fill x
pack $top -side top -expand yes

set restartline 0

proc runStart {} {
    if {[emc_mode] != "mdi"} {
        emc_mode mdi
    }
    runMain 0
}

set textwin $top2.text
proc runMain {number} {
    global textwin
    set firstplace 1.0
    scan [$textwin index end] %d nl
    for {set i 1} {$i < $nl} {incr i} {
        set mdistring [$textwin get $i.0 $i.end]
        emc_mdi $mdistring
        emc_wait received
        puts "$i $mdistring"
    }
}

proc runExit {} {
    emc_abort
    emc_mode manual
    exit
    destroy .
}

proc fileDialog {} {
    global top programEntry mdifilename
    set types {
        {"NC files" {.nc .ngc}}
        {"Text files" {.txt}}
        {"All files" *}
    }
    set temp [tk_getOpenFile -filetypes $types -parent $top -initialdir $programEntry]
    if {$temp == ""} {
        return
    } else {
        set mdifilename $temp
        loadProgramText
    }
}

proc loadProgramText {} {
    global mdifilename lastline top2
    # clear out program text
    $top2.text config -state normal
    $top2.text delete 1.0 end
    # close the current program
    catch {close $programin}
    # open the new program, if it's not "none"
    if {[catch {open $mdifilename} programin]} {
        puts stdout "can't open $mdifilename"
    } else {
        $top2.text insert end [read $programin]
        catch {close $programin}
    }
}

proc saveFileAs {} {
    global mdifilename
    set fname [tk_getSaveFile -initialfile $mdifilename]
    if {[string length $fname] == 0} {
        return
    }
    set mdifilename $fname
    geneditSaveFile $fname
}

proc geneditSaveFile {name} {
    global mdifilename textwin
    set ed .$name
    catch {file copy -force $mdifilename $mdifilename.bak}
    if {[catch {open $mdifilename w} fileout]} {
        puts stdout "can't save $mdifilename"
        return
    }
    puts $fileout [$textwin get 1.0 end]
    catch {close $fileout}
}

set stoptext "FEEDHOLD"
set feedoverride [emc_feed_override]
set lastfeedoverride 100
proc subroutineStop {} {
    global stoptext top3
    global feedoverride lastfeedoverride
    set temp [emc_feed_override]
    if {$temp != 0} {set lastfeedoverride $temp}
    switch -- $stoptext {
        FEEDHOLD {
            set feedoverride 0
            emc_feed_override $feedoverride
            $top3.button5 configure -bg green -activebackground lightgreen
            set stoptext "CONTINUE"
        }
        CONTINUE {
            set feedoverride $lastfeedoverride
            emc_feed_override $feedoverride
            $top3.button5 configure -bg red -activebackground #f84c50
            set stoptext "FEEDHOLD"
        }
        default {
        }
    }
}

proc loop {} {
    global stoptext

    if {[emc_estop] == "on"} {
        emc_abort
        emc_mode manual
        emc_task_plan_init
    }

    if {$stoptext == "CONTINUE"} {
        if {[emc_feed_override] > 0} {
            subroutineStop
        }
    }

    after 200 loop
}

loop
