#!/bin/sh
# the next line restarts using emcsh \
exec plat/nonrealtime/bin/emcsh "$0" "$@"

# Program: tkbackplot
# Version: 2.2.1
# Tcl version: 8.0
# Tk version: 8.0
#
# Description:
# This version of tkbackplot is designed to 'plug in to' tkemc.
# features include a zoom and 3 axis rotate without having to re-run the
# program each time.
# To use, copy this file into emc/plat/nonrealtime/bin rename it
# tkbackplot and make it executable (if it isn't already).
#
# 14-11-00 (PC) EMC now pauses during a redraw if it was running.
#                         Data storage has been improved.

# These set the axis directions.  +1 one plots like graph
# paper with positive up and to the right.
# to reverse an axis change its value to -1 (don't put other numbers here)
set xdir 1
set ydir -1
set zdir -1
set adir 1
set bdir 1
set cdir 1

# These set top left and bottom right of the canvas
global topx topy bot x boty
set topx -1000
set botx  1000
set topy -500
set boty 500

# This variable sets the default size of the plot and can be thought of as number of
# pixels per inch of machine motion.  The default (40) will give about .55 screen
# inch per inch moved.  

set size 40

# Set the colours for different modes here
set plotcolour {limegreen black red blue yellow3 white}

# Evaluate the number of clock clicks per second
# Value dependent on machine !!
set t1 [clock clicks]
after 10000
set t2 [clock clicks]
set ticks [expr ($t2-$t1)/10000]

# procedure to show window .
proc popupPlot {} {

    set d .plot

    if {[winfo exists $d]} {
	wm deiconify $d
	raise $d
	focus $d
	return
    }
    toplevel $d
    wm title $d "TkEmc BackPlot"

  # build widget .plot.view
  frame .plot.view \
    -borderwidth {2} \
    -relief {raised} \
    -width {150}

  # build widget .plot.view.3dview
  canvas .plot.view.3dview \
    -background linen \
    -height {150} \
    -highlightbackground {#ffffff} \
    -highlightcolor {#000000} \
    -relief {raised} \
    -width {150}

  # build widget .plot.view.redraw
  button .plot.view.redraw \
    -activebackground {#ff6ec7} \
    -activeforeground {#000000} \
    -background {#eed8ae} \
    -command redraw \
    -font {-adobe-helvetica-medium-r-normal--14-140-75-75-p-77-iso8859-1} \
    -foreground {#000000} \
    -highlightbackground {#eed8ae} \
    -highlightcolor {#000000} \
    -padx {9} \
    -pady {3} \
    -text {Refresh Plot}

  # build widget .plot.view.reset
  button .plot.view.reset \
    -activebackground {#ff2400} \
    -activeforeground {#000000} \
    -background {#eed8ae} \
    -command erasePlot \
    -font {-adobe-helvetica-medium-r-normal--14-140-75-75-p-77-iso8859-1} \
    -foreground {#000000} \
    -highlightbackground {#eed8ae} \
    -highlightcolor {#000000} \
    -padx {9} \
    -pady {3} \
    -text {Reset Display}

  # build widget .plot.view.quit
  button .plot.view.quit \
    -activebackground red \
    -activeforeground {#000000} \
    -background {#eed8ae} \
    -command {destroy .plot} \
    -font {-adobe-helvetica-medium-r-normal--14-140-75-75-p-77-iso8859-1} \
    -foreground {#000000} \
    -highlightbackground {#eed8ae} \
    -highlightcolor {#000000} \
    -padx {9} \
    -pady {3} \
    -text {Quit Backplot}

  # build widget .plot.menu
  frame .plot.menu \
    -borderwidth {2} \
    -relief {raised}

  # build widget .plot.menu.scale_z
  scale .plot.menu.scale_z \
    -activebackground {#d9d9f3} \
    -background {#eed8ae} \
    -command 3Dview \
    -font {-adobe-helvetica-medium-r-normal--14-140-75-75-p-77-iso8859-1} \
    -foreground {#000000} \
    -from {-180.0} \
    -highlightbackground {#eed8ae} \
    -highlightcolor {#000000} \
    -label { Rotate about Z } \
    -length {120} \
    -orient {horizontal} \
    -relief {raised} \
    -to {180.0} \
    -troughcolor {#eed8ae} \
    -variable {z_rotate}

  # build widget .plot.menu.scale_y
  scale .plot.menu.scale_y \
    -activebackground {#d9d9f3} \
    -background {#eed8ae} \
    -command 3Dview \
    -font {-adobe-helvetica-medium-r-normal--14-140-75-75-p-77-iso8859-1} \
    -foreground {#000000} \
    -from {-180.0} \
    -highlightbackground {#eed8ae} \
    -highlightcolor {#000000} \
    -label { Rotate about Y } \
    -length {120} \
    -orient {horizontal} \
    -relief {raised} \
    -to {180.0} \
    -troughcolor {#eed8ae} \
    -variable {y_rotate}

  # build widget .plot.menu.scale_x
  scale .plot.menu.scale_x \
    -activebackground {#d9d9f3} \
    -background {#eed8ae} \
    -command 3Dview \
    -font {-adobe-helvetica-medium-r-normal--14-140-75-75-p-77-iso8859-1} \
    -foreground {#000000} \
    -from {-180.0} \
    -highlightbackground {#eed8ae} \
    -highlightcolor {#000000} \
    -label { Rotate about X } \
    -length {120} \
    -orient {horizontal} \
    -relief {raised} \
    -to {180.0} \
    -troughcolor {#eed8ae} \
    -variable {x_rotate}

  # build widget .plot.menu.scale_zoom
  scale .plot.menu.scale_zoom \
    -activebackground {#d9d9f3} \
    -background {#eed8ae} \
    -command 3Dview \
    -font {-adobe-helvetica-medium-r-normal--14-140-75-75-p-77-iso8859-1} \
    -foreground {#000000} \
    -from {-10.0} \
    -highlightbackground {#eed8ae} \
    -highlightcolor {#000000} \
    -label { Zoom level  } \
    -length {120} \
    -orient {horizontal} \
    -relief {raised} \
    -resolution {0.1} \
    -to {10.0} \
    -troughcolor {#eed8ae} \
    -variable {zoom_level}

  # build widget .plot.3dplot
  frame .plot.3dplot \
    -borderwidth {2} \
    -height {275} \
    -relief {raised} \
    -width {500}

  # build widget .plot.3dplot.vscroll
  scrollbar .plot.3dplot.vscroll \
    -activebackground {#d9d9f3} \
    -background {#eed8ae} \
    -command {.plot.3dplot.3dplot yview} \
    -cursor {left_ptr} \
    -highlightbackground {#eed8ae} \
    -highlightcolor {#000000} \
    -relief {raised} \
    -troughcolor {#eed8ae}

  # build widget .plot.3dplot.hscroll
  scrollbar .plot.3dplot.hscroll \
    -activebackground {#d9d9f3} \
    -background {#eed8ae} \
    -command {.plot.3dplot.3dplot xview} \
    -cursor {left_ptr} \
    -highlightbackground {#eed8ae} \
    -highlightcolor {#000000} \
    -orient {horizontal} \
    -relief {raised} \
    -troughcolor {#eed8ae}

  # build widget .plot.3dplot.3dplot
  canvas .plot.3dplot.3dplot \
    -background white \
    -highlightbackground {#ffffff} \
    -highlightcolor {#000000} \
    -relief {raised} \
    -scrollregion {-1500 -750 1500 750} \
    -xscrollcommand {.plot.3dplot.hscroll set} \
    -yscrollcommand {.plot.3dplot.vscroll set}

  # pack master .plot
  pack configure .plot.view \
    -fill y \
    -side right
  pack configure .plot.menu \
    -fill y \
    -side right
  pack configure .plot.3dplot \
    -expand 1 \
    -fill both \
    -side left

  # pack master .plot.view
  pack configure .plot.view.3dview \
    -anchor n \
    -fill both
  pack configure .plot.view.redraw \
    -expand 1 \
    -fill both
  pack configure .plot.view.reset \
    -expand 1 \
    -fill both
  pack configure .plot.view.quit \
    -expand 1 \
    -fill both

  # pack master .plot.menu
  pack configure .plot.menu.scale_z \
    -expand 1 \
    -fill both
  pack configure .plot.menu.scale_y \
    -expand 1 \
    -fill both
  pack configure .plot.menu.scale_x \
    -expand 1 \
    -fill both
  pack configure .plot.menu.scale_zoom \
    -expand 1 \
    -fill both

  # pack master .plot.3dplot
  pack configure .plot.3dplot.vscroll \
    -fill y \
    -side left
  pack configure .plot.3dplot.hscroll \
    -fill x \
    -side bottom
  pack configure .plot.3dplot.3dplot -expand 1 -fill both -side left

  # pack master .
#  pack configure .plot -expand 1 -fill both

  # build canvas items .plot.view.3dview
  # build canvas items .plot.3dplot.3dplot
    global Xlast Ylast Zlast Alast Blast Clast
    global X Y Z A B C 3d_plot
            set 3d_plot {0 0 0 0}
	set Xlast [format "%9.4f" $X ]
	set Ylast [format "%9.4f" $Y ]
	set Zlast [format "%9.4f" $Z ]
 	set Alast [format "%9.4f" $A ]
	set Blast [format "%9.4f" $B ]
	set Clast [format "%9.4f" $C ]
erasePlot
}

proc vector {} {
    global X Y Z A B C
# 3D vector conversion
# X Y and Z point is converted into polar notation
# then rotated about the A B and C axis.
# Finally to be converted back into rectangular co-ords.
#
# As the three lines of the 3D Origin need to be resolved as well as well
# as the cutter position, I thought a proc would be more efficient. 

# Rotate about A - X axis
        set angle [expr $A * 0.01745329]
     if { $Y != 0 || $Z != 0 } {
	set angle [expr atan2($Y,$Z) + $angle]
	}
	set vector [expr hypot($Y,$Z)]
	set Z [expr $vector * cos($angle)]
	set Y [expr $vector * sin($angle)]

# Rotate about B - Y axis
        set angle [expr $B * 0.01745329]
     if { $X != 0 || $Z != 0 } {
	set angle [expr atan2($Z,$X) + $angle]
	}
	set vector [expr hypot($X,$Z)]
	set X [expr $vector * cos($angle)]
	set Z [expr $vector * sin($angle)]

# Rotate about C - Z axis
        set angle [expr $C * 0.01745329]
     if { $X != 0 || $Y != 0 } {
	set angle [expr atan2($Y,$X) + $angle]
	}
	set vector [expr hypot($X,$Y)]
	set X [expr $vector * cos($angle)]
	set Y [expr $vector * sin($angle)]
  }

# initialize global variables
global {3dview}
set 3dview {.plot.view.3dview}
global {3dplot}
set 3dplot  {.plot.3dplot.3dplot}
global {X}
set {X} {0}
global {Y}
set {Y} {0}
global {Z}
set {Z} {0}
global {A}
set {A} {0}
global {B}
set {B} {0}
global {C}
set {C} {0}
global {x_rotate}
set {x_rotate} {0}
global {xlast}
set {xlast} {0}
global {xnext}
set {xnext} {0}
global {y_rotate}
set {y_rotate} {0}
global {ylast}
set {ylast} {0}
global {ynext}
set {ynext} {0}
global {z_rotate}
set {z_rotate} {0}
global {zoom}
set {zoom} {0}
global {zoom_level}
set {zoom_level} {0}
 
set lastline ??
set Gmode 4
set Glast 5
set delta_Alast 0
set delta_Blast 0
set delta_Clast 0

proc erasePlot {} {
    global 3dplot topx topy botx boty
    global X Z centerx centery
    global 3d_plot
    global Xlast Ylast Zlast Glast

# Get current canvas size
set plotx [$3dplot cget -width]
set ploty [$3dplot cget -height]

# Computes view (0,0) and attempts to center it
set totalx [expr {abs($topx) + abs($botx)}] 
set lowerx [expr {abs($topx) - ($plotx / 2)}]
if {$lowerx <= 0} {
    set centerx 0
    } else {
    set centerx [expr {double($lowerx) / double($totalx)*1.05}]
}

set totaly [expr {abs($topy) + abs($boty)}] 
set lowery [expr {abs($topy) - ($ploty / 2)}]
if {$lowery <= 0} {
    set centery 0
    } else {
    set centery [expr {double($lowery) / double($totaly)*1.05}]
}
    $3dplot addtag all all
    $3dplot delete all
    $3dplot xview moveto $centerx
    $3dplot yview moveto $centery

# Delete all plot data except the last one.
set 3d_plot [list $Xlast $Ylast $Zlast $Glast]

3Dview
redraw
 }

# Procedure: 3Dview
proc 3Dview {args} {
global x_rotate y_rotate z_rotate
global 3dview plotcolour
global X Y Z A B C
global 3d_plot a_plot b_plot c_plot

set 3d_object {25 25 25 3 25 -25 25 3 -25 -25 25 3 -25 25 25 3 25 25 25 3 \
    0 0 -30 2 -25 -25 25 2 -25 25 25 3 0 0 -30 2 25 -25 25 4}

set atemp $A; set btemp $B; set ctemp $C

set A $x_rotate ; set B $y_rotate ; set C $z_rotate

# Delete the image and put in an origin piont.
    $3dview delete all

$3dview create oval 72 72 78 78 -fill green

# set the first set of coordinates from the list then call vector
set X [lindex $3d_object 0]
set Y [lindex $3d_object 1]
set Z [lindex $3d_object 2]
set colour [lindex $3d_object 3]

vector
# then use these to start the plot 
 set xlast $X ; set ylast $Z

foreach {X Y Z colour} $3d_object {
vector
set xnext $X ; set ynext $Z
$3dview create line [expr $xlast+75] [expr $ylast+75] [expr $xnext+75] [expr $ynext+75] \
  -fill [lindex $plotcolour $colour]
 set xlast $X ; set ylast $Z
 }
set A $atemp; set B $btemp; set C $ctemp
# If 3d_plot has been reset - do a redraw first
#  to update the rotates and zoom
 if {[llength $3d_plot] <=9} {
   redraw 
     }
}


# Procedure: redraw 
 proc redraw {args} {
global zoom_level zoom
global 3d_plot ticks
global xlast ylast
global x_rotate y_rotate z_rotate
global a_plot b_plot c_plot
global A B C X Y Z
global 3dplot plotcolour
global centerx centery

set atemp $A; set btemp $B; set ctemp $C
set a_plot $x_rotate
set b_plot $y_rotate
set c_plot $z_rotate
set zoom $zoom_level

   if { $zoom <0 } {
        set zoom  [ expr abs(1.0/($zoom-1)) ]
      } else {
       set zoom [ expr $zoom+1]
     }

  if {[emc_program_status] == "running" } {
    set statusflag 1
# If emc is running, issue a pause command
# and wait until IDLE status is reached.
     emc_pause
	while {[emc_program_status] == "running" } {
# Just loop until paused
	  }
    } else {
    set statusflag 0
    }
# time the redraw proc
set timer [clock clicks]

 $3dplot delete all

# set the first set of coordinates from the list then call vector
set X [lindex $3d_plot 0]
set Y [lindex $3d_plot 1]
set Z [lindex $3d_plot 2]
set colour [lindex $3d_plot 3]
    set A $x_rotate
    set B $y_rotate
    set C $z_rotate
vector

# then use these to start the plot 
 set x_last $X ; set y_last $Z
foreach {X Y Z colour} $3d_plot {
    set A $x_rotate
    set B $y_rotate
    set C $z_rotate
   vector
    set x_next $X ; set y_next $Z
   $3dplot create line [expr $x_last*$zoom+$centerx] [expr $y_last*$zoom+$centery] \
     [expr $x_next*$zoom+$centerx] [expr $y_next*$zoom+$centery] \
       -fill [lindex $plotcolour $colour]
    set x_last $X ; set y_last $Z
   }

 set xlast $X ; set ylast $Z

$3dplot create line [expr $x_last*$zoom+$centerx] [expr $y_last*$zoom+$centery] \
   [expr ($x_last*$zoom+$centerx)+5] [expr ($y_last*$zoom+$centery)-5] \
     -fill red -arrow first -tags 3darrow
 set A $atemp; set B $btemp; set C $ctemp

set timer [expr ([clock clicks] - $timer)/$ticks]
# If emc was forced to pause - restart.
  if {$statusflag == 1} {
      emc_resume
     }
puts "$timer mSec to plot [llength $3d_plot] vectors"

}


proc updatePlot {} {

    global size 3dplot
    global xdir ydir zdir zoom
    global adir bdir cdir
    global programfiletext posdigit0 posdigit1 posdigit2 activeLine plotcolour
    global posdigit3 posdigit4 posdigit5
    global Xlast Ylast Zlast Alast Blast Clast lastline
    global xlast ylast xnext ynext centerx centery
    global X Y Z A B C
    global a_plot b_plot c_plot 3d_plot Gmode Glast
    global delta_Alast delta_Blast delta_Clast

    # Color plot line by setting active line to upcase currentline
        set currentline [string toupper [$programfiletext get $activeLine.0 $activeLine.end]] 
    # Search currentline for g0-3 but make modal with no else
    if { [string first G2 $currentline] != -1 || \
        [string first G02 $currentline] != -1  } {
            set Gmode 2
    } elseif { [string first G3 $currentline] != -1 || \
        [string first G03 $currentline] != -1 } {
            set Gmode 3
    } elseif { [string first G1 $currentline] != -1 || \
        [string first G01 $currentline] != -1 } {
            set Gmode 1
    } elseif { [string first G0 $currentline] != -1 } { 
        set Gmode 0
    }
set delta_A 0
set delta_B 0
set delta_C 0

	set X [emc_abs_act_pos 0]
	set Y [emc_abs_act_pos 1]
	set Z [emc_abs_act_pos 2]
	set A [emc_abs_act_pos 3]
	set B [emc_abs_act_pos 4]
	set C [emc_abs_act_pos 5]
 
        set X [expr $X * $size * $xdir]
        set Y [expr $Y * $size * $ydir]
        set Z [expr $Z * $size * $zdir]
        set A [expr $A * $adir]
        set B [expr $B * $bdir]
        set C [expr $C * $cdir]

vector
	    set X [format "%10.1f" $X]
	    set Y [format "%10.1f" $Y]
	    set Z [format "%10.1f" $Z]

	if {$Alast != $A || $Blast != $B || $Clast != $C || $Gmode != $Gmode} {
	  lappend 3d_plot $X $Y $Z $Glast
	} else {
	  if {$Xlast != $X || $Ylast != $Y || $Zlast != $Z} {
# Test to see if move is a straight line
	    set d_x [expr $X-$Xlast]
	    set d_y [expr $Y-$Ylast]
	    set d_z [expr $Z-$Zlast]
# Calculate the angles for the relative move
     if { $d_z != 0 } {
	set delta_A [expr atan2($d_y,$d_z)]
	} else {
	set delta_A 1.570796327
	}
     if { $d_x != 0 } {
	set delta_B [expr atan2($d_z,$d_x)]
	} else {
	set delta_B 1.570796327
	}
     if { $d_x != 0 } {
	set delta_C [expr atan2($d_y,$d_x)]
	} else {
	set delta_C 1.570796327
	}
# If the relative angles of the current move are the same
#  as the last relative move - Then it *must* be a continuation
# of a straight line move !
	  if { $delta_A != $delta_Alast || $delta_B != $delta_Blast \
		|| $delta_C != $delta_Clast } {
# If not, save the vector.
	  lappend 3d_plot $Xlast $Ylast $Zlast $Glast
	  }
		set delta_Alast $delta_A
		set delta_Blast $delta_B
		set delta_Clast $delta_C

	  }
	}
	set Xlast $X
	set Ylast $Y
	set Zlast $Z
	set Alast $A
	set Blast $B
	set Clast $C
	set Glast $Gmode

# Plot 3D xy and move tool if X Y or Z has changed
	set A $a_plot; set B $b_plot; set C $c_plot
	vector
	set xnext $X ; set ynext $Z
	  if {$Gmode >= 5} {
	        set plotcolor yellow
	     } else {
	        set plotcolor [lindex $plotcolour $Gmode]
	    }
	$3dplot create line [expr $xlast*$zoom+$centerx] [expr $ylast*$zoom+$centery] \
	   [expr $xnext*$zoom+$centerx] [expr $ynext*$zoom+$centery] \
	    -fill [lindex $plotcolour $Gmode]
	$3dplot move 3darrow [expr ($xnext - $xlast)*$zoom] [expr ($ynext - $ylast)*$zoom]
	set xlast $X ; set ylast $Z
}



