
# emccalib.tcl
# EMC parameter setting application
# Needs emcsh to run-- this is sourced by tkemc, but it can be run
# standalone. Make sure directory containing emcsh is in your path,
# or edit the exec line above with the path to emcsh.

# check if any emc commands exist, and quit if not
if {! [string length [info commands emc_plat]]} {
    error "emccalib needs to run from \"emcsh\""
}

if {! [info exists activeAxis]} {
    set activeAxis 0
}

proc popdownCalibration {w} {
    killPopin $w
    if {$w == "."} {
        exit
    }
}

proc popinCalibration {{w .calibwindow}} {
    global axisentry pgain igain dgain ff0 ff1 ff2
    global backlash bias maxerror deadband
    global activeAxis popinframe

    # use "lw" as name of top level from now on
set lw [frame $popinframe.calib]
set w $lw
pack $lw

    # force the selected axis to be the active axis
    set axisentry $activeAxis

    # get the parameters from the controller
    set pgain [emc_axis_gains $activeAxis p]
    set igain [emc_axis_gains $activeAxis i]
    set dgain [emc_axis_gains $activeAxis d]
    set ff0 [emc_axis_gains $activeAxis ff0]
    set ff1 [emc_axis_gains $activeAxis ff1]
    set ff2 [emc_axis_gains $activeAxis ff2]
    set backlash [emc_axis_gains $activeAxis backlash]
    set bias [emc_axis_gains $activeAxis bias]
    set maxerror [emc_axis_gains $activeAxis maxerror]
    set deadband [emc_axis_gains $activeAxis deadband]

    label $lw.title -text "Axis Calibration" -width 30 -anchor center

    frame $lw.axis
    label $lw.axis.lab -text "Axis:" -anchor w
    entry $lw.axis.ent -width 12 -textvariable axisentry
    pack $lw.axis.lab -side left
    pack $lw.axis.ent -side right

    frame $lw.p
    label $lw.p.lab -text "P Gain:" -anchor w
    entry $lw.p.ent -width 12 -textvariable pgain
    pack $lw.p.lab -side left
    pack $lw.p.ent -side right

    frame $lw.i
    label $lw.i.lab -text "I Gain:" -anchor w
    entry $lw.i.ent -width 12 -textvariable igain
    pack $lw.i.lab -side left
    pack $lw.i.ent -side right

    frame $lw.d
    label $lw.d.lab -text "D Gain:" -anchor w
    entry $lw.d.ent -width 12 -textvariable dgain
    pack $lw.d.lab -side left
    pack $lw.d.ent -side right

    frame $lw.ff0
    label $lw.ff0.lab -text "Pos FF Gain:" -anchor w
    entry $lw.ff0.ent -width 12 -textvariable ff0
    pack $lw.ff0.lab -side left
    pack $lw.ff0.ent -side right

    frame $lw.ff1
    label $lw.ff1.lab -text "Vel FF Gain:" -anchor w
    entry $lw.ff1.ent -width 12 -textvariable ff1
    pack $lw.ff1.lab -side left
    pack $lw.ff1.ent -side right

    frame $lw.ff2
    label $lw.ff2.lab -text "Acc FF Gain:" -anchor w
    entry $lw.ff2.ent -width 12 -textvariable ff2
    pack $lw.ff2.lab -side left
    pack $lw.ff2.ent -side right

    frame $lw.backlash
    label $lw.backlash.lab -text "Backlash:" -anchor w
    entry $lw.backlash.ent -width 12 -textvariable backlash
    pack $lw.backlash.lab -side left
    pack $lw.backlash.ent -side right

    frame $lw.bias
    label $lw.bias.lab -text "Bias:" -anchor w
    entry $lw.bias.ent -width 12 -textvariable bias
    pack $lw.bias.lab -side left
    pack $lw.bias.ent -side right

    frame $lw.maxerror
    label $lw.maxerror.lab -text "Max Cum Error:" -anchor w
    entry $lw.maxerror.ent -width 12 -textvariable maxerror
    pack $lw.maxerror.lab -side left
    pack $lw.maxerror.ent -side right

    frame $lw.deadband
    label $lw.deadband.lab -text "Deadband:" -anchor w
    entry $lw.deadband.ent -width 12 -textvariable deadband
    pack $lw.deadband.lab -side left
    pack $lw.deadband.ent -side right

    frame $lw.buttons
    button $lw.buttons.ok -text "LOAD VALUES" -default active -command "emc_axis_gains \$activeAxis all \$pgain \$igain \$dgain \$ff0 \$ff1 \$ff2 \$backlash \$bias \$maxerror \$deadband "
    button $lw.buttons.cancel -text "CLOSE WINDOW" -command "popdownCalibration $w"

    pack $lw.buttons.ok $lw.buttons.cancel -side top -fill both
    bind $lw <Return> "emc_axis_gains \$activeAxis all \$pgain \$igain \$dgain \$ff0 \$ff1 \$ff2 \$backlash \$bias \$maxerror \$deadband; popdownCalibration $w"

    grid $lw.title $lw.axis -sticky ew -padx 4 -pady 10
    grid $lw.p $lw.ff0 -sticky ew -padx 4
    grid $lw.i $lw.ff1 -sticky ew -padx 4
    grid $lw.d $lw.ff2 -sticky ew -padx 4
    grid $lw.backlash $lw.buttons -sticky ew -padx 4
    grid $lw.bias ^ -sticky ew -padx 4
    grid $lw.maxerror ^ -sticky ew -padx 4
    grid $lw.deadband ^ -sticky ew -padx 4



}

