
# emcdebug.tcl
# Sets value of EMC_DEBUG, so you can turn on/off what you want dumped
# Needs emcsh to run-- this is sourced by tkemc, but it can be run
# standalone. Make sure directory containing emcsh is in your path,
# or edit the exec line above with the path to emcsh.

# check if any emc commands exist, and quit if not
if {! [string length [info commands emc_plat]]} {
    error "emcdebug needs to run from \"emcsh\""
}

proc distributeDebug {} {
    global debug_invalid
    global debug_config
    global debug_defaults
    global debug_versions
    global debug_task_issue
    global debug_io_points
    global debug_nml
    global debug_motion_time
    global debug_interp
    global debug_rcs
    global debug_traj
    global debug_interp_list

    # get the debug level from the EMC
    set emc_debug_level [emc_debug]

    # set the initial check boxes to correspond to current debug settings

    set debug_invalid [expr $emc_debug_level % 2]

    set emc_debug_level [expr $emc_debug_level >> 1]
    set debug_config [expr $emc_debug_level % 2]

    set emc_debug_level [expr $emc_debug_level >> 1]
    set debug_defaults [expr $emc_debug_level % 2]

    set emc_debug_level [expr $emc_debug_level >> 1]
    set debug_versions [expr $emc_debug_level % 2]

    set emc_debug_level [expr $emc_debug_level >> 1]
    set debug_task_issue [expr $emc_debug_level % 2]

    set emc_debug_level [expr $emc_debug_level >> 1]
    set debug_io_points [expr $emc_debug_level % 2]

    set emc_debug_level [expr $emc_debug_level >> 1]
    set debug_nml [expr $emc_debug_level % 2]

    set emc_debug_level [expr $emc_debug_level >> 1]
    set debug_motion_time [expr $emc_debug_level % 2]

    set emc_debug_level [expr $emc_debug_level >> 1]
    set debug_interp [expr $emc_debug_level % 2]

    set emc_debug_level [expr $emc_debug_level >> 1]
    set debug_rcs [expr $emc_debug_level % 2]

    set emc_debug_level [expr $emc_debug_level >> 1]
    set debug_traj [expr $emc_debug_level % 2]

    set emc_debug_level [expr $emc_debug_level >> 1]
    set debug_interp_list [expr $emc_debug_level % 2]
}

proc collectDebug {} {
    global debug_invalid
    global debug_config
    global debug_defaults
    global debug_versions
    global debug_task_issue
    global debug_io_points
    global debug_nml
    global debug_motion_time
    global debug_interp
    global debug_rcs
    global debug_traj
    global debug_interp_list

    set emc_debug_level $debug_interp_list

    set emc_debug_level [expr $emc_debug_level << 1]
    set emc_debug_level [expr $emc_debug_level + $debug_traj]

    set emc_debug_level [expr $emc_debug_level << 1]
    set emc_debug_level [expr $emc_debug_level + $debug_rcs]

    set emc_debug_level [expr $emc_debug_level << 1]
    set emc_debug_level [expr $emc_debug_level + $debug_interp]

    set emc_debug_level [expr $emc_debug_level << 1]
    set emc_debug_level [expr $emc_debug_level + $debug_motion_time]

    set emc_debug_level [expr $emc_debug_level << 1]
    set emc_debug_level [expr $emc_debug_level + $debug_nml]

    set emc_debug_level [expr $emc_debug_level << 1]
    set emc_debug_level [expr $emc_debug_level + $debug_io_points]

    set emc_debug_level [expr $emc_debug_level << 1]
    set emc_debug_level [expr $emc_debug_level + $debug_task_issue]

    set emc_debug_level [expr $emc_debug_level << 1]
    set emc_debug_level [expr $emc_debug_level + $debug_versions]

    set emc_debug_level [expr $emc_debug_level << 1]
    set emc_debug_level [expr $emc_debug_level + $debug_defaults]

    set emc_debug_level [expr $emc_debug_level << 1]
    set emc_debug_level [expr $emc_debug_level + $debug_config]

    set emc_debug_level [expr $emc_debug_level << 1]
    set emc_debug_level [expr $emc_debug_level + $debug_invalid]

    emc_debug $emc_debug_level
}

proc popdownDebug {w} {
    killPopin $w
    if {$w == "."} {
        exit
    }
}

proc popinDebug {{w .debugwindow}} {
    global debug_invalid
    global debug_config
    global debug_defaults
    global debug_versions
    global debug_task_issue
    global debug_io_points
    global debug_nml
    global debug_motion_time
    global debug_interp
    global debug_rcs
    global debug_traj
    global debug_interp_list
    global popinframe

    set w [frame $popinframe.debug]
    set lw $w
    pack $lw

    # use "lw" as name of top level from now on

    # set the debug variables so that they're initially checked properly
    distributeDebug

    label $lw.label -text "DEBUG SETTINGS"

    checkbutton $lw.b1 -anchor w -text "Invalid INI file entries" -variable debug_invalid -relief flat
    checkbutton $lw.b2 -anchor w -text "Configuration information" -variable debug_config -relief flat
    checkbutton $lw.b3 -anchor w -text "Use of defaults" -variable debug_defaults -relief flat
    checkbutton $lw.b4 -anchor w -text "Version information" -variable debug_versions -relief flat
    checkbutton $lw.b5 -anchor w -text "Command issuing" -variable debug_task_issue -relief flat
    checkbutton $lw.b6 -anchor w -text "IO points" -variable debug_io_points -relief flat
    checkbutton $lw.b7 -anchor w -text "NML" -variable debug_nml -relief flat
    checkbutton $lw.b8 -anchor w -text "Motion time" -variable debug_motion_time -relief flat
    checkbutton $lw.b9 -anchor w -text "Interpreter" -variable debug_interp -relief flat
    checkbutton $lw.b10 -anchor w -text "RCS" -variable debug_rcs -relief flat
    checkbutton $lw.b11 -anchor w -text "Trajectory level" -variable debug_traj -relief flat
    checkbutton $lw.b12 -anchor w -text "Interpreter list" -variable debug_interp_list -relief flat

    frame $lw.buttons
    button $lw.buttons.ok -text OK -default active -command "collectDebug; popdownDebug $w"
    button $lw.buttons.cancel -text Cancel -command "popdownDebug $w"
    pack $lw.buttons.ok $lw.buttons.cancel -side left -expand 1
    bind $w <Return> "emc_debug [collectDebug] ; popdownDebug $w"

    grid $lw.label -sticky ew -pady 10
    grid configure $lw.label -columnspan 2
    grid $lw.b1 $lw.b7 -sticky ew -padx 4
    grid $lw.b2 $lw.b8 -sticky ew -padx 4
    grid $lw.b3 $lw.b9 -sticky ew -padx 4
    grid $lw.b4 $lw.b10 -sticky ew -padx 4
    grid $lw.b5 $lw.b11 -sticky ew -padx 4
    grid $lw.b6 $lw.b12 -sticky ew -padx 4
    grid $lw.buttons
    grid configure $lw.buttons -columnspan 2
}

