#!/bin/sh
# the next line restarts using emcsh \
exec plat/nonrealtime/bin/emcsh "$0" "$@"

# Extended Tk GUI for the Enhanced Machine Controller
# GPL Copyright 2001 Raymond E. Henry <rehenry@up.net>
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


# Notes

# 1. Widget focus. Keyboard traversal with TAB sets the focus to some of the
# widgets.  There may be binding problems hanging around.

# 2. Adds tkbackplot and scripts to menu.
# 3. Adds info to help menu that calls tcl.sysinfo.tcl and uses emc/emcversion
# 4. Auto updates and uses separate library emc/tcl/lib/tkemc/ for processes
# 5. Added settings->size to allow experimentation with some visual characteristics

set tkemc 1

# Set the source directory for other tcl files sourced here
set tclsource [file join [pwd] tcl ]
set exsourced [file join $tclsource sourced]

# update emc tcl lib directories using tcl/lib
set libemctcl [file join $tclsource lib]
source [file join $libemctcl updateLib]
updateLib $libemctcl

# read the application defaults
foreach f {TkEmc /usr/X11R6/lib/X11/app-defaults/TkEmc} {
    if {[file exists $f]} {
        option readfile $f startupFile
        break
    }
}

# test for windows flag
set windows 0

foreach arg $argv {
    if { $arg == "-win" } {
	set windows 1
    }
}

# pop up the about box
proc popupAboutx {} {
    if {[winfo exists .about]} {
        wm deiconify .about
        raise .about
        focus .about
        return
    }
    toplevel .about
    wm title .about "About TkEmc"
    message .about.msg -aspect 1000 -justify center -font {Helvetica 12 bold} -text "TkEmcex \
        \n\nTcl/Tk GUI for Enhanced Machine Controller\n\nGPL Copyright 2001 \
        \nRay Henry <rehenry@up.net>
        \nThis software comes with ABSOLUTELY NO GUARANTEE!  \
        \nFor details see the copyright.html file in this directory."
    frame .about.buttons
    button .about.buttons.ok -default active -text OK -command "destroy .about"
    pack .about.msg -side top
    pack .about.buttons -side bottom -fill x -pady 2m
    pack .about.buttons.ok -side left -expand 1
    bind .about <Return> "destroy .about"
}


# Read the ini file to determine what the axes and coordinates are.

set worldlabellist ""
set axiscoordmap ""
set numaxes [emc_ini "AXES" "TRAJ"]
set coordnames [ emc_ini "COORDINATES" "TRAJ"]
set coords [emc_ini "POSITION_OFFSET" "DISPLAY"]
set actcmd [emc_ini "POSITION_FEEDBACK" "DISPLAY"]
set numcoords 0
setCoordNamesx

# set the cycle time for display updating
set temp [emc_ini "CYCLE_TIME" "DISPLAY"]
if {[string length $temp] == 0} {
    set temp 0.200
}
set displayCycleTime [int [expr $temp * 1000 + 0.5]]

# set the debounce time for key repeats
set debounceTime 150
set wmgeom "620x440+0+0"

# Test for new emc commands for joint and kinematics
# isnew is used in several places to set old vs new conditions
set isnew "no"
catch {set isnew [emc_kinematics_type]}

# use the top-level window as our top-level window, and name it
wm title . "TkEMCex"
wm geometry . $wmgeom

# set several size variables for initial startup
set popinheight "10"
set popinwidth "130"
set downheight "40"

set top [frame .top ]
pack $top -side bottom -fill both -expand yes

# Pick up background color for background resets
set defbg [option get $top background background]
set deffg [option get $top foreground foreground]

# add a 2 col 3 row grid within top frame.
set left [frame $top.left]
set right [frame $top.right]
set up [frame $top.up]
set down [frame $top.down]
set mdiframe [frame $down.mdi]
set autoframe [frame $down.auto]
set manframe [frame $down.manual]
set setframe [frame $down.set]

grid $up -row 0 -column 0 -sticky nsew
grid $right -row 1 -column 1 -sticky nsew
grid columnconfigure $top 1 -minsize $popinwidth
grid rowconfigure $top 1 -minsize $popinheight
grid $left -row 1 -column 0 -sticky nsew
grid $down -row 3 -column 0 -sticky nsew
grid configure $up -columnspan 2
grid configure $down -columnspan 2
grid rowconfigure $top 3 -minsize $downheight

# Set a variable to the location of the popin window
set popinframe $right

if { $windows == 0 } {

    # load the backplotter
    source $exsourced/exbackplot.tcl

    # load the EMC calibrator
    source $exsourced/excalib.tcl

    # load the generic editor "startEditor <name>"
    source $exsourced/exgenedit.tcl

    # load the EMC data logger
    source $exsourced/exlog.tcl

    # load the EMC performance tester
    source $exsourced/extesting.tcl

    # load the EMC tuning program
    source $exsourced/extuning.tcl

    # load the EMC debug level setter
    if {$isnew != "no" } {
        source $exsourced/exdebug.tcl
    }

    # load balloon help
    source tcl/lib/tkemc/balloon.tcl
    source tcl/lib/tkemc/emchelp.tcl
    set do_balloons [emc_ini "BALLOON_HELP" "DISPLAY"]
    if {$do_balloons == ""} {
        set do_balloons 0
    }
}

# the line from which to run the program; 0 or 1 means from start
set runMark 0

set toolfilename [emc_ini "TOOL_TABLE" "EMCIO"]
if {[string length $toolfilename] == 0} {
    set toolfilename "emc.tbl"
}

set paramfilename [emc_ini "PARAMETER_FILE" "RS274NGC"]
if {[string length $paramfilename] == 0} {
    set paramfilename "emc.var"
}

# set waiting on EMC to wait until command is received, not finished, with
# timeout of 10 seconds
emc_set_timeout 1.0
emc_set_wait received

# force an update
emc_update

set modifier Alt
set modifierstring "Alt"

set programDirectory [emc_ini PROGRAM_PREFIX DISPLAY]
set helpfilename [emc_ini HELP_FILE DISPLAY]

# set initial value for program status comparison
set oldstatusstring "idle"

# build the left menu bar
set menubar [menu $top.menubar -tearoff 0]

# add the File menu
set filemenu [menu $menubar.file -tearoff 0]
$menubar add cascade -label "File" -menu $filemenu -underline 0
$filemenu add command -label "Open..." -command {fileDialog} -underline 0
bind . <$modifier-o> {fileDialog}
$filemenu add command -label "Edit..." -command {popIn popinProgramEditor} -underline 0
$filemenu add command -label "Reset" -command {emc_task_plan_init} -underline 0
$filemenu add separator
$filemenu add command -label "Exit" -command {after cancel updateStatus ; destroy . ; exit} -accelerator $modifierstring+X -underline 1
bind . <$modifier-x> {after cancel updateStatus ; destroy . ; exit}

# add the View menu
set viewmenu [menu $menubar.view -tearoff 0]
$menubar add cascade -label "View" -menu $viewmenu -underline 0
$viewmenu add command -label "Tools..." -command {popIn popinToolEditor} -underline 0
$viewmenu add command -label "Offsets and Variables..." -command {popIn popinParamEditor} -underline 0
$viewmenu add command -label "Diagnostics..." -command {popIn popinDiagnostics} -underline 0
$viewmenu add command -label "Backplot..." -command {popIn popinPlot} -underline 0

# add the Settings menu
set settingsmenu [menu $menubar.settings -tearoff 0]
$menubar add cascade -label "Settings" -menu $settingsmenu -underline 0
$settingsmenu add command -label "Calibration..." -command {popIn popinCalibration} -underline 0
$settingsmenu add command -label "Logging..." -command {popIn popinLog} -underline 0
$settingsmenu add command -label "Testing..." -command {popIn popinTesting} -underline 0
$settingsmenu add command -label "Debug..." -command {popIn popinDebug} -underline 0
$settingsmenu add command -label "Tuning..." -command {popIn popinTuning} -underline 1
$settingsmenu add command -label "Font..." -command {popIn popinFont} -underline 0
$settingsmenu add command -label "Size..." -command {popIn popinSetSize} -underline 0

# add the Units menu
set unitsmenu [menu $menubar.units -tearoff 0]
$menubar add cascade -label "Units" -menu $unitsmenu -underline 0
$unitsmenu add command -label "auto" -command {emc_linear_unit_conversion auto} -underline 0
$unitsmenu add command -label "inches" -command {emc_linear_unit_conversion inch} -underline 0
$unitsmenu add command -label "mm" -command {emc_linear_unit_conversion mm} -underline 0
$unitsmenu add command -label "cm" -command {emc_linear_unit_conversion cm} -underline 0

# Add a scripts menu that looks for *.tcl files in tcl/scripts subdirectory
set scriptsmenu [menu $menubar.scripts -tearoff 1]
$menubar add cascade -label "Scripts" -menu $scriptsmenu -underline 1
set scriptdir tcl/scripts

if { $windows == 0 } {
    set files [exec /bin/ls $scriptdir]
    foreach file $files {
    	if {[string match *.tcl $file]} {
	        set fname [file rootname $file]
	        # if it's executable, arrange for direct execution
    	    if [file executable $scriptdir/$file] {
	    	$scriptsmenu add command -label $fname -command "exec $scriptdir/$file -- -ini $EMC_INIFILE &"
	        }
	    }
    }
}

# add the help menu
set helpmenu [menu $menubar.help -tearoff 0]
$menubar add cascade -label "Help" -menu $helpmenu -underline 0
$helpmenu add command -label "Help..." -command {popinHelp} -underline 0
$helpmenu add check -label "Balloon help" -variable do_balloons
$helpmenu add separator
$helpmenu add command -label "Info..." -command {catch {exec tcl/sysinfo.tcl -- -ini $EMC_INIFILE}} -underline 0
$helpmenu add separator
$helpmenu add command -label "About..." -command {popupAboutx} -underline 0
$helpmenu add separator
$helpmenu add command -label "TclInfo..." -command {popinInfo} -underline 3


. configure -menu $menubar

# place an estop button up right
set estoplabel [emc_estop]
set stopbutton [button $up.stopbutton -text "ESTOP" \
    -bg red -width 12  -takefocus 0 \
        -command {
            toggleEstop
            emc_wait done
            emc_machine on
        }
    ]
pack $stopbutton -side right -fill x -expand yes

# color and move abortbutton to up frame second from right
set abortbutton [button $up.abort -text "ABORT" -width 8  -takefocus 0 \
    -bg yellow -fg black -activebackground yellow -activeforeground white ]
bind $abortbutton <ButtonPress-1> {emc_abort}
pack $abortbutton -side right -fill x -expand yes

# Add feedhold button in up frame third from right
set feedholdbutton [button $up.feedhold -textvariable feedtext -width 8  -takefocus 0 ]
pack $feedholdbutton -side right -fill x -expand yes
bind $feedholdbutton <ButtonPress-1> {toggleFeedhold}

# initial feedhold conditions
set feedtext "CONTINUE"
set feedoverride [emc_feed_override]
set lastfeedoverride 100
toggleFeedhold

set mdibutton [button $up.mdi -text "MDI" -width 8  -takefocus 0     ]
bind $mdibutton <ButtonPress-1> {showMode mdi}
pack $mdibutton -side right -fill x -expand yes

set autobutton [button $up.auto -text "AUTO" -width 8  -takefocus 0     ]
bind $autobutton <ButtonPress-1> {showMode auto}
pack $autobutton -side right -fill x -expand yes

set manbutton [button $up.manual -text "MANUAL" -width 8  -takefocus 0    ]
bind $manbutton <ButtonPress-1> {showMode manual}
pack $manbutton -side right -fill x -expand yes

set setbutton [button $up.set -text "SETTINGS" -width 8  -takefocus 0     ]
bind $setbutton <ButtonPress-1> {showMode set}
pack $setbutton -side right -fill x -expand yes


# Below are the left window widgets

set toolsetting 0
set tooloffsetsetting 0.0000
set offsetsetting ""
set unitsetting "custom"
set oldunitsetting $unitsetting

set settings [frame $left.settings ]
set settings1 [frame $left.settings1 ]
pack $settings $settings1 -side top -anchor w -fill both
set toollabel [label $settings.toollabel -text "Tool \#:" -anchor w]
set toolsetting [label $settings.toolsetting -textvariable toolsetting -width 3 -anchor w]
set tooloffsetlabel [label $settings.tooloffsetlabel -text "Offset:" -anchor w]
set tooloffsetsetting [label $settings.tooloffsetsetting -textvariable tooloffsetsetting -width 8 -anchor e]

pack $toollabel -side left -padx 1m -fill both
pack $toolsetting -side left -padx 1m -fill both
pack $tooloffsetlabel -side left -padx 1m -fill both
pack $tooloffsetsetting -side left -padx 1m -fill both

bind $tooloffsetlabel <ButtonPress-1> {popupToolOffset}
bind $tooloffsetsetting <ButtonPress-1> {popupToolOffset}
bind $tooloffsetlabel <ButtonPress-3> {popupToolOffset}
bind $tooloffsetsetting <ButtonPress-3> {popupToolOffset}

# jointworld can be joint or world
# set initial conditions for older emc systems using identity between joint/world.

set jointworld "world"
foreach axnum $axiscoordmap {
    set "poslabel${axnum}" [set "worldlabel${axnum}"]
    # create poslabel name list for globals
    lappend posnames poslabel${axnum} posdigit${axnum}
}

# reset these if emc_kinematics_type exists and needs joint definition
if {$isnew != "no"} {
    if { [emc_kinematics_type] != 1 } {
        set jointworld "joint"
        foreach axnum $axiscoordmap {
            set "poslabel${axnum}" [set "jointlabel${axnum}"]
        }
    }
}

set fontfamily Courier
set fontsize 24
set fontstyle bold
set userfont [emc_ini "POSITION_FONT" "DISPLAY"]

if {$userfont != ""} {
    set fontfamily [lindex $userfont 0]
    set fontsize [lindex $userfont 1]
    set fontstyle [lindex $userfont 2]
    set nf $userfont
}

set move [frame $left.move]
set position [frame $move.position]
pack $move -side top
pack $position -side left

# Defines position display widgets and some position related variables

foreach axnum $axiscoordmap {
    set pos${axnum} [frame $position.pos${axnum} -borderwidth 3]
    pack [set pos${axnum}] -side top
    set "pos${axnum}l" [label [set "pos${axnum}"].l -textvariable poslabel${axnum} -width 1 -anchor w]
    set "pos${axnum}d" [label [set "pos${axnum}"].d -textvariable posdigit${axnum} -width 9 -anchor w]
    pack [set "pos${axnum}l"] -side left
    pack [set "pos${axnum}d"] -side right
    bind [set "pos${axnum}l"] <ButtonPress-1> "axisSelectx ${axnum}"
    bind [set "pos${axnum}d"] <ButtonPress-1> "axisSelectx ${axnum}"
    bind [set "pos${axnum}l"] <ButtonPress-3> "axisSelectx ${axnum} ; popupAxisOffsetx ${axnum}"
    bind [set "pos${axnum}d"] <ButtonPress-3> "axisSelectx ${axnum} ; popupAxisOffsetx ${axnum}"
    # add widget names to posnames
    lappend posnames pos${axnum} "pos${axnum}l" "pos${axnum}d" jointlabel${axnum} worldlabel${axnum}
}

setfontx

set realfeedoverride [emc_feed_override]
set feedoverride $realfeedoverride
set oldfeedoverride $feedoverride

# read the max feed override from the ini file
set temp [emc_ini "MAX_FEED_OVERRIDE" "DISPLAY"]
if {[string length $temp] == 0} {
    set temp 1
}
set maxFeedOverride [int [expr $temp * 100 + 0.5]]
set oridescalelength "200"

set oride [frame $left.oride ]
set oridetop [frame $oride.top]
set oridebottom [frame $oride.bottom]
set oridelabel [label $oridetop.label -text "Feed Override:" -width 12]
set oridevalue [label $oridetop.value -textvariable realfeedoverride -width 6]
set oridescale [scale $oridebottom.scale -length $oridescalelength -from 0 -to $maxFeedOverride -variable feedoverride -orient horizontal -showvalue 0 -command {emc_feed_override} -takefocus 0]

pack $oride -side top -anchor w
pack $oridetop -side top
pack $oridebottom -side top
pack $oridelabel -side left
pack $oridevalue -side left -padx 2m ; # don't bump against label
pack $oridescale -side top

bind $oridelabel <ButtonPress-1> {popupOride}
bind $oridelabel <ButtonPress-3> {popupOride}
bind $oridevalue <ButtonPress-1> {popupOride}
bind $oridevalue <ButtonPress-3> {popupOride}


# Below are the settings window widgets packed by process tcl/lib/extkemc/showMode.tcl
# For now you will have to change the values there for different sized windows.

set limoridebutton [button $setframe.button -text "override limits" -command {toggleLimitOverride} -takefocus 0]
set limoridebuttonbg [$limoridebutton cget -background]
set limoridebuttonabg [$limoridebutton cget -activebackground]
bind $limoridebutton <ButtonPress-1> {emc_override_limit}

set lubebutton [button $setframe.lube -textvariable lubelabel -relief raised -width 15 \
    -command {toggleLube}]

set radiorel [radiobutton $setframe.rel -text "relative" -variable coords -value relative -command {} -takefocus 0 -anchor w]
set radioabs [radiobutton $setframe.abs -text "machine" -variable coords -value machine -command {} -takefocus 0 -anchor w]

set radioact [radiobutton $setframe.act -text "actual" -variable actcmd -value actual -command {} -takefocus 0 -anchor w]
set radiocmd [radiobutton $setframe.cmd -text "commanded" -variable actcmd -value commanded -command {} -takefocus 0 -anchor w]

set radiojoint [radiobutton $setframe.joint -text "joint" -variable jointworld -value joint -command {} -takefocus 0 -anchor w]
set radioworld [radiobutton $setframe.world -text "world" -variable jointworld -value world -command {} -takefocus 0 -anchor w]

set homebutton [button $setframe.home -text "home" -takefocus 0]
bind $homebutton <ButtonPress-1> {emc_home [ lindex $axiscoordmap $activeAxis]}

#set offsetbutton [button $setframe.of -text "Popup Offsets" -takefocus o -command popupOffsets ]
set offsetlabel [label $setframe.offsetlabel -text "Work Offsets: " -anchor e]
set workoffsets [label $setframe.offsetsetting -textvariable offsetsetting -width 40 -anchor w]

set offsetsetting ""

# set up help for these
set_balloon $offsetlabel [offsethelp]
set_balloon $workoffsets [offsethelp]

set unitlabel [label $setframe.unitlabel -textvariable unitsetting -width 12 -borderwidth 1 -relief sunken]
set stoplabel [label $setframe.stoplabel -textvariable estoplabel -width 12 -borderwidth 1 -relief sunken]

set size1 [button $setframe.size1 -text "Size 1" -command {exSetSize 1}]
set size2 [button $setframe.size2 -text "Size 2" -command {exSetSize 2}]
set size3 [button $setframe.size3 -text "Size 3" -command {exSetSize 3}]
set size4 [button $setframe.size4 -text "Size 4" -command {exSetSize 4}]

grid $limoridebutton -column 0 -row 0 -sticky nsew
grid $lubebutton -column 0 -row 1 -sticky nsew
grid $radiorel -column 1 -row 0 -sticky nsew
grid $radioabs -column 1 -row 1 -sticky nsew
grid $radioact -column 2 -row 0 -sticky nsew
grid $radiocmd -column 2 -row 1 -sticky nsew
grid $radiojoint -column 3 -row 0 -sticky nsew
grid $radioworld -column 3 -row 1 -sticky nsew
grid $homebutton -column 0 -row 3 -sticky nsew
grid $offsetlabel -column 1 -row 3 -sticky nsew
grid $workoffsets -column 2 -row 3 -sticky nsew
grid $workoffsets -columnspan 3
grid $unitlabel -column  4 -row 0 -sticky nsew
grid $stoplabel -column 4 -row 1 -sticky nsew
grid $size1 -column 0 -row 4 -sticky nsew
grid $size2 -column 1 -row 4 -sticky nsew
grid $size3 -column 2 -row 4 -sticky nsew
grid $size4 -column 3 -row 4 -sticky nsew
grid columnconfigure $setframe 0 -weight 1
grid columnconfigure $setframe 1 -weight 1
grid columnconfigure $setframe 2 -weight 1
grid columnconfigure $setframe 3 -weight 1
grid rowconfigure $setframe 0 -weight 1
grid rowconfigure $setframe 1 -weight 1
grid rowconfigure $setframe 2 -weight 1
grid rowconfigure $setframe 3 -weight 1
grid rowconfigure $setframe 4 -weight 1


# Below are manual mode window widgets and settings
axisSelectx [ lindex $axiscoordmap 0 ]

# read the default jog speed
set temp [emc_ini "DEFAULT_VELOCITY" "TRAJ"]
if {[string length $temp] == 0} {
    set temp 1
}
set jogSpeed [int [expr $temp * 60 + 0.5]]
set defaultJogSpeed $jogSpeed

# read the max jog speed
set temp [emc_ini "MAX_VELOCITY" "TRAJ"]
if {[string length $temp] == 0} {
    set temp 1
}
set maxJogSpeed [int [expr $temp * 60 + 0.5]]

set feeddefault [button $manframe.default -text "default" -width 8 -command {set jogSpeed $defaultJogSpeed}]
set feedlabel [label $manframe.label -text "Axis Speed:" -width 12]
set feedvalue [label $manframe.value -textvariable jogSpeed -width 6]
set feedrapid [button $manframe.rapid -text "rapid" -width 8 -command {set jogSpeed $maxJogSpeed}]
set feedscale [scale $manframe.scale -length 270 -from 0 -to $maxJogSpeed -variable jogSpeed -orient horizontal -showvalue 0 -takefocus 0]
bind $feedlabel <ButtonPress-1> {popupJogSpeed}
bind $feedlabel <ButtonPress-3> {popupJogSpeed}
bind $feedvalue <ButtonPress-1> {popupJogSpeed}
bind $feedvalue <ButtonPress-3> {popupJogSpeed}

# spindle control widgets
set sdoing [label $manframe.label1 -textvariable spindlelabel -relief flat]
set sforbutton [button $manframe.forward -text "Forward" -command {emc_spindle forward}]
set srevbutton [button $manframe.reverse -text "Reverse" -command {emc_spindle reverse}]
set sstopbutton [button $manframe.stop -text "Spindle off" -command {emc_spindle off}]

set decrbutton [button $manframe.decr -text "Slower" -takefocus 0]
bind $decrbutton <ButtonPress-1> {emc_spindle decrease}
bind $decrbutton <ButtonRelease-1> {emc_spindle constant}

set incrbutton [button $manframe.incr -text "Faster" -takefocus 0]
bind $incrbutton <ButtonPress-1> {emc_spindle increase}
bind $incrbutton <ButtonRelease-1> {emc_spindle constant}

set brakebutton [button $manframe.brake -textvariable brakelabel -relief raised \
    -command {toggleBrake}]

set mistbutton [button $manframe.mist -textvariable mistlabel -relief raised -width 15 \
    -command {toggleMist}
    ]

set floodbutton [button $manframe.flood -textvariable floodlabel -relief raised -width 15 \
    -command {toggleFlood}
    ]

set jogType continuous
set jogIncrement [emc_ini  "DEFAULT_JOG_INCREMENT"  "DISPLAY"]

radiobutton $manframe.r1 -text "0.0001" -variable jogIncrement -value "0.0001" -anchor w \
        -anchor w -padx 4 -command {set jogtype increment} -state disabled
radiobutton $manframe.r2 -text "0.0010" -variable jogIncrement -value "0.0010" -anchor w \
        -anchor w -padx 4 -command {set jogtype increment} -state disabled
radiobutton $manframe.r3 -text "0.0100" -variable jogIncrement -value "0.0100" -anchor w \
        -anchor w -padx 4 -command {set jogtype increment} -state disabled
radiobutton $manframe.r4 -text "0.1000" -variable jogIncrement -value "0.1000" -anchor w \
        -anchor w -padx 4 -command {set jogtype increment} -state disabled
radiobutton $manframe.r5 -text "1.0000" -variable jogIncrement -value "1.0000" -anchor w \
        -anchor w -padx 4 -command {set jogtype increment} -state disabled

set jognegbutton [button $manframe.neg -text "Jog -" -takefocus 0]
set jogposbutton [button $manframe.pos -text "Jog +" -takefocus 0]
set jogtypebutton [button $manframe.type -textvariable  jogType -takefocus 0 -width 12 -command {toggleJogTypex}]

bind $jognegbutton <ButtonPress-1> {jogNeg $activeAxis}
bind $jognegbutton <ButtonRelease-1> {jogStop}
bind $jogposbutton <ButtonPress-1> {jogPos $activeAxis}
bind $jogposbutton <ButtonRelease-1> {jogStop}

grid $feeddefault -column 0 -row 0 -sticky nsew
grid $feedlabel -column 1 -row 0 -sticky nsew
grid $feedvalue -column 2 -row 0 -sticky nsew
grid $feedrapid -column 3 -row 0 -sticky nsew
grid $feedscale -column 0 -row 1 -sticky nsew
grid configure $feedscale -columnspan 4
grid $mistbutton -column 0 -row 2 -sticky nsew
grid $sdoing -column 1 -row 2 -sticky nsew
grid configure $sdoing -columnspan 2
grid $floodbutton -column 3 -row 2 -sticky nsew
grid $sforbutton -column 3 -row 3 -sticky nsew
grid $sstopbutton -column 1 -row 3 -sticky nsew
grid configure $sstopbutton -columnspan 2
grid $srevbutton -column 0 -row 3 -sticky nsew
grid $decrbutton -column 0 -row 4 -sticky nsew
grid $brakebutton -column 1 -row 4 -sticky nsew
grid configure $brakebutton -columnspan 2
grid $incrbutton -column 3 -row 4 -sticky nsew
grid $jogposbutton -column 4 -row 0 -sticky nsew
grid configure $jogposbutton -rowspan 2
grid $jogtypebutton -column 4 -row 2 -sticky nsew
grid $jognegbutton -column 4 -row 3 -sticky nsew
grid configure $jognegbutton -rowspan 2
grid $manframe.r5 -column 5 -row 0 -sticky nsew
grid $manframe.r4 -column 5 -row 1 -sticky nsew
grid $manframe.r3 -column 5 -row 2 -sticky nsew
grid $manframe.r2 -column 5 -row 3 -sticky nsew
grid $manframe.r1 -column 5 -row 4 -sticky nsew
grid columnconfigure $manframe 0 -weight 1
grid columnconfigure $manframe 1 -weight 1
grid columnconfigure $manframe 2 -weight 1
grid columnconfigure $manframe 3 -weight 1
grid columnconfigure $manframe 4 -weight 1
grid columnconfigure $manframe 5 -weight 1
grid rowconfigure $manframe 0 -weight 1
grid rowconfigure $manframe 1 -weight 1
grid rowconfigure $manframe 2 -weight 1
grid rowconfigure $manframe 3 -weight 1
grid rowconfigure $manframe 4 -weight 1


# Below are the mdi mode window widgets

set mditext ""
set mdilabel [label $mdiframe.label -text "MDI:" -width 4 -anchor w]
set mdientry [entry $mdiframe.entry -textvariable mditext -width 73 -takefocus 1]
bind $mdientry <Return> {$mdientry select range 0 end ; sendMdi $mditext}

set programcodestring ""
set programcodes [label $mdiframe.programcodes -textvariable programcodestring]

grid $mdilabel
grid $mdientry
grid $programcodes

# Below are the auto mode window widgets

set programnamestring "none"
set programstatusstring "none"
set programin 0
set activeLine 0

# programstatus is "Program: <name> Status: idle"
set programstatus [frame $autoframe.programstatus]

# programstatusname is "Program: <name>" half of programstatus
set programstatusname [frame $programstatus.name]
set programstatusnamelabel [label $programstatusname.label -text "Program: "]
set programstatusnamevalue [label $programstatusname.value -textvariable programnamestring -width 50 -anchor e]

# programstatusstatus is "Status: idle" half of programstatus
set programstatusstatus [frame $programstatus.status]
set programstatusstatuslabel [label $programstatusstatus.label -text "  -  Status: "]
set programstatusstatusvalue [label $programstatusstatus.value -textvariable programstatusstring -anchor w]

pack $programstatus -side top -anchor w
pack $programstatusname $programstatusstatus -side left
pack $programstatusnamelabel $programstatusnamevalue -side left
pack $programstatusstatuslabel $programstatusstatusvalue -side left

# programframe is the frame for the button widgets for run, pause, etc.
set programframe [frame $autoframe.programframe]
set programopenbutton [button $programframe.open -text "Open..." -command {fileDialog} -takefocus 0]
set programrunbutton [button $programframe.run -text "Run" -command {emc_run $runMark ; set runMark 0} -takefocus 0]
set programpausebutton [button $programframe.pause -text "Pause" -command {emc_pause} -takefocus 0]
set programresumebutton [button $programframe.resume -text "Resume" -command {emc_resume} -takefocus 0]
set programstepbutton [button $programframe.step -text "Step" -command {emc_step} -takefocus 0]
set programverifybutton [button $programframe.verify -text "Verify" -command {emc_run -1} -takefocus 0]

pack $programframe -side top -anchor w -fill both -expand true
pack $programopenbutton $programrunbutton $programpausebutton $programresumebutton $programstepbutton $programverifybutton -side left -fill both -expand true

set textwidth "80"
set textheight "6"

# programfileframe is the frame for the program text widget showing the file
set programfileframe [frame $autoframe.programfileframe]
set programfiletext [text $programfileframe.text -height $textheight -width $textwidth -relief sunken -state disabled]
# FIXME-- hard-coded resources
$programfiletext tag configure highlight -background white -foreground black

pack $programfileframe -side top -anchor w
pack $programfiletext -side top

exSetSize 1

setKeyBindingsx
set activeAxis 0
set syncingFeedOverride 0

# force explicit updates, so calls to emc_estop, for example, don't
# always cause an NML read; they just return last latched status
emc_update none

# Set up the display for manual mode, which may be reset
# immediately in updateStatus if it's not correct.
# This lets us check if there's a change and reset bindings, etc.
# on transitions instead of every cycle.
set modelabel "MANUAL"

# Set manual bindings
showMode manual
showMode set

# Record that we're in manual display mode
set modeInDisplay "manual"
set oldmode "manual"
set jogType incremental
toggleJogTypex

set lastjointworld $jointworld
set lastactcmd $actcmd
set lastcoords $coords
set emc_teleop_enable_command_given 0

# Need to set a variable here to use tkemcex for older EMCs
# for units and for teleop and set isnew to no or ?.
# this is also read by the units setting in updateStatus2

if {$isnew == "no"} {
    $menubar entryconfigure 3 -state disabled ; # units menu
    $radiojoint configure -state disabled ; #joint mode
    $settingsmenu entryconfigure 3 -state disabled ; #debug
}

updateStatus2
