#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"
#this is only necessary if starting from outside tkEMC, in which case we don't want emcsh anyway.

#tks_client.tcl --
# this is the client-side implementation of TkServ
# i.e. this runs on the computer where TkEMC - and the user - is.
#"server side" is where EMC is running.
#
# I believe that all globals and procs specific to tksc are preceeded by "tksc_" to avoid conflicts... if you modify this please ensure your changes don't interfere with anything.
#
#As of Aug 20 2003, there are several things that need changing... search for FIXME.
#
# Statements that are commented out are probably for debugging.
#
#--------------------------------------------------------------------------------------------
# License: GPL (http://www.fsf.org/licenses/gpl.html)
#
# Copyright (C)2003, Mark Pictor
#--------------------------------------------------------------------------------------------

#portions of this code are taken from the demos that come with tcl/tk (and modified)
#image2.tcl, search.tcl
#Per the licenses of those files, since I modified what I used, I can place this under a different license (i.e. the GPL) 


# tksc_textLoadHelp --
# This procedure below loads the file help.txt into a text widget, discarding
# the previous contents of the widget. Tags for the old widget are
# not affected, however.
#---modified from a proc in search.tcl, a demo that comes with TK
#
# Arguments:
# w -		The window into which to load the file.  Must be a
#		text widget.

proc tksc_textLoadHelp {h} {
    set f [open ./tks_help.txt]
    $h delete 1.0 end
    while {![eof $f]} {
	$h insert end [read $f 10000]
    }
    close $f
    $h insert end "\n\n\n";	#blank lines at end - workaround scroll bug that appears when using wheel
}

# tksc_fileDclick --
# Given the mouse position, extracts the directory 
# entry (a file) under the mouse and sends that file path to TkEMC.
#
# Arguments:
# w - 			Window with listbox in it
# x, y-			Mouse position within the listbox.

proc tksc_fileDclick {w x y} {
    global tksc_dirName
    set file [file join $tksc_dirName [$w.f.list get @$x,$y]]
    tksc_path_to_tkemc $file
    destroy $w;	#close file selection window.
}

# tksc_path_to_tkemc --
# tell TkEMC the file name to use.
# called on either remote *or* local browse.
#
# args -
#file - the file name
#w - window handle
#
proc tksc_path_to_tkemc {file} {
	tk_messageBox -icon info -message "procedure tksc_path_to_tkemc is incomplete,\ntkEMC should load this file:\n$file" -type ok
	#FIXME - communicate with tkemc.
	#maybe also display a confirmation like this:
	#tk_messageBox -icon info -message "EMC loaded this file:\n$file" -type ok
}

# tksc_dirDclick --
# Given the mouse position, extracts the directory 
# entry (a dir) under the mouse and calls tksc_updateDir
#
# Arguments:
# w - 			Window with listbox in it
# x, y-			Mouse position within the listbox.

proc tksc_dirDclick {w x y} {
	global tksc_dirName
	set dir [file join $tksc_dirName [$w.dir.list get @$x,$y]]
	#tk_messageBox -icon info -message "tksc_dirDclick: file $file" -type ok -parent $w; ################	debug message	################
	tksc_updateDir $w $dir
}

# tksc_updateDir --
# sends a CD command to TkServ
# updates dirlist, filelist (and the GUI)
#
# args
# w - window with listboxes in it
# dir - where we should CD to.

proc tksc_updateDir {w dir} {
	global tksc_dirName tksc_SERVER tksc_TPORT
	set dirlist ""; set filelist ""
	if {[catch {set sock_id [socket $tksc_SERVER $tksc_TPORT]} err]} {
	    	tk_messageBox -icon error -message "Can't connect to $tksc_SERVER\n$err" -type ok
		return
	} else {
		fconfigure $sock_id -buffering line -blocking 1 -encoding utf-8
		#tk_messageBox -icon info -message "dir $dir" -type ok
		puts $sock_id "cd $dir"
		after 200;		#if we go before data recieved, all kinds o' problems. avoid them!!!  waiting is easier than a workaround.
		gets $sock_id tksc_dirName
		#tk_messageBox -icon info -message "pwd $tksc_dirName" -type ok
		puts $sock_id "dirlist"
		after 200
		gets $sock_id dirlist
		#tk_messageBox -icon info -message "dirs $dirlist" -type ok
		puts $sock_id "filelist"
		after 200
		gets $sock_id filelist
		#tk_messageBox -icon info -message "files $filelist" -type ok
	}
	#now update UI.
	$w.f.list delete 0 end
	foreach i [lsort $filelist] {
		$w.f.list insert end [file tail $i]
    	}
	$w.dir.list delete 0 end
	$w.dir.list insert end "..";	#add up one level
	foreach i [lsort $dirlist] {
		$w.dir.list insert end [file tail $i]
    	}
}

# tksc_helpwin --
# This procedure creates a new window with a scrolling text widget 
# and reads tks_help.txt into that widget.
proc tksc_helpwin {} {
	set h .tks_client_help
	catch {destroy $h}
	toplevel $h
	wm title $h "TkServ Client: Help"
	wm iconname $h "tks_client_help" 
	wm geometry $h -50+50

	button $h.close -text "Close Help" -command "destroy $h"
	pack $h.close -side bottom
	
	text $h.text -yscrollcommand "$h.scroll set" -setgrid true -wrap word;	#**how to lock text?
	scrollbar $h.scroll -command "$h.text yview"
	pack $h.scroll -side right -fill y
	pack $h.text -expand yes -fill both

	tksc_textLoadHelp $h.text; 		#load text from the file help.txt
}

# tksc_remote --
# gets dirlist from remote computer, displays it, then we send the result back to that computer.
# tksc_remote is actually only the GUI stuff for this...
proc tksc_remote {} {
	global tksc_dirName
	#our window
 	set w .tksc_remote
 	catch {destroy $w}
 	toplevel $w
 	wm title $w "TkServ Client"
 	wm iconname $w "tks_client"

 	label $w.msg -text "Navigate directories by typing in box then pressing enter or clicking Use, or by double-clicking in dir list.  Select a file by double-clicking." -wraplength 400 -justify left
 	pack $w.msg -side top
	#buttons at bottom	
	frame $w.buttons
	pack $w.buttons -side bottom -fill x -pady 2m
	button $w.buttons.help -text "Help" -command {tksc_helpwin}
	button $w.buttons.exit -text "Close" -command "destroy $w"
	pack $w.buttons.help $w.buttons.exit -side left -expand 1

	#dir
	#see glob -types {} in tcl help... add dir list and change this section to "path entry"
	labelframe $w.dir -text "Directory:" -padx 2m -pady 2m
	frame $w.dir.t
	entry $w.dir.t.e -width 30 -textvariable tksc_dirName
	button $w.dir.t.b -text "Use" -command "tksc_updateDir $w {$tksc_dirName}"
	bind $w.dir.t.e <Return> "tksc_updateDir $w {$tksc_dirName}"
	listbox $w.dir.list -height 10 -width 30 -yscrollcommand "$w.dir.scroll set"
	scrollbar $w.dir.scroll -command "$w.dir.list yview"
	bind $w.dir.list <Double-1> "tksc_dirDclick $w %x %y"
	pack $w.dir.t.e -side top -fill y -padx 2m -pady {2m 1m} -expand true
	pack $w.dir.t.b -side bottom -pady {0 2m}
	pack $w.dir.t -side top
	pack $w.dir.list -side left -fill both -expand 1
	pack $w.dir.scroll -side left -fill y -expand 1
	pack $w.dir -side left -padx {2m 0} -pady 2m
	#file
	labelframe $w.f -text "File:" -padx 2m -pady 2m
	listbox $w.f.list -yscrollcommand "$w.f.scroll set"
	scrollbar $w.f.scroll -command "$w.f.list yview"
	pack $w.f.list -side left -fill both -expand 1
	pack $w.f.scroll -side left -fill y -expand 1
	bind $w.f.list <Double-1> "tksc_fileDclick $w %x %y"
	pack $w.f -side right -fill both -padx 2m -pady 2m
	tksc_updateDir $w ".";	#where is TkServ?  this . becomes part of 'cd .' so it doesn't change anything...
}

# tksc_transfer --
# transfer the file using netcat, return the full path/file name so EMC can find it.
# only used with local browse.
#
# args - 
# $file - local file to xfer
proc tksc_transfer {file} {
	global tksc_SERVER tksc_FPORT
#	catch {
	 	exec "./nc" $tksc_SERVER $tksc_FPORT "<$file"
#	 	}
}
# tksc_local --
# user chooses local file, then we need to send that file, along with the md5sum, to the remote computer.
proc tksc_local {} {
	global tksc_lsock_id; 	#UGGGH! what a *kludge* - tksc_lsock_id used in tksc_init_xfer (called from here), then here. already returning another value.
	set file [tk_getOpenFile -parent .];	#FIXME - need to handle the cancel button!!!
	if {$file != ""} {;		#handle cancel button by skipping everything...
		set file [file normalize $file]	
		#re-format string
		tk_messageBox -icon info -message "tksc_local:\nfile $file" -type ok
		
	    	set md5 [string range [exec "./md5sum" $file] 0 31];	#we just want the md5, not the file name... md5 is 32 chars
	    	#tk_messageBox -icon info -message "tksc_local:\nfile ---$file---\nmd5 ---$md5---" -type ok; ################	debug message	################
	    	if {[set r [tksc_init_xfer $md5]]} {
		    	# error!!!
		    	return
	    	} else {
			#do the transfer.

			#FIXME: transfer does not work, somehow.
			#FIXME: in transfer func, we should to wait and retry if there's a failure. (implemented but not yet tested!)

			set again 10
		    	while {$again} {;	#loop until file transferred correctly or until there are 10 failures.
		    		set again [expr $again - 1]
				puts $tksc_lsock_id "GO"
		    		tksc_transfer $file
			    	gets $tksc_lsock_id md5fname;	#either "bad md5" or the path and filename
			    	if {$md5fname != "bad md5"} {
				    	tksc_path_to_tkemc $md5fname;	#the file name is now its md5 - we tell EMC about the file.
				    	set again 10;		# 
				    	break
	    			}
	    		}
	    		if {$again == 0} {
		    		tk_messageBox -icon info -message "Transfer error" -type ok
		    		}
	    	};		#else
    	};		#if (open file box cancel)
}

# tksc_init_xfer --
# tell server that we're ready to transfer a file.  Send the md5 for the file also.
#
# mating procs in tkserv:
# incoming - handles all comms except the file transfer itself.
proc tksc_init_xfer {md5} {
	global tksc_SERVER tksc_TPORT tksc_FPORT tksc_lsock_id
	#open socket, send md5, get md5 back to verify/as confirmation to xfer.
	#need to make sure all is OK, return error if not.
	if {[catch {set tksc_lsock_id [socket $tksc_SERVER $tksc_TPORT]} err]} {
	    	tk_messageBox -icon error -message "Can't connect to $tksc_SERVER\n$err" -type ok
		return -1;		# error
	} else {
		fconfigure $tksc_lsock_id -buffering line -blocking 1 -encoding utf-8

		set again 10;			# we don't want to loop forever!
		while {$again} {;	#loop until we get a good response.
			puts $tksc_lsock_id "Begin Transfer, md5=$md5"
			after 50;	#wait 50 ms
			gets $tksc_lsock_id resp
			#tk_messageBox -icon info -message "md5:$md5-\ngot:$resp-" -type ok; ################	debug message	################
			if {[string equal $resp $md5]} {;	#good response, ready to xfer
				break
			}
			set again {again -1}
		}
		if {$again == 0} {
			tk_messageBox -icon error -message "Too many transmission errors!" -type ok
			#if you get this message, are you sure you're using the right port and not connecting to some other service?
			return -1;			# failed, count went to zero.
		}
		return 0;	#0 for success
	}

}

#----------------------------------------------------
#primary window
proc tksc_pri_window {} {
	if {[info exists tkemc]} {
		set w .tksc
		catch {destroy $w}
		toplevel $w
		wm title $w "TkServ Client"
		wm iconname $w "tks_client" 
		wm geometry $w +50+50
	} else {;			#not running from TkEMC
		set w ""
		wm title . "TkServ Client"
		wm iconname . "tks_client" 
		wm geometry . +50+50
	}
	button  $w.help -text "Help!!!" -command "tksc_helpwin"
	button  $w.local -text "Local Browse" -command "tksc_local";# -state disabled;	#faulty
	button  $w.remote -text "Remote Browse" -command "tksc_remote"
	pack  $w.help -side left -padx {2m 0} -pady 2m
	pack $w.local -side left -padx {2m 0} -pady 2m
	pack $w.remote -side left -padx 2m -pady 2m
}

# init --
# set global variables.
# FIXME: *should* load them from a file, but for now they're defined here.
proc tksc_init {} {
	global tksc_SERVER tksc_TPORT tksc_FPORT
	if {[info exists tkemc]} {
		#load from emc.ini
		#set var [emc_ini "item" "section"]
		#these keys/values DO NOT EXIST YET in emc.ini... it may be necessary to change them or use a different file
		set tksc_SERVER [emc_ini "SERVER" "TKS_CLIENT"]; 	#server's address
		set tksc_TPORT [emc_ini "TPORT" "TKS_CLIENT"];		#port for tcl script to use.
		set tksc_FPORT [emc_ini "FPORT" "TKS_CLIENT"];		#port for transferring files.
	} else {
		set tksc_SERVER localhost
		#set tksc_SERVER 192.168.0.1
		set tksc_TPORT 12345
		set tksc_FPORT 23456
	}
}

#-----------------------------------------------------------------------------------------------------------------------
#entry point
#-----------------------------------------------------------------------------------------------------------------------
tksc_init
tksc_pri_window