#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

#tkserv.tcl --
# this is the server-side implementation of TkServ
# i.e. this runs on the computer where EMC - and the machine - is.
#"client side" is where TkEMC is running.
#
#--------------------------------------------------------------------------------------------
# License: GPL (http://www.fsf.org/licenses/gpl.html)
#
# Copyright (C)2003, Mark Pictor
#--------------------------------------------------------------------------------------------

# listen --
# listen for a connection.
#
# Args
# port - the port to listen on
proc listen {port} {
	puts "listening on port $port"
	set s [socket -server heard $port]
	vwait forever
}

# heard --
# when a connection opens
# 
# Args
# sock - socket
# addr - address
# port - local port
proc heard {sock addr port} {
    	# Ensure that each "puts" by the server
    	# results in a network transmission
	puts "heard addr $addr port $port"
    	fconfigure $sock -buffering line -encoding utf-8

    	# Set up a callback for when the client sends data
    	fileevent $sock readable [list incoming $sock]
}

# incoming --
# when data is coming in on the connection

# Args
# sock - socket
proc incoming {sock} {
	#mating procs in tks_client:
	#tksc_init_xfer
	#
	#
	global MD5
	if {[eof $sock] || [catch {gets $sock line}]} {
		close $sock
		puts "incoming: closed socket with no read - either an error or a disconnect"
	} else {
		#gets still read $line above, even though it didn't cause an error...
		puts "incoming: $line"
		if {$line == "GO"} {
			# they are ready to do a transfer.
			recieve $sock $MD5
		} elseif {[string range $line 0 19] == "Begin Transfer, md5="} {
			# save, then send back, md5 for verification
			set MD5 [string range $line 20 end]
			puts $sock $MD5
		} elseif {[string range $line 0 2] == "cd "} {
			#change directory
			set dir [file join [string range $line 3 end]];	#separate the path part, and correct directory separation symbol.			
			catch {exec [cd $dir]};	#don't need to do anything with the error
						#FIXME: this (exec cd) ALWAYS generates an error... why?!
			puts $sock [pwd]
		} elseif {$line == "dirlist"} {
			#send a list of directories in the pwd.
			set dl [glob -nocomplain -types {d r} *]
			puts $sock $dl
		} elseif {$line == "filelist"} {
			#send a list of files in the pwd.
			puts $sock [glob -nocomplain -types {f r} *]
		} else {;				#error... print that, otherwise ignore
			if {$line != ""} {;	#don't bother with empty string.
				puts "Error: procedure incoming - bad data was -->$line<--\n( between --> and <-- )"
			}
		}
	}
}

# recieve --
# recieve file, check against md5 $md5... repeat transfer until md5 checks.
# FIXME: add a count to halt it if it repeatedly fails?
#
#args
#sock - socket we're talking on
#md5 - md5 sent by of the file.
proc recieve {sock md5} {
	global tks_incoming tks_fport
	set new_md5 0
	puts "recieve"
	set fname [file join $tks_incoming $md5];		#tks_incoming is the path to the folder for incoming files.
	catch {exec nc "-l -p $tks_fport -w 30" > $fname}
	catch {set new_md5 [exec md5sum \"$fname\"]}
	if {$new_md5 != $md5} {
		puts "error, md5 mismatch: orig $md5\t\tnew $new_md5"
		#file delete $fname
		puts $sock "bad md5"
		return
	}
	puts $sock [file normalize [file join $tks_incoming $new_md5]] #confirmation... we return the full path (normalize) and (join) file name (which is also the md5)
}

# init --
# initialization stuff, when tkserv starts up... only needs to be done once.
#FIXME: this stuff should be read from a config file.
proc init {} {
	puts "INIT"
	global MD5 tks_incoming tks_lport tks_fport
	set MD5 0
	set tks_incoming "./test"
	set tks_lport 12345
	set tks_fport 23456
	puts "incoming dir: $tks_incoming\tlisten port: $tks_lport\ttransfer port: $tks_fport"
	
}

init
listen $tks_lport